/*  ===========================================================================
 *   Copyright (c) Texas Instruments Inc 2006
 *
 *   Use of this software is controlled by the terms and conditions found in
 *   the license agreement under which this software has been supplied.
 *   ==========================================================================
 */


/** ===========================================================================
 * @desc TCP2 channel density test.
 *
 * ============================================================================
 */
/* ============================================================================
 *  Revision History
 *  ===============
 *  11-July-2006   Brighton     File Created.
*   11-May-2008   jane          File Modified for Curie.
 * ============================================================================
 */



#include <stdio.h>
#include <c6x.h>
#include <csl_tcp2.h>
#include <csl_tcp2Aux.h>
#include <csl_cache.h>
#include "val_util.h"
#include "tcp2_edma.h"
#include "TCP2_Channel_Density.h"
#include "VCP2_TCP2_EDMA.h"

#define IS_3GPP 	1
#define DETAILED_TIME 	0

/* Global variable declarations */

#pragma DATA_SECTION(TurboOriginalBits, "TCP2Data")
#pragma DATA_SECTION(TurboEncodedBits, "TCP2Data")
#pragma DATA_SECTION(TurboModulatedData, "TCP2Data")
#pragma DATA_SECTION(TurboCorruptedData, "TCP2Data")
#pragma DATA_SECTION(TurboScaledData, "TCP2Data")
#pragma DATA_SECTION(TurboRate5Data, "TCP2Data")
#pragma DATA_SECTION(TurboInterleaverTable, "TCP2Data")
#pragma DATA_SECTION(TurboDeinterTable, "TCP2Data")
#pragma DATA_SECTION(TurboHD, "TCP2Data")
//Align to L2 cache line size for cache operation
#pragma DATA_ALIGN(TurboOriginalBits, 128)
#pragma DATA_ALIGN(TurboEncodedBits, 128)
#pragma DATA_ALIGN(TurboModulatedData, 128)
#pragma DATA_ALIGN(TurboCorruptedData, 128)
#pragma DATA_ALIGN(TurboScaledData, 128)
#pragma DATA_ALIGN(TurboRate5Data, 128)
#pragma DATA_ALIGN(TurboInterleaverTable,128)
#pragma DATA_ALIGN(TurboDeinterTable, 128)
#pragma DATA_ALIGN(TurboHD, 128)
//All array length add 8 to support speculative optimization
//Original bits
Uint8 TurboOriginalBits[(MAX_TCP2_FRAME_LEN/8)+8];
//Encoded bits
Uint32 TurboEncodedBits[(MAX_TCP2_FRAME_LEN*5+MAX_TCP2_TAIL_LEN)/32+8];
//modulated user data
Uint8 TurboModulatedData[(MAX_TCP2_FRAME_LEN*5+MAX_TCP2_TAIL_LEN)+8];
//User data corrupted by noise
Uint8 TurboCorruptedData[(MAX_TCP2_FRAME_LEN*5+MAX_TCP2_TAIL_LEN)+8];
//User data after scale(multiply scale factor), the array size is double
Uint16 TurboScaledData[(MAX_TCP2_FRAME_LEN*5+MAX_TCP2_TAIL_LEN)+8];
//The input data for TCP2, it is 1/5 rate
Uint32 TurboRate5Data[(MAX_TCP2_FRAME_LEN+MAX_TCP2_TAIL_LEN)+8];
//Interleave table
Uint16 TurboInterleaverTable[(MAX_TCP2_FRAME_LEN+MAX_TCP2_TAIL_LEN)+8];
//Deinterleave table for processing interleaved parities
Uint16 TurboDeinterTable[(MAX_TCP2_FRAME_LEN+MAX_TCP2_TAIL_LEN)+16];
//Hard decision of TCP2
Uint8 TurboHD[TCP2_CHANNELS_PER_TIME][((MAX_TCP2_FRAME_LEN+7)/8+7)/8*8];
Uint32 hTcpOutParms[4]={0,0,0};

//Align to 16 bytes for EDMA transfer
#pragma DATA_ALIGN(TcpConfigIc, 16)
TCP2_ConfigIc    TcpConfigIc;
Uint32 Typical3GPPFrame[]= {640, 2576, 2896, 3856, 2730, 2718, 4062, 5076, 4902, 4826, 714, 640, 1296, 2896, 3856, 640};
Uint32 Typical3GPP2Frame[]= {640, 3, 570	,4,1146	,4,2298	,4,4602	,4,9210	,4,20730	,2,	
							640, 3,378	,4,762	,4,1530	,4,3066	,4,6138	,3,12282	,3,
							640, 3, 640, 3};

CSL_Edma3CmdIntr EdmaregionIntr;

void TCP2_Channel_Density_Test()
{
	Uint32 uiTotalErrorBits=0;
	TCP2_BaseParams TCP2Params;
	Uint32 i, j;
	Uint32 uiFrameLength=2576;  
	Uint32 uiIteratNum= 8;
	Uint32 uiRate= 3;
	Uint32 uiMaxStarEn= 1;
	Uint32 uiRrologRedEn= 1;
	Uint32 uiPrologSize=12;
	Uint32 uiSigma=64; 	//Sigma is unisgned in Q7 format

//        TCP_setInterEndian(CSL_TCP2_TCPEND_ENDIAN_I0_NATIVE16);
        for (i=0;i<16;i++) 
        	TCP2Params.extrScaling[i] = 0;

//	for(uiMaxStarEn=0; uiMaxStarEn<=1; uiMaxStarEn++)
//	for(uiIteratNum=1; uiIteratNum<=32; uiIteratNum+=1)
//	for(uiRate=2; uiRate<=5; uiRate++)
//	for(uiFrameLength=MAX_TCP2_FRAME_LEN; uiFrameLength>320; uiFrameLength-=256)
//	for(uiFrameLength=5114; uiFrameLength>320; uiFrameLength-=64)
//	for(uiSigma=64; uiSigma<(128*1.2); uiSigma+=1)
//	for(uiPrologSize=4; uiPrologSize<=48; uiPrologSize++)
//	for(uiRrologRedEn=0; uiRrologRedEn<=1; uiRrologRedEn++)
	for(i=0; i<16; i++)
	{
#if 	IS_3GPP
		TCP2Params.standard= TCP2_STANDARD_3GPP;
		uiFrameLength= Typical3GPPFrame[i];
#else
		TCP2Params.standard= TCP2_STANDARD_IS2000;
		uiFrameLength= Typical3GPP2Frame[2*i];
		uiRate= Typical3GPP2Frame[2*i+1];
#endif
		TCP2Params.frameLen = uiFrameLength;
		TCP2Params.rate          = uiRate;       
		TCP2Params.minIter = TCP2Params.maxIter= (uiIteratNum&0x1f);
		TCP2Params.maxStarEn     = uiMaxStarEn;
		TCP2Params.prologRedEn   = uiRrologRedEn;
		TCP2Params.prologSize    = uiPrologSize; 
		TCP2Params.intFlag       = 1;    
		TCP2Params.outParmFlag   = 1;
		TCP2Params.snr           = 0;        
		TCP2Params.map           = 0;
		TCP2Params.inputSign     = 1;
		TCP2Params.outputOrder   = 0;
		TCP2Params.numCrcPass    = 0;
		TCP2Params.crcLen        = 0;
		TCP2Params.crcPoly       = 0;
		printf("Sigma=%3d/128  ", uiSigma);
		printf("ProSize=%2d  ", uiPrologSize);
		printf("Iter=%2d  Rate=1/%d  ", uiIteratNum, uiRate);
		printf("FL=%5d  ", uiFrameLength);
		uiTotalErrorBits+=TestTCP2(&TCP2Params, (Uint8)uiSigma);
		printf("\n");
	}
	printf("Test complete. Total Error Bits: %d\n", uiTotalErrorBits);
}

int TestTCP2(TCP2_BaseParams *tcpParameters, Uint8 uiSigma)
{
	TCP2_Params      TcpConfigParms;		
	Uint16 			index;
	TCP2_UserData    * hXabData;
	Int16    *ipTailData;
	Uint32 i,j;
	TCP2_Edma3_Config tcp2_Edma3_Config;
	CSL_Uint64 	CycleCounter;
	Uint32 uiPreprocessingTime;
	Uint32 nTails;
	Uint32 iErrorBits;

	//Clear content in cache, to make the test more close to real system
	//for repeat test, some data may alrady in cache for second test
	//This is not the truth for most real system
	CACHE_wbInvAllL1d(CACHE_WAIT);
//	CACHE_wbInvAllL2(CACHE_WAIT);

	if (tcpParameters->standard==0) /* 3GPP */
	{
		nTails=12;
		tcpParameters->rate=3;
	}
	else	/* 3GPP2 */
	{
		switch (tcpParameters->rate)
		{
			case 2:
				nTails=12;
				break;
			case 3:
				nTails=18;
				break;
			case 4:
				nTails=24;
				break;
			case 5:
				nTails=30;
				break;
			default:
				printf("\nRate not allowed\n");
				return 1;
		}
	}

	for(i=0; i<TCP2_CHANNELS_PER_TIME; i++)
		for(j=0; j<((MAX_TCP2_FRAME_LEN/32+1)/8+1)*8; j++)
			TurboHD[i][j]=0;
	
	if(GenerateBits(TurboOriginalBits,tcpParameters->frameLen))
		puts("Fails Generating Bits");
		
#if 	IS_3GPP
	if(TurboEncode3GPP(TurboOriginalBits,TurboEncodedBits,TurboInterleaverTable,tcpParameters->frameLen,tcpParameters->rate))
#else
	if(TurboEncode3GPP2(TurboOriginalBits,TurboEncodedBits,TurboInterleaverTable,tcpParameters->frameLen,tcpParameters->rate))
#endif
		puts("Fails Turbo Encoding");

	//Generate Deinterleave Table
	for (i=0;i<tcpParameters->frameLen;i++) 
		TurboDeinterTable[TurboInterleaverTable[i]]=i;
	for(;i<tcpParameters->frameLen+8; i++)
	{
		//TCP2 may require 3 extra interleaver table elements to be transfered. 
		TurboInterleaverTable[i]=i;
		//support speculative optimization
		TurboDeinterTable[i]=i;
	}

	MapOnBaseband(TurboEncodedBits,tcpParameters->frameLen*tcpParameters->rate+nTails, (Uint32 *)TurboModulatedData);
	AddAWGN_fixpt((Uint32 *)TurboModulatedData,(Uint32 *)TurboCorruptedData,uiSigma,tcpParameters->frameLen*tcpParameters->rate+nTails);

	CycleCounter=CSL_tscRead();
	TCP2_ScaleData((Int8*)TurboCorruptedData, (Int16 *) TurboScaledData, tcpParameters->frameLen*tcpParameters->rate+nTails);
	CycleCounter=CSL_tscRead()-CycleCounter;
	uiPreprocessingTime= CycleCounter;
#if DETAILED_TIME
	printf("ScalingCyles:%6ld  ", CycleCounter);
#endif
	CycleCounter=CSL_tscRead();
	TCP2_deintUnpunctData((Int16 *) TurboScaledData, TurboRate5Data, TurboDeinterTable, tcpParameters);
	//Keep coherency for the case the data are in external memory
//	CACHE_wbInvL2(TurboRate5Data, tcpParameters->frameLen*4, CACHE_WAIT);
//	CACHE_wbInvL2(TurboInterleaverTable, tcpParameters->frameLen*2, CACHE_WAIT);
	CycleCounter=CSL_tscRead()-CycleCounter;
#if DETAILED_TIME
	printf("DeintUnpunctCyles:%6ld  ", CycleCounter);
#endif
	uiPreprocessingTime+= CycleCounter;


	CycleCounter=CSL_tscRead();
    	/* Generate all TCP parameters */
	TCP2_genParams_bf(tcpParameters, &TcpConfigParms);

        /* Pointer to the X, A, and B data  */	    
        index = TcpConfigParms.frameLen * TcpConfigParms.rate;
        hXabData = &(((Uint8*)TurboCorruptedData)[index]);
        ipTailData= &(((Int16*)TurboScaledData)[index]);
        //Saturate tail data
        for(i=0; i<nTails; i++)
        {
		hXabData[i]= _extu(_sshl(ipTailData[i], 26),0,26);
	}
	/* Calculate the TCP IC values */
        TCP2_genIc(&TcpConfigParms, (TCP2_TailData*)hXabData, &TcpConfigIc);

	/* Program EDMA to service User Channel */
	CycleCounter=CSL_tscRead()-CycleCounter;
#if DETAILED_TIME
	printf("TCPCfgCycles:%4ld  ", CycleCounter);
#endif
	uiPreprocessingTime+= CycleCounter;


#ifdef CSL_TCP_EMDA_MODE	
	CycleCounter=CSL_tscRead();

    setupTcpEdma(&TcpConfigIc, TurboRate5Data, 			
                 TcpConfigParms, TurboInterleaverTable,
                  TurboHD, hTcpOutParms);


#else
	/* Program EDMA to service User Channel */
	tcp2_Edma3_Config.inputConfigAddr= (Uint32)&TcpConfigIc;
	tcp2_Edma3_Config.sysParDataAddr= (Uint32)TurboRate5Data;
	tcp2_Edma3_Config.frameLength= tcpParameters->frameLen;
	tcp2_Edma3_Config.interleaverTableAddr= (Uint32)TurboInterleaverTable;
	tcp2_Edma3_Config.intFlag= tcpParameters->intFlag;
	tcp2_Edma3_Config.hardDecisionAddr= (Uint32)TurboHD;
	tcp2_Edma3_Config.outParmAddr= (Uint32)hTcpOutParms;
	tcp2_Edma3_Config.outParmFlag= tcpParameters->outParmFlag;
	CycleCounter=CSL_tscRead()-CycleCounter;

     CycleCounter=CSL_tscRead();
     tcp2SubmitEdma(&tcp2_Edma3_Config);
#endif
	   
	CycleCounter= (CSL_tscRead()-CycleCounter)/TCP2_CHANNELS_PER_TIME;
#if DETAILED_TIME
	printf("EDMACfgCycles:%4ld  ", CycleCounter);
#endif
	uiPreprocessingTime+= CycleCounter;
	printf("PrePrecessingTime:%6dns  ", uiPreprocessingTime);


	//Keep coherency for the case the data are in external memory
//	CACHE_wbInvAllL2(CACHE_WAIT);
	CycleCounter=CSL_tscRead();
	/* Start the TCP to begin the EDMA transfers and decoding data */    
    
     TCP2_setExtEndian(0);
     TCP2_setInterEndian(0); 
     (*(volatile Uint32*)(TCP_EMULATION_CONTROL_REG_ADDRESS)) = (Uint32) TCP_EMULATION_WORD;      
     TCP2_start();

	//Wait for TCP2 processing complete
#ifdef CSL_TCP_EMDA_MODE	
      EdmaregionIntr.region = CSL_EDMA3_REGION_1;
      EdmaregionIntr.intr = 0xFFFFFFFF;
      EdmaregionIntr.intrh = 0;

       /* Check of TX EDMA channel completion */ 
      do
      { 	
    	CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&EdmaregionIntr);
      } while (!(EdmaregionIntr.intr & (1 << CSL_XMT_TCC)));


      /* Check of RX EDMA channel completion */
      do
      {	
	CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&EdmaregionIntr);
      } while (!(EdmaregionIntr.intr & (1 << CSL_RCV_TCC)));

#else
	//Wait the RX EDMA interrupt flag
	while((EDMA3CC_Regs->IPR&(1<<CSL_EDMA3_CHA_TCP2REVT))==0);

#endif
      CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTRPEND_CLEAR,&EdmaregionIntr);

	CycleCounter=CSL_tscRead()-CycleCounter;
	printf("DecodingTime:%8ldns  ", CycleCounter/TCP2_CHANNELS_PER_TIME);

	iErrorBits=0;
	//Verify hard decision for all user channels
	for (j=0;j<(tcpParameters->frameLen/8);j++)
//		iErrorBits+= _bitc4(TurboOriginalBits[j]^TurboHD[TCP2_CHANNELS_PER_TIME-1][j]);
		iErrorBits+= _bitc4(TurboOriginalBits[j]^TurboHD[0][j]);
	if (iErrorBits)
	{
		printf("BitErrorRate:%4d/%d ", iErrorBits, tcpParameters->frameLen);
	}
	return iErrorBits;
}
//------------------------------------------------------------
//------------------------------------------------------------
/*
TCP2 input data is required to be scaled by 2*(fading/amplifying factor)/Sigma^2
If input data is X , so
Scale factor= 2*Mean(X)/(Mean{X^2}-Mean{X}^2)
                 = 2*(Sum(X)/Len)/(Sum(X^2)/Len-(Sum(X)/Len)^2)
                 = 2*(Sum(X)*Len)/(Sum(X^2)*Len-Sum(X)^2)
Input data Length is frameLength*Rate+Tail
OuData is 16 bit Q2 format
Data array must aligh to 8 byte
*/
void TCP2_ScaleData(Int8 * restrict InData, Int16 * restrict OutData, Uint32  Length)
{
	Uint32 i, uiAbsSum=0, uiSquareSum=0, uiAbsData0, uiAbsData1;
	Uint32 uiEstLength, uiStep, uiRightShiftBits;
	double *dpInData= (double *)InData, *dpOutData=(double *)OutData;
	double dInput, dTemp0, dTemp1;
	unsigned long long llSigma, llMean;
	Uint32 uiSigma, uiMean;
	Uint32 uiScaleFactor, uiFixScaleFactor_4;

	/*Here only use less than 1<<TCP2_SCALE_EST_LEN_BITS (2048 by default) 
	data to estimate the scale factor. To avoid overflow of 32bit intermedial uiSquareSum, 
	TCP2_SCALE_EST_LEN_BITS must be no more 16. This should be acceptible since the 
	scale factor is a statistic value, and 2048 samples should be enough for statistics.
	Another benifit is save MIPS. To save more MIPS, this number can be decreased 
	further according your application */
	if(Length<(1<<TCP2_SCALE_EST_LEN_BITS))
	{
		uiEstLength= Length;
		uiStep=1;
	}
	else
	{
		uiRightShiftBits= 32-_lmbd(1, Length)-TCP2_SCALE_EST_LEN_BITS;
		//Right shif to get a length less than (1<<TCP2_SCALE_EST_LEN_BITS)
		uiEstLength=Length>>(uiRightShiftBits);	
		uiStep= 1<<(uiRightShiftBits);
	}
	uiEstLength= (uiEstLength/8)*8; //round to multiple of 8
	for(i=0; i< uiEstLength/8; i++)
	{
		dInput= *dpInData;
		dpInData+=uiStep;
		//Absolute four data
		uiAbsData0=_minu4(_lo(dInput), _sub4(0x00000000,_lo(dInput)));
		uiAbsData1=_minu4(_hi(dInput), _sub4(0x00000000,_hi(dInput)));
		//Sum of four data
 		uiAbsSum+=_dotpu4(uiAbsData0, 0x01010101);
 		uiAbsSum+=_dotpu4(uiAbsData1, 0x01010101);
 		//Sum of the square of four data
 		uiSquareSum+=_dotpu4(uiAbsData0, uiAbsData0);
 		uiSquareSum+=_dotpu4(uiAbsData1, uiAbsData1);
 	}
	llSigma= _mpy32u(uiSquareSum, uiEstLength)- _mpy32u(uiAbsSum, uiAbsSum);
	
	/* 2*uiAbsSum* uiEstLength is the original numerator, 
	multiple 64 to get Q6 result, so it is 64 * 2*uiAbsSum* uiEstLength  
							= uiAbsSum* uiEstLength<<(7)  */
	llMean= _mpy32u(uiAbsSum, uiEstLength<<(7) ); 

	//Shift both numerator and denominator to less than 32 bit for division
	uiRightShiftBits= 32- _min2(_lmbd(1, _hill(llSigma)), _lmbd(1, _hill(llMean)));
	uiSigma= (_hill(llSigma)<<(32-uiRightShiftBits))|(_loll(llSigma)>>uiRightShiftBits);
	uiMean= (_hill(llMean)<<(32-uiRightShiftBits))|(_loll(llMean)>>uiRightShiftBits);

	//Calculate the scale factor
	uiScaleFactor= uiMean/uiSigma;
	if(uiScaleFactor>0xff)
		uiScaleFactor=0xff;
	else if(uiScaleFactor==0)
		uiScaleFactor=1;
//	printf("ScaleFactor=%3d  ", uiScaleFactor);

	//Pack 4 scale factors into one 32 bit scale factor
	uiFixScaleFactor_4=_packl4(_pack2(uiScaleFactor, uiScaleFactor), 
						_pack2(uiScaleFactor, uiScaleFactor));

	//Scale the data, Length/8+1 is for speculative optimization
	dpInData= (double *)InData; 
	for(i=0; i< (Length/8+1); i++)
	{
		dInput= *dpInData++;
		
		//8bit x 8bit generat 16bit data, Q6
 		dTemp0=_mpysu4(_lo(dInput), uiFixScaleFactor_4);
 		dTemp1=_mpysu4(_hi(dInput), uiFixScaleFactor_4);
 		
		//Shift right by 4 to get Q2 format
 		*dpOutData++=_itod(_shr2(_hi(dTemp0), 4), _shr2(_lo(dTemp0), 4));
 		*dpOutData++=_itod(_shr2(_hi(dTemp1), 4), _shr2(_lo(dTemp1), 4));
	}
}
//------------------------------------------------------------
//Calculate BER according the known sigam
void TCP2_ScaleData_Theory(Int8 * restrict InData, Int16 * restrict OutData, Uint32  Length, Uint32 uiSigma)
{
	Uint32 i;
	double *dpInData= (double *)InData, *dpOutData=(double *)OutData;
	double dInput, dTemp0, dTemp1;
	Uint32 uiScaleFactor, uiFixScaleFactor_4;

	uiScaleFactor= 2*128*128/(uiSigma*uiSigma)*16;	//x16 get Q4 format
	if(uiScaleFactor>0xff)
		uiScaleFactor=0xff;
	else if(uiScaleFactor==0)
		uiScaleFactor=1;
//	printf("ScaleFactor=%3d ", uiScaleFactor);

	//Pack 4 scale factors into one 32 bit scale factor
	uiFixScaleFactor_4=_packl4(_pack2(uiScaleFactor, uiScaleFactor), 
						_pack2(uiScaleFactor, uiScaleFactor));

	//Scale the data, Length/8+1 is for speculative optimization
	dpInData= (double *)InData; 
	for(i=0; i< (Length/8+1); i++)
	{
		dInput= *dpInData++;
		
		//8bit x 8bit generat 16bit data, Q4 x Q4 generate Q8
 		dTemp0=_mpysu4(_lo(dInput), uiFixScaleFactor_4);
 		dTemp1=_mpysu4(_hi(dInput), uiFixScaleFactor_4);
 		
		//Shift right by 4 to get Q2 format
 		*dpOutData++=_itod(_shr2(_hi(dTemp0), 6), _shr2(_lo(dTemp0), 6));
 		*dpOutData++=_itod(_shr2(_hi(dTemp1), 6), _shr2(_lo(dTemp1), 6));
	}
}
//------------------------------------------------------------
//------------------------------------------------------------
//Saturate and conver data to Q4.2 format
//Unpunct 1/n data to 1/5 data
//Deinterleave the interleaved parity data
void TCP2_deintUnpunctData(Int16   *restrict InputData,  Uint32 *restrict	OutputData,	
	Uint16 *restrict DeintTable, TCP2_BaseParams *tcpParameters)
{
	Uint32 i, uiTemp0, uiTemp1;
	Uint32 iFrameLen=tcpParameters->frameLen;
	Uint32 iRate=tcpParameters->rate;
	double * dOutData= (double *)OutputData;
	Uint32 * uipDeIntTable= (Uint32 *)DeintTable;
	double dParityIndex;
	if(iRate==2)
	{
		//iFrameLen+2 is speculative optimization
		for(i=0; i< iFrameLen+2; i+=2)
		{
			//_sshl is for saturate, rotl to specified position
			uiTemp0 = (_extu(_sshl(InputData[i*2], 26), 0, 26))| 		//X
				(_rotl(_sshl(InputData[i*2+1], 26)&0xfc000000, 12)); 	//A
			uiTemp1 = (_extu(_sshl(InputData[i*2+2], 26), 0, 26))| 	//X
				//A', Deinterleave the parity
				(_rotl(_sshl(InputData[DeintTable[i+1]*2+1], 26)&0xfc000000, 24)); 	
			*dOutData++= _itod(uiTemp1, uiTemp0);
		}
	}
	else if(iRate==3)
	{
/*		#pragma MUST_ITERATE(40, 20730)
		for(i=0; i< iFrameLen; i++)
		{
			//_sshl is for saturate
			uiTemp = (_rotl(_sshl(InputData[i*3], 26), 6))|
				(_rotl(_sshl(InputData[i*3+1], 26)&0xfc000000, 12))|
				//Deinterleave the last parities
				(_rotl(_sshl(InputData[DeintTable[i]*3+2], 26)&0xfc000000, 24)); 	
			OutputData[i] = uiTemp;
		}*/
		for(i=0; i< iFrameLen+2; i+=2)
		{
			//Get Index of Interleaved Parities	
			dParityIndex=_mpy2(*uipDeIntTable++, 0x00030003);  	
			//_sshl is for saturate, rotl to specified position
			uiTemp0 = (_extu(_sshl(InputData[i*3], 26), 0, 26))| 	//X
				(_rotl(_sshl(InputData[i*3+1], 26)&0xfc000000, 12))| 	//A
				//A', Deinterleave the  parities
				(_rotl(_sshl(InputData[_lo(dParityIndex)+2], 26)&0xfc000000, 24)); 	
			uiTemp1 = (_extu(_sshl(InputData[i*3+3], 26), 0, 26))| 	//X
				(_rotl(_sshl(InputData[i*3+4], 26)&0xfc000000, 12))| 	//A
				//A', Deinterleave the  parities
				(_rotl(_sshl(InputData[_hi(dParityIndex)+2], 26)&0xfc000000, 24)); 	
			*dOutData++= _itod(uiTemp1, uiTemp0);
		}
		
	}
	else if(iRate==4)
	{
		for(i=0; i< iFrameLen+2; i+=2)
		{
			//Get Index of Interleaved Parities	
			dParityIndex=_mpy2(*uipDeIntTable++, 0x00040004);  
			//_sshl is for saturate, rotl to specified position
			uiTemp0 = (_extu(_sshl(InputData[i*4], 26), 0, 26))| 	//X
				(_rotl(_sshl(InputData[i*4+1], 26)&0xfc000000, 12))| 	//A
				(_rotl(_sshl(InputData[i*4+2], 26)&0xfc000000, 18))| 	//B
				//B', Deinterleave the  parities
				(_rotl(_sshl(InputData[_lo(dParityIndex)+3], 26)&0xfc000000, 30)); 	
			uiTemp1 = (_extu(_sshl(InputData[i*4+4], 26), 0, 26))| 	//X
				(_rotl(_sshl(InputData[i*4+5], 26)&0xfc000000, 12))| 	//A
				//A', Deinterleave the  parities
				(_rotl(_sshl(InputData[_hi(dParityIndex)+2], 26)&0xfc000000, 24))| 	
				//B', Deinterleave the  parities
				(_rotl(_sshl(InputData[_hi(dParityIndex)+3], 26)&0xfc000000, 30)); 	
			*dOutData++= _itod(uiTemp1, uiTemp0);
		}
	}
	else if(iRate==5)
	{
		for(i=0; i< iFrameLen+2; i+=2)
		{
			//Get Index of Interleaved Parities	
			dParityIndex=_mpy2(*uipDeIntTable++, 0x00050005);  
			//_sshl is for saturate, rotl to specified position
			uiTemp0 = (_extu(_sshl(InputData[i*5], 26), 0, 26))| 	//X
				(_rotl(_sshl(InputData[i*5+1], 26)&0xfc000000, 12))| 	//A
				(_rotl(_sshl(InputData[i*5+2], 26)&0xfc000000, 18))| 	//B
				//A', Deinterleave the  parities
				(_rotl(_sshl(InputData[_lo(dParityIndex)+3], 26)&0xfc000000, 24))| 	
				//B', Deinterleave the  parities
				(_rotl(_sshl(InputData[_lo(dParityIndex)+4], 26)&0xfc000000, 30)); 	
			uiTemp1 = (_extu(_sshl(InputData[i*5+5], 26), 0, 26))| 	//X
				(_rotl(_sshl(InputData[i*5+6], 26)&0xfc000000, 12))| 	//A
				(_rotl(_sshl(InputData[i*5+7], 26)&0xfc000000, 18))| 	//B
				//A', Deinterleave the  parities
				(_rotl(_sshl(InputData[_hi(dParityIndex)+3], 26)&0xfc000000, 24))| 
				//B', Deinterleave the  parities
				(_rotl(_sshl(InputData[_hi(dParityIndex)+4], 26)&0xfc000000, 30)); 	
			*dOutData++= _itod(uiTemp1, uiTemp0);
		}
	}
}
/** ============================================================================
 *   @n@b TCP2_genParams_bf  
 *
 *   @b Description
 *   @n This function copies the basic parameters, to the configParams 
 *		parameters structure.
 *
 *   @b Arguments
     @verbatim
            configBase      Pointer to the TCP2_BaseParams structure 
            
            configParams    Pointer to the TCP configuration parameters 
                            structure.

     @endverbatim
 *
 *   <b> Return Value </b>  Uint32
 *   @n                     The number of sub frames for shared processing mode                       
 *
 *   <b> Pre Condition </b>
 *   @n  configBase is populated with all the configurtaion parameters
 *
 *   <b> Post Condition </b>
 *   @n  None 
 *
 *   @b Modifies
 *   @n The configParams argument passed.   
 *
 *   @b Example
 *   @verbatim
        extern TCP2_UserData *userData;
        
	    TCP2_BaseParams	configBase;
	    TCP2_Params      configParams;
        TCP2_TailData    *xabData;
    	Uint32          frameLen = 40;

        xabData = &userData [frameLen];
        
        // assign the configuration parameters 
	    configBase.frameLen		= frameLen;
	    configBase.inputSign	= TCP2_INPUT_SIGN_POSITIVE;
	    configBase.intFlag		= 1;
	    configBase.maxIter		= 8;
	    configBase.maxStarEn	= TRUE;
	    configBase.standard		= TCP2_STANDARD_3GPP;
	    configBase.crcLen		= 0;
	    configBase.crcPoly		= 0;
	    configBase.minIter		= 1;
	    configBase.numCrcPass	= 1;
	    configBase.outParmFlag	= 0;
	    configBase.outputOrder	= TCP2_OUT_ORDER_0_31;
	    configBase.prologRedEn	= FALSE;
	    configBase.prologSize	= 24;
	    configBase.rate         = TCP2_RATE_1_3;
	    configBase.snr          = 0;

	    for (cnt = 0; cnt < 16; cnt++)
		    configBase.extrScaling [cnt] = 32;

	    // setup the TCP configuration registers parameters 
	    TCP2_genParams_bf (&configBase, &configParams);

     @endverbatim
 * ===========================================================================
 */
Uint32 TCP2_genParams_bf (
    TCP2_BaseParams  *restrict configBase,
    TCP2_Params      *restrict configParams
) 
{
	Uint8 cnt;
	Uint32 numSubFrames = 0;
	Uint32 frameLen = configBase->frameLen;

    /* Assign the configuration parameters */
	configParams->standard    = configBase->standard;
	configParams->rate        = configBase->rate;
	configParams->intFlag     = configBase->intFlag;
	configParams->outParmFlag = configBase->outParmFlag;
	configParams->prologSize  = configBase->prologSize;
	configParams->maxIter     = configBase->maxIter;
	configParams->snr         = configBase->snr;
	configParams->maxStarEn	  = configBase->maxStarEn;
	configParams->prologRedEn = configBase->prologRedEn;
	configParams->minIter	  = configBase->minIter;
	configParams->inputSign	  = configBase->inputSign;	
	configParams->outputOrder = configBase->outputOrder;	
	configParams->numCrcPass	 = configBase->numCrcPass;	
	configParams->crcLen		 = configBase->crcLen;
	configParams->crcPoly	     = configBase->crcPoly;
	configParams->map		     = configBase->map;

	for (cnt = 0; cnt < NUM_EXTRINSIC_FACTORS; cnt++)
		configParams->extrScaling [cnt] = configBase->extrScaling [cnt];

	/* calculate the sub blocks */
    configParams->frameLen   = frameLen;

    if (frameLen <= TCP2_FLEN_MAX) {             
        /* Configure for Standalone processing */
        
	    /* to assign the frame length and mode */
        configParams->mode 		= TCP2_MODE_SA;

        /* Calculate the sub blocks, reliabilty length and 
         * num of sliding windows 
         */
		TCP2_calcSubBlocks_bf (configParams);
	}
	return numSubFrames;
} /* end TCP2_genParams_bf */


/* ============================================================================
 *   @n@b TCP2_calcSubBlocks_bf 
 *
 *   @b Description
 *   @n This function calculates the number of sub blocks for the TCP 
 *		processing. The reliability length is also calculated and the 
 *		configParms structure is populated.
 *
 *   @b Arguments
     @verbatim
            configParms     Pointer to the structure holding the TCP 
                            configuration parameters.

     @endverbatim
 *
 *   <b> Return Value </b>  
 *   @n  None
 *
 *   <b> Pre Condition </b>
 *   @n  None
 *
 *   <b> Post Condition </b>
 *   @n  None 
 *
 *   @b Modifies
 *   @n The configParms argument passed.   
 *
 *   @b Example
 *   @verbatim
		extern TCP2_Params *configParms;
		...
		TCP2_calcSubBlocks_bf (configParms);
     @endverbatim
 * ===========================================================================
 */
void TCP2_calcSubBlocks_bf (
	TCP2_Params *configParms 
)
{
	Uint16 frameLen   = configParms->frameLen;
	Uint16 winSize;
	Uint16 numSlidingWindow;
	Uint16 relLen;
	Uint16 numSubBlock = 1;
	Uint16 relLenMax = TCP2_RLEN_MAX;

    /*  calculate the number of sliding windows */
    if (frameLen <= 128) { 
    	numSlidingWindow = 1;
		relLen = frameLen;
	}
    else 
    	numSlidingWindow = 2;
    
    /* Calculate the reliability length and number of sub blocks */
    if (2 == numSlidingWindow) {
	    	winSize 	= TCP2_normalCeil (frameLen, numSlidingWindow);
	   	do {
	   		numSubBlock = TCP2_normalCeil (winSize, relLenMax);
	    		relLen 		= winSize / numSubBlock;
			
			if ((relLen * numSubBlock) < winSize)
				relLen++;

			relLenMax--;
		} while (((numSubBlock * relLen * numSlidingWindow) - frameLen) >= 
									(relLen - configParms->prologSize));
    }
    
    
    configParms->relLen          = relLen - 1;
    configParms->numSubBlock     = numSubBlock;
	configParms->numSlideWin	 = numSlidingWindow - 1;

    return;
} /* end TCP2_calcSubBlocks_bf () */

