/*  ===========================================================================
 *   Copyright (c) Texas Instruments Inc 2006
 *
 *   Use of this software is controlled by the terms and conditions found in
 *   the license agreement under which this software has been supplied.
 *   ==========================================================================
 */


/** ===========================================================================
 * @desc VCP2 channel density test.
 *
 * ============================================================================
 */
/* ============================================================================
 *  Revision History
 *  ===============
 *  11-July-2006   Brighton     File Created.
 * ============================================================================
 */



#include <stdio.h>
#include <c6x.h>
#include <csl_VCP2.h>
#include <csl_VCP2Aux.h>
#include <csl_cache.h>
#include "VCP2_Channel_Density.h"
#include "VCP2_TCP2_EDMA.h"

#define DETAILED_TIME 	0

#pragma DATA_SECTION(OriginalBits, "VCP2Data")
#pragma DATA_SECTION(EncodedBits, "VCP2Data")
#pragma DATA_SECTION(modulatedData, "VCP2Data")
#pragma DATA_SECTION(corruptedData, "VCP2Data")
#pragma DATA_SECTION(branch_metrics, "VCP2Data")
#pragma DATA_SECTION(packedHD, "VCP2Data")
//Align to L2 cache line size for cache operation
#pragma DATA_ALIGN(OriginalBits, 128)
#pragma DATA_ALIGN(EncodedBits, 128)
#pragma DATA_ALIGN(modulatedData, 128)
#pragma DATA_ALIGN(corruptedData, 128)
#pragma DATA_ALIGN(branch_metrics, 128)
 #pragma DATA_ALIGN(packedHD, 128)
//All array length add 8 to support speculative optimization
//Original bits
Uint8 OriginalBits[(MAX_VCP2_FRAME_LEN/8)+8];
//Encoded bits
Uint32 EncodedBits[(MAX_VCP2_FRAME_LEN+MAX_VCP2_TAIL_LEN)*4/32+8];
//modulated user data
Uint8 modulatedData[(MAX_VCP2_FRAME_LEN+MAX_VCP2_TAIL_LEN+8)*4];
//User data corrupted by noise
Uint8 corruptedData[(MAX_VCP2_FRAME_LEN+MAX_VCP2_TAIL_LEN+8)*4];
//Branch Metrics
Uint8 branch_metrics[(MAX_VCP2_FRAME_LEN+MAX_VCP2_TAIL_LEN+8)*8];
 //Hard decision of VCP2
Uint8 packedHD[VCP2_CHANNELS_PER_TIME][((MAX_VCP2_FRAME_LEN+7)/8+7)/8*8];
//Output parameter of VCP2
Uint32 VcpOutParms[4];

//Align to 16 bytes for EDMA transfer
#pragma DATA_ALIGN(VcpConfigIc, 16)
VCP2_ConfigIc    VcpConfigIc;
Uint32 Typical3GFrame[]= {
	640, 3, 
	/*3GPP*/
	112, 3,
	164, 3,
	260, 3,
	89,	 3,
	103, 3,
	60,	 2,
	73,	 3,
	99,	 3,
	40,	 2,
	83,	 3,
	84,	 3,
	69,	 3,
	87,	 3,
	66,	 3,
	76,	 3,
	63,	 3,
	63,	 3,
	57,	 3,
	54,	 3,
	50,	 3,
	53,	 3,
	47,	 3,

	640, 3,
	/*3GPP2*/
	16  ,3,
	40  ,3,
	88  ,3,
	184 ,3,
	640, 3, 
	28  ,2,
	64  ,2,
	136 ,2,
	280 ,2,
	640, 3, 
	22  ,4,
	46  ,4,
	88  ,4,
	184 ,4,
	376 ,4,
	760 ,4,
	1528,4,
	3064,4,
	640, 3, 
	28  ,4,
	64  ,4,
	136 ,4,
	280 ,4,
	568 ,4,
	1144,4,
	2296,4,
	4600,4,
	9208,4,

	640, 3, 
	640, 3};

void VCP2_Channel_Density_Test()
{
	Uint32 uiTotalErrorBits=0;
	VCP2_BaseParams vcpParameters;
	Uint32 i, j;
	Uint32 uiFrameLength=89; 
	Uint32 uiRate= 3;
	Uint32 uiConstLen= 9; 	//constraint length is 9 for 3G
	Uint32 uiSigma=64; 	//Sigma is unisgned in Q7 format

//	for(uiSigma=64; uiSigma<(128*1.2); uiSigma+=1)
//	for(uiRate=2; uiRate<=4; uiRate++)
//	for(uiConstLen=5; uiConstLen<=9; uiConstLen++)
//	for(uiFrameLength=8; uiFrameLength<3*1024; )
	for(i=0; i<53; i++)
	{
		uiFrameLength= Typical3GFrame[2*i];
		uiRate= Typical3GFrame[2*i+1];
	    	vcpParameters.frameLen    	= uiFrameLength;
		vcpParameters.rate        		= uiRate;
		vcpParameters.constLen    	= uiConstLen; 		
		vcpParameters.decision   		= 0;		//Hard decison
		vcpParameters.readFlag    	= 1; 		//Read output parameters
		vcpParameters.tbConvrgMode   = 0; 	//tail mode
		vcpParameters.tailBitEnable     	= 0; 		//Traceback state index disabled
		vcpParameters.traceBackIndex  = 0;
		vcpParameters.yamTh       	= 16;
		vcpParameters.stateNum   	= 0; 		//Init state is always 0
		vcpParameters.outOrder          = 0;
		vcpParameters.perf              = VCP2_PERF_MOST_CRITICAL;//VCP2_SPEED_MOST_CRITICAL; //;

		printf("Sigma=%3d/128  ", uiSigma);
		printf("Rate=1/%d  ", uiRate);
		printf("ConstLen=%d  ", uiConstLen);
		printf("FL=%5d  ", uiFrameLength);
		uiTotalErrorBits+=TestVCP2(&vcpParameters, (Uint8)uiSigma);
		printf("\n");
#if 1
		if(uiFrameLength<120)
			uiFrameLength+=8;
		else if (uiFrameLength<2048)
			uiFrameLength+= 64; 
		else if (uiFrameLength<4096)
			uiFrameLength+= 128; 
		else
			uiFrameLength+= 512; 
#endif
	}
	printf("Test complete. Total Error Bits: %d\n", uiTotalErrorBits);
}



int TestVCP2(VCP2_BaseParams *VcpParameters, Uint8 uiSigma)
{
	VCP2_Params      VcpConfigParms;		
	Uint16 			index;
	Uint32 i,j;
	VCP2_Edma3_Config VCP2_Edma3_Config;
	volatile CSL_Uint64 	CycleCounter;
	Uint32 uiPreprocessingTime=0;
	Uint32 iErrorBits;
	Uint8 	ubTemp;
	Uint16          	 poly[4] = {0,0,0,0};    

	//Clear content in cache, to make the test more close to real system
	//for repeat test, some data may alrady in cache for second test
	//This is not the truth for most real system
//	CACHE_wbInvAllL1d(CACHE_WAIT);
//	CACHE_wbInvAllL2(CACHE_WAIT);

	//Clear output buffer for hard decision 
	for(i=0; i<VCP2_CHANNELS_PER_TIME; i++)
		for(j=0; j<((MAX_VCP2_FRAME_LEN+7)/8+7)/8*8; j++)
			packedHD[i][j]=0x5a;
	
	CycleCounter=CSL_tscRead();
	/* Fill out VCP2_ parameters */
	VCP2_genParams_bf(VcpParameters, &VcpConfigParms);  
	/* Calculate the VCP2_ IC values */
	VCP2_genIc(&VcpConfigParms, &VcpConfigIc);

	VCP2_Edma3_Config.inputConfigAddr= (Uint32)&VcpConfigIc;
	VCP2_Edma3_Config.branchMetricsAddr= (Uint32)branch_metrics;
	VCP2_Edma3_Config.hardDecisionAddr= (Uint32)packedHD;
	VCP2_Edma3_Config.outParmAddr= (Uint32)VcpOutParms;
	VCP2_Edma3_Config.outParmFlag= VcpConfigParms.readFlag;
	VCP2_Edma3_Config.traceBack= VcpConfigParms.traceBack;
	VCP2_Edma3_Config.bmBuffLen= 4*(VcpConfigParms.bmBuffLen+1)*(1<<(VcpConfigParms.rate-1));
	VCP2_Edma3_Config.numBmFrames= VcpConfigParms.numBmFrames;
	VCP2_Edma3_Config.decBuffLen= 8*(VcpConfigParms.decBuffLen+1);
	VCP2_Edma3_Config.numDecFrames= ((VcpConfigParms.frameLen+7)/8+
		VCP2_Edma3_Config.decBuffLen-1)/VCP2_Edma3_Config.decBuffLen;
	CycleCounter=CSL_tscRead()-CycleCounter;
#if DETAILED_TIME
	printf("VCPCfgCycles:%4ld  ", CycleCounter);
#endif
	uiPreprocessingTime= CycleCounter;
			
	CycleCounter=CSL_tscRead();
	/* Program EDMA to service User Channel */
	vcp2SubmitEdma(&VCP2_Edma3_Config);
	CycleCounter= (CSL_tscRead()-CycleCounter)/VCP2_CHANNELS_PER_TIME;
#if DETAILED_TIME
	printf("EDMACfgCycles:%4ld  ", CycleCounter);
#endif
	uiPreprocessingTime+= CycleCounter;

	if (VcpConfigParms.poly0) poly[0] = VcpConfigParms.poly0|0x100;
	if (VcpConfigParms.poly1) poly[1] = VcpConfigParms.poly1|0x100;
	if (VcpConfigParms.poly2) poly[2] = VcpConfigParms.poly2|0x100;
	if (VcpConfigParms.poly3) poly[3] = VcpConfigParms.poly3|0x100; 


	if(GenerateBits((Uint32 *)OriginalBits,VcpParameters->frameLen))
		puts("Fails Generating Bits.");
		
	if (ConvolutionalEncode((Uint32 *)OriginalBits,EncodedBits,VcpParameters->frameLen,poly))
		puts("Fails Convolution Encoding.");

	MapOnBaseband(EncodedBits,(VcpParameters->frameLen+VcpParameters->constLen-1)*
		VcpParameters->rate, (Uint32 *)modulatedData);
	AddAWGN_fixpt((Uint32 *)modulatedData,(Uint32 *)corruptedData,uiSigma,
		(VcpParameters->frameLen+VcpParameters->constLen-1)*VcpParameters->rate);

	CycleCounter=CSL_tscRead();
	VCP2_ScaleInputData_CalcBM((Int8 *)corruptedData, (Int8 *)branch_metrics, VcpParameters->rate,
		(VcpParameters->frameLen+VcpParameters->constLen-1));
	CycleCounter=CSL_tscRead()-CycleCounter;
#if DETAILED_TIME
	printf("ScaleBMCalcCycles:%6ld  ", CycleCounter);
#endif
	uiPreprocessingTime+= CycleCounter;
	printf("PrePrecessingTime:%7dns  ", uiPreprocessingTime);


	//Keep coherency for the case the data are in external memory
//	CACHE_wbInvAllL2(CACHE_WAIT);
	CycleCounter=CSL_tscRead();
	/* Start the VCP to begin the EDMA transfers and decoding data */    
	VCP2_reset();
	VCP2_start();

	//Wait for VCP2 processing complete
	//Wait the RX EDMA interrupt flag
	while((EDMA3CC_Regs->IPR&(1<<CSL_EDMA3_CHA_VCP2REVT))==0);
	CycleCounter=CSL_tscRead()-CycleCounter;
	printf("DecodingTime:%8ldns  ", CycleCounter/VCP2_CHANNELS_PER_TIME);

	iErrorBits=0;
	//Verify hard decision for all user channels
	for (j=0;j<(VcpParameters->frameLen/8);j++)
	{
		iErrorBits+= _bitc4(((Uint32)OriginalBits[j])^
				((Uint32)packedHD[VCP2_CHANNELS_PER_TIME-1][j]));
	}
	if (iErrorBits)
	{
		printf("BitErrorRate:%4d/%d ", iErrorBits, VcpParameters->frameLen);
	}
	return iErrorBits;
}
/*
Scale input data to following range
Rate     Range
2       [-64;+63]
3       [-42;+42]
4       [-31;+31]
SoftInput must align to 8 byte boundary, 
For speculative optimization, 
BranchEetrics buffer must be big enough length+7 data*/
void	VCP2_ScaleInputData_CalcBM(Int8 * restrict softInput, Int8 * restrict BranchMetrics, Uint32 Rate, Uint32 Length)
{
	Int32 i;
	Uint32 uiMaxLo=0, uiMaxHi=0;
	Uint32 uiAbsDataLo, uiAbsDataHi;
	Uint32 uiRange, uiFixScaleFactor_4, uiScaleFactor;
	Uint32 uiInput3_0, uiInput2, *uipOutData;
	Uint32 uiTemp1_0, uiBM1_0, uiBM3_2, uiBM3_0;
	double dInput,  dTemp3_0, dTemp7_4;
	double * dpInData= (double *)softInput, *dpOutData;

	if(Rate==3)
		uiRange=42;
	else if(Rate==4)
		uiRange=31;
	else if(Rate==2)
		uiRange=63;
	else
		return;

 	for(i=0; i<Length*Rate/8; i++)
 	{
 		dInput=*dpInData++;
 		//Get absolute value
 		uiAbsDataLo=  _minu4(_lo(dInput), _sub4(0x00000000,_lo(dInput)));
 		uiAbsDataHi= _minu4(_hi(dInput), _sub4(0x00000000,_hi(dInput)));
 		uiMaxLo=_maxu4(uiMaxLo,uiAbsDataLo);
 		uiMaxHi=_maxu4(uiMaxHi, uiAbsDataHi);
 	}
	uiMaxLo= _maxu4(uiMaxLo, uiMaxHi);
	uiMaxLo= _maxu4(uiMaxLo, _packlh2(uiMaxLo, uiMaxLo));
	uiMaxLo= _maxu4(uiMaxLo, _swap4(uiMaxLo))&0xff;

	//Compute scaling factor
	uiScaleFactor= 128*uiRange/uiMaxLo; //x256 to get Q7 formart
	if(uiScaleFactor>128)
		uiScaleFactor=128;
//	uiScaleFactor=128; 	//bypass sacling for test
//	printf("ScaleFactor= %d/128 ", uiScaleFactor);
	//Pack 4 scale factors into one 32 bit scale factor
	uiFixScaleFactor_4=_packl4(_pack2(uiScaleFactor, uiScaleFactor), 
						_pack2(uiScaleFactor, uiScaleFactor));

	if(Rate==3)
	{
		uipOutData= (Uint32 *)BranchMetrics;
		//make length to be multiple of 2, output buffer maybe speculative overwrite for optimization
		Length=(Length+1)/2*2; 	
		#pragma UNROLL(2)
	 	for(i=0; i<Length; i++)
	 	{
			//Read 4 bytes, only use 3 bytes per time, which denote one encoded bit
	 		uiInput3_0=  _mem4(&softInput[i*3]);

			//Scale
			//8bit x 8bit generat 16bit data, Q0 x Q7 generate Q7
	 		dTemp3_0=_mpysu4(uiInput3_0, uiFixScaleFactor_4);
	 		//shift r2 to Q5 format
	 		uiInput2= _shr2(_hi(dTemp3_0),2);

			//Calculate branch metrics for fist bit
			//swap (r1,r0) to (r0, r1)
	 		uiTemp1_0=_packlh2(_lo(dTemp3_0), _lo(dTemp3_0));
	 		//(r0+r1j)*(1+j)=(r0-r1)+(r0+r1)j
	 		/*0x40004000 denotes (1+1j) with Q14 format, so Q7 x Q14= Q21, 
	 		cmpyr only save higher 16 bits, so the result is Q5*/
 			uiTemp1_0=_cmpyr(uiTemp1_0, 0x40004000);

 			//pack to get (r0+r1, r2)
 			uiBM1_0= _pack2(uiTemp1_0, uiInput2);
 			//((r0+r1)+r2j)*(1+j)=(r0+r1-r2)+(r0+r1+r2)j
	 		/*0x08000800 denotes (1+1j) with Q11 format, so Q5 x Q11= Q16, 
	 		cmpyr only save higher 16 bits, so the result is Q0, just is the result we need*/
 			uiBM1_0=_cmpyr(uiBM1_0, 0x08000800);

 			//pack to get (r0-r1, r2)
 			uiBM3_2= _packhl2(uiTemp1_0, uiInput2);
 			//((r0-r1)+r2j)*(1+j)=(r0-r1-r2)+(r0-r1+r2)j
 			uiBM3_2=_cmpyr(uiBM3_2, 0x08000800);

			//Pack the lower bytes as output
			*uipOutData++=_packl4(uiBM3_2, uiBM1_0);
	 	}
	}
	else if(Rate==4)
	{
		Uint32 uiTemp2_1, uiInput3, uiInput1_0, uiInput1, uiInput0, uiBM7_4;
		dpOutData= (double *)BranchMetrics;
		//make length to be multiple of 2, output buffer maybe speculative overwrite for optimization
		Length=(Length+1)/2*2; 	
		#pragma UNROLL(2)
	 	for(i=0; i<Length; i++)
	 	{
			//Read 4 bytes, which denote one encoded bits
	 		uiInput3_0=  _amem4(&softInput[i*4]);

			//Scale
			//8bit x 8bit generat 16bit data, Q0 x Q7 generate Q7
	 		dTemp3_0=_mpysu4(uiInput3_0, uiFixScaleFactor_4);

			//Calculate branch metrics for fist bit
			//pack to get (r1, r2)
	 		uiTemp2_1=_packhl2(_lo(dTemp3_0), _hi(dTemp3_0));
	 		//(r1+r2j)*(1+j)=(r1-r2)+(r1+r2)j
	 		/*0x40004000 denotes (1+1j) with Q14 format, so Q7 x Q14= Q21, 
	 		cmpyr only save higher 16 bits, so the result is Q5*/
 			uiTemp2_1=_cmpyr(uiTemp2_1, 0x40004000);

	 		//shift r3 to Q5 format
	 		uiInput3= _shr2(_hi(dTemp3_0),2);
 			//pack to get (r1+r2, r3)
 			uiBM1_0= _packlh2(uiTemp2_1, uiInput3);
 			//((r1+r2)+r3j)*(1+j)=(r1+r2-r3)+(r1+r2+r3)j
	 		/*0x08000800 denotes (1+1j) with Q11 format, so Q5 x Q11= Q16, 
	 		cmpyr only save higher 16 bits, so the result is Q0, just is the result we need*/
 			uiBM1_0=_cmpyr(uiBM1_0, 0x08000800);

 			//pack to get (r1-r2, r3)
 			uiBM3_2= _packh2(uiTemp2_1, uiInput3);
 			//((r1-r2)+r3j)*(1+j)=(r1-r2-r3)+(r1-r2+r3)j
 			uiBM3_2=_cmpyr(uiBM3_2, 0x08000800);
			//Pack the lower bytes
			uiBM3_0=_packl4(uiBM3_2, uiBM1_0);

			//shift r1, r0 7 bits to get Q0
			uiInput1_0=_shr2(_lo(dTemp3_0),7);
			uiInput1_0=_packl4(uiInput1_0, uiInput1_0);
			//pack to get packed 4 byte (r0,r0,r0,r0)
			uiInput0=_packl4(uiInput1_0, uiInput1_0);
			uiBM3_0= _add4(uiInput0, uiBM3_0);

			//pack to get packed 4 byte (r1,r1,r1,r1)
			uiInput1=_packh4(uiInput1_0, uiInput1_0);
			uiBM7_4= _sub4(_sub4(uiBM3_0, uiInput1), uiInput1);
			
			*dpOutData++= _itod(uiBM7_4, uiBM3_0);
	 	}
	}
	else 	//Rate=2
	{
		Uint32 uiBM7_6, uiBM5_4;
		dpOutData= (double *)BranchMetrics;
		//make length to be multiple of 8, output buffer maybe speculative overwrite for optimization
		Length=(Length+7)/8*8; 	
		#pragma UNROLL(2)
	 	for(i=0; i<(Length)/4; i++)
	 	{
			//Read 8 bytes, which denote 4 encoded bits
			dInput=  _amemd8(&softInput[i*8]);

			//Scale
			//8bit x 8bit generat 16bit data, Q0 x Q7 generate Q7
	 		dTemp3_0=_mpysu4(_lo(dInput), uiFixScaleFactor_4);

			//Calculate branch metrics for fist bit
			//swap (r1,r0) to (r0, r1)
	 		uiBM1_0=_packlh2(_lo(dTemp3_0), _lo(dTemp3_0));
	 		//(r0+r1j)*(1+j)=(r0-r1)+(r0+r1)j
	 		/*0x02000200 denotes (1+1j) with Q9 format, so Q7 x Q9= Q16, 
	 		cmpyr only save higher 16 bits, so the result is Q0*/
 			uiBM1_0=_cmpyr(uiBM1_0, 0x02000200);

	 		uiBM3_2=_packlh2(_hi(dTemp3_0), _hi(dTemp3_0));
 			uiBM3_2=_cmpyr(uiBM3_2, 0x02000200);

			//8bit x 8bit generat 16bit data, Q0 x Q7 generate Q7
	 		dTemp7_4=_mpysu4(_hi(dInput), uiFixScaleFactor_4);

	 		uiBM5_4=_packlh2(_lo(dTemp7_4), _lo(dTemp7_4));
 			uiBM5_4=_cmpyr(uiBM5_4, 0x02000200);

	 		uiBM7_6=_packlh2(_hi(dTemp7_4), _hi(dTemp7_4));
 			uiBM7_6=_cmpyr(uiBM7_6, 0x02000200);

			//Pack the lower bytes as output
			*dpOutData++=_itod(_packl4(uiBM7_6, uiBM5_4),_packl4(uiBM3_2, uiBM1_0));
	 	}
	}

	
}

//rewrite CSL function to fix a bug
void
VCP2_genParams_bf (
	VCP2_BaseParams * restrict pConfigBase,
	VCP2_Params * restrict pConfigParams
)
{
    VCP2_Rate rate = pConfigBase->rate;
    Bool tbConvrgMode = pConfigBase->tbConvrgMode;
    Bool tailBitEnable = pConfigBase->tailBitEnable;
    Uint8 constLen = pConfigBase->constLen;
    Uint8 decision = pConfigBase->decision;
    Uint8 readFlag = pConfigBase->readFlag;
    Uint8 stateNum = pConfigBase->stateNum;
    Uint8 outOrder = pConfigBase->outOrder;
    Uint8 perf = pConfigBase->perf;
    Uint16 frameLen = pConfigBase->frameLen;
    Uint16 yamTh = pConfigBase->yamTh;
    Uint16 traceBackIndex = pConfigBase->traceBackIndex;

    Uint8 poly0 = 0;
    Uint8 poly1 = 0;
    Uint8 poly2 = 0;
    Uint8 poly3 = 0;
    Uint16 relLen = 0;
    Uint16 convDist = 0;
    Int16 minSm = 0;
    Int16 maxSm = 0x400; /* Select a maximum value for initial state metric */
    Uint8 bmBuffLen;
    Uint8 decBuffLen;
    Uint16 numBmFrames;
    Uint16 numDecFrames;
    Uint8 traceBack;

    Uint8 i;
    Uint8 convMultiplier;
    Uint16 polynomes[4];
    Uint16 fMax;
    Uint16 rcMax;
    Uint16 multiple;
    Uint16 nsw;
    Uint16 mask;
    Uint16 bmNum;
    Uint32 bmCnt;

    /* VCPIC0 parameters */
    if (rate == VCP2_RATE_1_2) {
        if (constLen == 9) {
            polynomes[0] = VCP2_POLY0_FOR_CONST9_RATE_1_2;
            polynomes[1] = VCP2_POLY1_FOR_CONST9_RATE_1_2;
        }                       /* end if constLen == 9 */
        else if (constLen == 8) {
            polynomes[0] = VCP2_POLY0_FOR_CONST8_RATE_1_2;
            polynomes[1] = VCP2_POLY1_FOR_CONST8_RATE_1_2;
        }                       /* end if constLen == 8 */
        else if (constLen == 7) {
            polynomes[0] = VCP2_POLY0_FOR_CONST7_RATE_1_2;
            polynomes[1] = VCP2_POLY1_FOR_CONST7_RATE_1_2;
        }                       /* end if constLen == 7 */
        else if (constLen == 6) {
            polynomes[0] = VCP2_POLY0_FOR_CONST6_RATE_1_2;
            polynomes[1] = VCP2_POLY1_FOR_CONST6_RATE_1_2;
        }                       /* end if constLen == 6 */
        else {                  /* (constLen == 5) */
            polynomes[0] = VCP2_POLY0_FOR_CONST5_RATE_1_2;
            polynomes[1] = VCP2_POLY1_FOR_CONST5_RATE_1_2;
        }                       /* end if constLen == 5 */
    }                           /* end if rate 1/2 */
    else if (rate == VCP2_RATE_1_3){                      /* if rate 1/3*/
        if (constLen == 9) {
            polynomes[0] = VCP2_POLY0_FOR_CONST9_RATE_1_3;
            polynomes[1] = VCP2_POLY1_FOR_CONST9_RATE_1_3;
            polynomes[2] = VCP2_POLY2_FOR_CONST9_RATE_1_3;
        }                       /* end if constLen == 9 */
        else if (constLen == 8) {
            polynomes[0] = VCP2_POLY0_FOR_CONST8_RATE_1_3;
            polynomes[1] = VCP2_POLY1_FOR_CONST8_RATE_1_3;
            polynomes[2] = VCP2_POLY2_FOR_CONST8_RATE_1_3;
        }                       /* end if constLen == 8 */
        else if (constLen == 7) {
            polynomes[0] = VCP2_POLY0_FOR_CONST7_RATE_1_3;
            polynomes[1] = VCP2_POLY1_FOR_CONST7_RATE_1_3;
            polynomes[2] = VCP2_POLY2_FOR_CONST7_RATE_1_3;
        }                       /* end if constLen == 7 */
        else if (constLen == 6) {
            polynomes[0] = VCP2_POLY0_FOR_CONST6_RATE_1_3;
            polynomes[1] = VCP2_POLY1_FOR_CONST6_RATE_1_3;
            polynomes[2] = VCP2_POLY2_FOR_CONST6_RATE_1_3;
        }                       /* end if constLen == 6 */
        else {                  /* (constLen == 5) */
            polynomes[0] = VCP2_POLY0_FOR_CONST5_RATE_1_3;
            polynomes[1] = VCP2_POLY1_FOR_CONST5_RATE_1_3;
            polynomes[2] = VCP2_POLY2_FOR_CONST5_RATE_1_3;
        }                       /* end if constLen == 5 */
    }                           
    else {                      /* if rate 1/4 */
        if (constLen == 9) {
            polynomes[0] = VCP2_POLY0_FOR_CONST9_RATE_1_4;
            polynomes[1] = VCP2_POLY1_FOR_CONST9_RATE_1_4;
            polynomes[2] = VCP2_POLY2_FOR_CONST9_RATE_1_4;
            polynomes[3] = VCP2_POLY3_FOR_CONST9_RATE_1_4;
        }                       /* end if constLen == 9 */
        else if (constLen == 8) {
            polynomes[0] = VCP2_POLY0_FOR_CONST8_RATE_1_4;
            polynomes[1] = VCP2_POLY1_FOR_CONST8_RATE_1_4;
            polynomes[2] = VCP2_POLY2_FOR_CONST8_RATE_1_4;
            polynomes[3] = VCP2_POLY3_FOR_CONST8_RATE_1_4;
        }                       /* end if constLen == 8 */
        else if (constLen == 7) {
            polynomes[0] = VCP2_POLY0_FOR_CONST7_RATE_1_4;
            polynomes[1] = VCP2_POLY1_FOR_CONST7_RATE_1_4;
            polynomes[2] = VCP2_POLY2_FOR_CONST7_RATE_1_4;
            polynomes[3] = VCP2_POLY3_FOR_CONST7_RATE_1_4;
        }                       /* end if constLen == 7 */
        else if (constLen == 6) {
            polynomes[0] = VCP2_POLY0_FOR_CONST6_RATE_1_4;
            polynomes[1] = VCP2_POLY1_FOR_CONST6_RATE_1_4;
            polynomes[2] = VCP2_POLY2_FOR_CONST6_RATE_1_4;
            polynomes[3] = VCP2_POLY3_FOR_CONST6_RATE_1_4;
        }                       /* end if constLen == 6 */
        else {                  /* (constLen == 5) */
            polynomes[0] = VCP2_POLY0_FOR_CONST5_RATE_1_4;
            polynomes[1] = VCP2_POLY1_FOR_CONST5_RATE_1_4;
            polynomes[2] = VCP2_POLY2_FOR_CONST5_RATE_1_4;
            polynomes[3] = VCP2_POLY3_FOR_CONST5_RATE_1_4;
        }                       /* end if constLen == 5 */
    }                           

    /* zero out any unneeded polynomials, based on the data rate */
    for (i = VCP2_RATE_1_4; i > rate; i--) {
        polynomes[i - 1] = 0;
    }
    /* the polynome sent to the coprocessor has always the most
       significant bit masked */
    mask = 0x00FF;

    /* Polynomes coefficients */
    poly0 = polynomes[0] & mask;
    poly1 = polynomes[1] & mask;
    poly2 = polynomes[2] & mask;
    poly3 = polynomes[3] & mask;


    /* Calculation of reliability length */
    /* Identify Fmax for a given constraint length and decision type */
    if (decision == VCP2_DECISION_HARD) {
        if (constLen == 9) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_HD_CONST9;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_HD_CONST9;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_HD_CONST9;
        }                       /* end if constLen == 9 */
        if (constLen == 8) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_HD_CONST8;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_HD_CONST8;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_HD_CONST8;
        }                       /* end if constLen == 8 */
        if (constLen == 7) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_HD_CONST7;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_HD_CONST7;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_HD_CONST7;
        }                       /* end if constLen == 7 */
        if (constLen == 6) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_HD_CONST6;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_HD_CONST6;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_HD_CONST6;
        }                       /* end if constLen == 6 */
        if (constLen == 5) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_HD_CONST5;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_HD_CONST5;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_HD_CONST5;
        }                       /* end if constLen == 5 */
    }                           /* end if hard decision */
    else {                      /* if soft decision */
        if (constLen == 9) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_SD_CONST9;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_SD_CONST9;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_SD_CONST9;
        }                       /* end if constLen == 9 */
        if (constLen == 8) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_SD_CONST8;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_SD_CONST8;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_SD_CONST8;
        }                       /* end if constLen == 8 */
        if (constLen == 7) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_SD_CONST7;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_SD_CONST7;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_SD_CONST7;
        }                       /* end if constLen == 7 */
        if (constLen == 6) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_SD_CONST6;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_SD_CONST6;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_SD_CONST6;
        }                       /* end if constLen == 6 */
        if (constLen == 5) {
            fMax = VCP2_MAX_FRAME_SIZE_FOR_SD_CONST5;
            rcMax = VCP2_MAX_RELLEN_CONV_DIST_FOR_SD_CONST5;
            multiple = VCP2_TRL_STAGES_IN_2_PASSES_FOR_SD_CONST5;
        }                       /* end if constLen == 5 */
    }                           /* end if soft decision */

    /* If mixed or convergent traceback mode */
    if ((tbConvrgMode == TRUE) || (frameLen > fMax)) {
        if (perf == VCP2_SPEED_MOST_CRITICAL) {  /* Speed most critical */
            convMultiplier = 3;                 /* convMultiplier
			                                       values  - as
			                                       specified in the
			                                       'TPC2_VCP2_API_
			                                       Requirements_v1.4'
			                                       spread sheet */
        }
        else if (perf == VCP2_PERF_CRITICAL) {   /* Performance critical */
            if (constLen == 9) {
                convMultiplier = 6;
            }
            else {
                convMultiplier = 9;
            }

        }
        else if (perf == VCP2_PERF_MOST_CRITICAL) {
	              /* Performance most critical */
            if (constLen == 9) {
                convMultiplier = 6;
            }
            else if (constLen == 8) {
                convMultiplier = 12;
            }
            else {
                convMultiplier = 18;
            }
        }
        else {                  /* for speed critical and default case */

            if (constLen == 9) {
                convMultiplier = 3;
            }
            else {
                convMultiplier = 6;
            }
        }
		/* Calculation of Reliability length and Convergence distance */
        convDist = convMultiplier * (constLen - 1);
        nsw = VCP2_normalCeil (frameLen, rcMax - convDist);
        relLen = multiple * VCP2_normalCeil (frameLen, nsw * multiple);

        if ((decision == VCP2_DECISION_SOFT) & (constLen < 7)) {
            while (relLen > VCP2_REL_LEN_LIMIT_FOR_A_SILDING_WINDOW) {
                nsw++;
                relLen = multiple * VCP2_normalCeil (frameLen, nsw * multiple);
            }
        }
    }                          /* end if mixed or convergent traceback modes */


    /* Trace back mode */
    if (tbConvrgMode == TRUE) {
        traceBack = VCP2_TRACEBACK_CONVERGENT;
    }
    else {
        if (frameLen > fMax)
            traceBack = VCP2_TRACEBACK_MIXED;
        else
            traceBack = VCP2_TRACEBACK_TAILED;
    }


    /* Calculation of branch metric buffer length */
    /* Number of branch metrics per trellis stage */
    if (rate == VCP2_RATE_1_4) {
        bmNum = VCP2_BM_PER_TRL_FOR_RATE_1_4;
        bmBuffLen = VCP2_SYMX_FOR_128_BYTE_PER_RXEVENT_RATE_1_4;
    }
    else if (rate == VCP2_RATE_1_3) {
        bmNum = VCP2_BM_PER_TRL_FOR_RATE_1_3;
        bmBuffLen = VCP2_SYMX_FOR_128_BYTE_PER_RXEVENT_RATE_1_3;
    }
    else if (rate == VCP2_RATE_1_2) {
        bmNum = VCP2_BM_PER_TRL_FOR_RATE_1_2;
        bmBuffLen = VCP2_SYMX_FOR_128_BYTE_PER_RXEVENT_RATE_1_2;
    }

    /* Maximum number of branch metrics */
    if (traceBack == VCP2_TRACEBACK_CONVERGENT)
        bmCnt = bmNum * (frameLen + convDist);
    else
        bmCnt = bmNum * (frameLen + constLen - 1);
    /* Number of frame transfers */
    numBmFrames = VCP2_normalCeil (bmCnt, VCP2_BYTE_COUNT_PER_RXEVENT);


    /* Calculation of Decision buffer length */
    if (decision == VCP2_DECISION_HARD) {
        if (frameLen > VCP2_MAX_HD_IN_OUTFIFO)
            decBuffLen = (VCP2_OUTFIFO_WORDS) - 1;
        else {
			decBuffLen = VCP2_normalCeil (frameLen, VCP2_HD_PER_OUTFIFO_WORD)
			               - 1;
            numDecFrames = 1;  /* Not being used here */
        }
    }
    else {
        if (frameLen > VCP2_MAX_SD_IN_OUTFIFO)
            decBuffLen = (VCP2_OUTFIFO_WORDS) - 1;
        else {
			decBuffLen = VCP2_normalCeil (frameLen, VCP2_SD_PER_OUTFIFO_WORD)
			   - 1;
            numDecFrames = 1;  /* Not being used here */
        }
    }

    pConfigParams->rate = rate;
    pConfigParams->constLen = constLen;
    pConfigParams->poly0 = poly0;
    pConfigParams->poly1 = poly1;
    pConfigParams->poly2 = poly2;
    pConfigParams->poly3 = poly3;
    pConfigParams->yamTh = yamTh;
    pConfigParams->frameLen = frameLen;
    pConfigParams->relLen = relLen;
    pConfigParams->convDist = convDist;
    pConfigParams->traceBackEn = tailBitEnable;
    pConfigParams->traceBackIndex = traceBackIndex;
    pConfigParams->outOrder = outOrder;
    pConfigParams->maxSm = maxSm;
    pConfigParams->minSm = minSm;
    pConfigParams->stateNum = stateNum;
    pConfigParams->bmBuffLen = bmBuffLen;
    pConfigParams->decBuffLen = decBuffLen;
    pConfigParams->traceBack = traceBack;
    pConfigParams->readFlag = readFlag;
    pConfigParams->decision = decision;
    pConfigParams->numBmFrames = numBmFrames;
    pConfigParams->numDecFrames = numDecFrames;

}

