/*  ===========================================================================
 *   Copyright (c) Texas Instruments Inc 2006
 *
 *   Use of this software is controlled by the terms and conditions found in
 *   the license agreement under which this software has been supplied.
 *   ==========================================================================
 */


/** ===========================================================================
 * VCP2_TCP2_EDMA.h
 * @desc EDMA configuration for VCP2/TCP2 channel density test.
 * ============================================================================
 */
/* ============================================================================
 *  Revision History
 *  ===============
 *  11-July-2006   Brighton     File Created.
 * ============================================================================
 */


#include <csl_edma3.h>
#include <csl_tcp2.h>
#include <csl_chip.h>
#include <soc.h>
#include <stdio.h>
#include <c6x.h>
#include "TCP2_Channel_Density.h"
#include "VCP2_Channel_Density.h"
#include "VCP2_TCP2_EDMA.h"

/* EDMA global declarations */
CSL_Edma3Context context;
CSL_Edma3ChannelObj VCP2TxChObj, VCP2RxChObj, TCP2TxChObj, TCP2RxChObj;
CSL_Edma3ChannelHandle hVCP2TxChannel, hVCP2RxChannel, hTCP2TxChannel, hTCP2RxChannel;
CSL_Edma3ChannelAttr VCP2TxChParam, VCP2RxChParam, TCP2TxChParam, TCP2RxChParam;
CSL_Edma3ChannelErr chErrClear;
CSL_Edma3ccRegs  * EDMA3CC_Regs= (CSL_Edma3ccRegs  *)CSL_EDMA3CC_0_REGS;


void Init_VCP2_TCP2_EDMA()
{
	Init_EDMA_PaRAM();
	Config_VCP2_TCP2_Edma();
}


 /* ============================================================================
 *   @func   Init_EDMA_PaRAM
 *
 *   @desc
 *     This function initialize the PaRAM for VCP2/TCP2
 * ============================================================================
*/
void Init_EDMA_PaRAM()
{
	Uint32 i;
	Uint32 * uipPaRAM= (Uint32 *)&(EDMA3CC_Regs->PARAMSET[DUMMY_PARAM_NUM]);
	for(i=0; i<8; i++) 	//Clear DUMMY PARAM
		*uipPaRAM++=0;
	//set CCNT to make a DUMMY PaRAM
	EDMA3CC_Regs->PARAMSET[DUMMY_PARAM_NUM].CCNT=1; 
	//Link to itself
	EDMA3CC_Regs->PARAMSET[DUMMY_PARAM_NUM].LINK_BCNTRLD=
		((Uint32)EDMA3CC_Regs->PARAMSET+(DUMMY_PARAM_NUM<<5))&0xffff;; 

	uipPaRAM= (Uint32 *)&(EDMA3CC_Regs->PARAMSET[CSL_EDMA3_CHA_VCP2XEVT]);
	for(i=0; i<8; i++) 	//Clear VCP2 TX channel
		*uipPaRAM++=0;

	uipPaRAM= (Uint32 *)&(EDMA3CC_Regs->PARAMSET[CSL_EDMA3_CHA_VCP2REVT]);
	for(i=0; i<8; i++) 	//Clear VCP2 RX channel
		*uipPaRAM++=0;

	uipPaRAM= (Uint32 *)&(EDMA3CC_Regs->PARAMSET[VCP2_PARAM_OFFSET]);
	for(i=0; i<VCP2_PARAM_LEN*8; i++) 	
		*uipPaRAM++=0; 	//clear PaRAM for VCP2

	uipPaRAM= (Uint32 *)&(EDMA3CC_Regs->PARAMSET[CSL_EDMA3_CHA_TCP2XEVT]);
	for(i=0; i<8; i++) 	//Clear TCP2 TX channel
		*uipPaRAM++=0;

	uipPaRAM= (Uint32 *)&(EDMA3CC_Regs->PARAMSET[CSL_EDMA3_CHA_TCP2REVT]);
	for(i=0; i<8; i++) 	//Clear TCP2 RX channel
		*uipPaRAM++=0;

	uipPaRAM= (Uint32 *)&(EDMA3CC_Regs->PARAMSET[TCP2_PARAM_OFFSET]);
	for(i=0; i<TCP2_PARAM_LEN*8; i++) 	
		*uipPaRAM++=0; 	//clear PaRAM TCP2

}
/*
 * ============================================================================
 *   @func   vcp2_configEdma
 *   @desc
 *      Configures EDMA channels for TCP2/VCP2
 * ============================================================================
 */
void Config_VCP2_TCP2_Edma()
{
	CSL_Status chStatus, chStatus1;

	/* EDMA Initialization */
	CSL_edma3Init (&context);

	CSL_edma3Open (NULL, CSL_EDMA3, NULL, NULL);

	/* Channel Configuration for VCP2XEVT event */
	/* Channel Open */
	VCP2TxChParam.regionNum = CSL_EDMA3_REGION_GLOBAL;
	VCP2TxChParam.chaNum = CSL_EDMA3_CHA_VCP2XEVT;
	hVCP2TxChannel = CSL_edma3ChannelOpen (&VCP2TxChObj, CSL_EDMA3, &VCP2TxChParam, &chStatus);
	if ((chStatus != CSL_SOK) || (hVCP2TxChannel == NULL)) {
	    printf ("Error in EDMA channel open function\n");
	    return;
	}
	/* Channel Setup */
	if (CSL_SOK != CSL_edma3HwChannelSetupParam (hVCP2TxChannel,
	        CSL_EDMA3_CHA_VCP2XEVT/* PaRAM entry */ )) {
	    printf ("Error in Tx EDMA channel setup\n");
	    return;
	}
	if (CSL_SOK != CSL_edma3HwChannelSetupQue (hVCP2TxChannel,
	   CSL_EDMA3_QUE_0)) {
	    printf ("Error in Tx EDMA channel setup\n");
	    return;
	}

	/* Channel Configuration for VCP2REVT event */
	/* Channel Open */
	VCP2RxChParam.regionNum = CSL_EDMA3_REGION_GLOBAL;
	VCP2RxChParam.chaNum = CSL_EDMA3_CHA_VCP2REVT;
	hVCP2RxChannel =
	    CSL_edma3ChannelOpen (&VCP2RxChObj, CSL_EDMA3, &VCP2RxChParam, &chStatus1);
	if ((chStatus1 != CSL_SOK) | (hVCP2RxChannel == NULL)) {
	    printf ("Error in EDMA channel open function\n");
	    return;
	}
	/* Channel Setup */
	if (CSL_SOK != CSL_edma3HwChannelSetupParam (hVCP2RxChannel,
	        CSL_EDMA3_CHA_VCP2REVT /* PaRAM entry */ )) {
	    printf ("Error in Rx EDMA channel setup\n");
	    return;
	}
	if (CSL_SOK != CSL_edma3HwChannelSetupQue (hVCP2RxChannel,
	    CSL_EDMA3_QUE_0)) {
	    printf ("Error in Rx EDMA channel setup\n");
	    return;
	}

		/* Channel Configuration for TCP2XEVT event */
	/* Channel Open */
	TCP2TxChParam.regionNum = CSL_EDMA3_REGION_GLOBAL;
	TCP2TxChParam.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
	hTCP2TxChannel = CSL_edma3ChannelOpen (&TCP2TxChObj, CSL_EDMA3, &TCP2TxChParam, &chStatus);
	if ((chStatus != CSL_SOK) || (hTCP2TxChannel == NULL)) {
	    printf ("Error in EDMA channel open function\n");
	    return;
	}
	/* Channel Setup */
	if (CSL_SOK != CSL_edma3HwChannelSetupParam (hTCP2TxChannel,
	        CSL_EDMA3_CHA_TCP2XEVT/* PaRAM entry */ )) {
	    printf ("Error in Tx EDMA channel setup\n");
	    return;
	}
	if (CSL_SOK != CSL_edma3HwChannelSetupQue (hTCP2TxChannel,
	   CSL_EDMA3_QUE_0)) {
	    printf ("Error in Tx EDMA channel setup\n");
	    return;
	}

	/* Channel Configuration for TCP2REVT event */
	/* Channel Open */
	TCP2RxChParam.regionNum = CSL_EDMA3_REGION_GLOBAL;
	TCP2RxChParam.chaNum = CSL_EDMA3_CHA_TCP2REVT;
	hTCP2RxChannel =
	    CSL_edma3ChannelOpen (&TCP2RxChObj, CSL_EDMA3, &TCP2RxChParam, &chStatus1);
	if ((chStatus1 != CSL_SOK) | (hTCP2RxChannel == NULL)) {
	    printf ("Error in EDMA channel open function\n");
	    return;
	}
	/* Channel Setup */
	if (CSL_SOK != CSL_edma3HwChannelSetupParam (hTCP2RxChannel,
	        CSL_EDMA3_CHA_TCP2REVT /* PaRAM entry */ )) {
	    printf ("Error in Rx EDMA channel setup\n");
	    return;
	}
	if (CSL_SOK != CSL_edma3HwChannelSetupQue (hTCP2RxChannel,
	    CSL_EDMA3_QUE_0)) {
	    printf ("Error in Rx EDMA channel setup\n");
	    return;
	}
}

/*
 * ============================================================================
 *   @func   vcp2SubmitEdma
 *
 *   @desc
 *     This function Setup EDMA for VCP2
 *
 *  @arg  
 *      vcp2_Edma3_Config: EDMA parameters for VCP2
 *
 *  @return
 *      None
 * ============================================================================
*/
void vcp2SubmitEdma(VCP2_Edma3_Config * vcp2_Edma3_Config)
{
	Uint32 i, uiPaRAM_Index, uiLastRxIndex, uiLastTxIndex;
	Uint32 uiGlobalL2Offset= 0x10000000+0x01000000*CSL_chipReadReg (CSL_CHIP_DNUM);

	//Since EDMA can only access global address, if data are in L2, here convert the local address to global address
#if 0
	if(vcp2_Edma3_Config->inputConfigAddr<0x10000000)
		vcp2_Edma3_Config->inputConfigAddr += uiGlobalL2Offset;		
	if(vcp2_Edma3_Config->branchMetricsAddr< 0x10000000)
		vcp2_Edma3_Config->branchMetricsAddr += uiGlobalL2Offset;		
	if(vcp2_Edma3_Config->hardDecisionAddr< 0x10000000)
		vcp2_Edma3_Config->hardDecisionAddr += uiGlobalL2Offset;		
	if(vcp2_Edma3_Config->outParmAddr< 0x10000000)
		vcp2_Edma3_Config->outParmAddr += uiGlobalL2Offset;		
#endif	
	/* clear the EDMA error registers */
	chErrClear.missed = TRUE;
	chErrClear.secEvt = TRUE;
	CSL_edma3HwChannelControl (hVCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_DISABLE, NULL);
	CSL_edma3HwChannelControl (hVCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_DISABLE, NULL);
	CSL_edma3HwChannelControl (hVCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_CLEARERR, &chErrClear);
	CSL_edma3HwChannelControl (hVCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_CLEARERR, &chErrClear);
	CSL_edma3HwChannelControl (hVCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_CLEAR, NULL);
	CSL_edma3HwChannelControl (hVCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_CLEAR, NULL);
	//Clear interrupt
	EDMA3CC_Regs->ICR= (1<<CSL_EDMA3_CHA_VCP2REVT);

	uiPaRAM_Index=VCP2_PARAM_OFFSET; 	

	//Config first user channel
	vcp2_Edma3_Config->inputConfigPaRAMIndex= CSL_EDMA3_CHA_VCP2XEVT;
	vcp2_Edma3_Config->branchMetricsPaRAMIndex= uiPaRAM_Index++; 
	vcp2_Edma3_Config->decisionPaRAMIndex= CSL_EDMA3_CHA_VCP2REVT;
	if(vcp2_Edma3_Config->outParmFlag)
		vcp2_Edma3_Config->outPaRAMIndex= uiPaRAM_Index++;
	vcp2_Edma3_Config->nextInputPaRAMIndex= uiPaRAM_Index;
	vcp2_Edma3_Config->nextDecisionPaRAMIndex= uiPaRAM_Index+2;		
	//Config EDMA for one user channel
	Config_VCP2_EDMA3_PaRAM(vcp2_Edma3_Config);
	//Different hard decision buffer for different user channel, 
	//the purpose is to make sure every user channel are correct decoded
	vcp2_Edma3_Config->hardDecisionAddr+= ((MAX_VCP2_FRAME_LEN+7)/8+7)/8*8;

	//Config the rest user channels
	for(i=1; i< VCP2_CHANNELS_PER_TIME; i++) 
	{
		vcp2_Edma3_Config->inputConfigPaRAMIndex= uiPaRAM_Index++;
		vcp2_Edma3_Config->branchMetricsPaRAMIndex= uiPaRAM_Index++; 
		vcp2_Edma3_Config->decisionPaRAMIndex= uiPaRAM_Index++;
		if(vcp2_Edma3_Config->outParmFlag)
			vcp2_Edma3_Config->outPaRAMIndex= uiPaRAM_Index++;
		vcp2_Edma3_Config->nextInputPaRAMIndex= uiPaRAM_Index;
		vcp2_Edma3_Config->nextDecisionPaRAMIndex= uiPaRAM_Index+2;		
		//Config EDMA for one user channel
		Config_VCP2_EDMA3_PaRAM(vcp2_Edma3_Config);
		vcp2_Edma3_Config->hardDecisionAddr+= ((MAX_VCP2_FRAME_LEN+7)/8+7)/8*8;
	}

	//Config the link and interrupt for last user channel
	if(vcp2_Edma3_Config->outParmFlag||VCP2_CHANNELS_PER_TIME>1)
		uiLastRxIndex= uiPaRAM_Index-1;
	else
		uiLastRxIndex= CSL_EDMA3_CHA_VCP2REVT;
	//linklast RX edma to NULL
	EDMA3CC_Regs->PARAMSET[uiLastRxIndex].LINK_BCNTRLD= CSL_EDMA3_LINK_NULL;  
	//last RX edma generate interrupt
	EDMA3CC_Regs->PARAMSET[uiLastRxIndex].OPT|= (1<<CSL_EDMA3CC_OPT_TCINTEN_SHIFT) ;  

	if(vcp2_Edma3_Config->outParmFlag)
	{
		if((VCP2_CHANNELS_PER_TIME>1))
		 	uiLastTxIndex= uiPaRAM_Index-3;
		else
		 	uiLastTxIndex= uiPaRAM_Index-2;
	}
	else		
	{
		if((VCP2_CHANNELS_PER_TIME>1))
		 	uiLastTxIndex= uiPaRAM_Index-2;
		else
		 	uiLastTxIndex= uiPaRAM_Index-1;
	}
	//Link last TX edma to DUMMY
	EDMA3CC_Regs->PARAMSET[uiLastTxIndex].LINK_BCNTRLD=
		( (Uint32)EDMA3CC_Regs->PARAMSET+(DUMMY_PARAM_NUM<<5))&0xffff;		

	/* Enable Channel */
	CSL_edma3HwChannelControl (hVCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_ENABLE, NULL);
	CSL_edma3HwChannelControl (hVCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_ENABLE, NULL);
}

/*
 * ============================================================================
 *   @func   Config_VCP2_EDMA3_PaRAM
 *
 *   @desc
 *     This function Setup EDMA PaRAM for one VCP2 processing task.
 *
 *  @arg  
 *      configration paramter for EDMA3 of VCP2
 *
 *  @return
 *      None
 * ============================================================================
*/
void Config_VCP2_EDMA3_PaRAM(VCP2_Edma3_Config * vcp2_Edma3_Config)
{
	Uint32 inputConfigPaRAMIndex, branchMetricsPaRAMIndex; 
	Uint32 decisionPaRAMIndex, outPaRAMIndex;
	Uint32 uiPaRAMBaseAddr= (Uint32)EDMA3CC_Regs->PARAMSET;
	
	inputConfigPaRAMIndex= vcp2_Edma3_Config->inputConfigPaRAMIndex;
	branchMetricsPaRAMIndex= vcp2_Edma3_Config->branchMetricsPaRAMIndex;
 	decisionPaRAMIndex= vcp2_Edma3_Config->decisionPaRAMIndex;
 	outPaRAMIndex= vcp2_Edma3_Config->outPaRAMIndex;

	//Setup Input Configuration PaRAM
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].OPT = VCP2_INPUT_CONFIG_EDMA_OPT;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].SRC= vcp2_Edma3_Config->inputConfigAddr;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].A_B_CNT= 
		CSL_EDMA3_CNT_MAKE (VCP2_NUM_IC<<2, 1);
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].DST=(Uint32) &hVcp2Vbus->VCPIC0;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].SRC_DST_BIDX= 0;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].LINK_BCNTRLD= 
		(uiPaRAMBaseAddr+(branchMetricsPaRAMIndex<<5))&0xffff;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].CCNT= 1;

 	//Setup  Branch Metrics PaRAM
	EDMA3CC_Regs->PARAMSET[branchMetricsPaRAMIndex].OPT = VCP2_BRANCH_METRICS_EDMA_OPT;
	EDMA3CC_Regs->PARAMSET[branchMetricsPaRAMIndex].SRC= vcp2_Edma3_Config->branchMetricsAddr;
	EDMA3CC_Regs->PARAMSET[branchMetricsPaRAMIndex].A_B_CNT= 
		CSL_EDMA3_CNT_MAKE (vcp2_Edma3_Config->bmBuffLen, vcp2_Edma3_Config->numBmFrames);
	EDMA3CC_Regs->PARAMSET[branchMetricsPaRAMIndex].DST=  (Uint32) & (hVcp2Vbus->VCPWBM);
	EDMA3CC_Regs->PARAMSET[branchMetricsPaRAMIndex].SRC_DST_BIDX= 
		CSL_EDMA3_BIDX_MAKE(vcp2_Edma3_Config->bmBuffLen, 0) ;
	EDMA3CC_Regs->PARAMSET[branchMetricsPaRAMIndex].LINK_BCNTRLD= 
		(uiPaRAMBaseAddr+(vcp2_Edma3_Config->nextInputPaRAMIndex<<5))&0xffff;
	EDMA3CC_Regs->PARAMSET[branchMetricsPaRAMIndex].CCNT= 1;

 	//Setup Hard Decision PaRAM
 	if(vcp2_Edma3_Config->traceBack== VCP2_TRACEBACK_TAILED)
 	{	//the hard decision is in reverse order for tailed mode, EDMA is configure to swap the HD to normal order
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].OPT = VCP2_TAILED_HARD_DECISION_EDMA_OPT;
		//vcp2_Edma3_Config->numDecFrames must less than 1, becasue frame length must less 2048 for tailed mode
		//So, only need one SYNC_AB transfer for all data
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].A_B_CNT= 
			CSL_EDMA3_CNT_MAKE (8, vcp2_Edma3_Config->decBuffLen/8);
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].SRC_DST_BIDX= 
			CSL_EDMA3_BIDX_MAKE(0, 0xFFF8) ; //destination index is -8
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].DST= vcp2_Edma3_Config->hardDecisionAddr
			+vcp2_Edma3_Config->decBuffLen -8;          
 	}
 	else
 	{
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].OPT = VCP2_HARD_DECISION_EDMA_OPT;
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].A_B_CNT= 
			CSL_EDMA3_CNT_MAKE (vcp2_Edma3_Config->decBuffLen, vcp2_Edma3_Config->numDecFrames);
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].SRC_DST_BIDX= 
			CSL_EDMA3_BIDX_MAKE(0, vcp2_Edma3_Config->decBuffLen) ;
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].DST=  vcp2_Edma3_Config->hardDecisionAddr;          /* Destination */
 	}
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].SRC=  (Uint32)&(hVcp2Vbus->VCPRDECS);
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].CCNT= 1;

	if(vcp2_Edma3_Config->outParmFlag) 	//if Read output parameter
	{
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(outPaRAMIndex<<5))&0xffff;
	 	//Setup Output Parameter PaRAM
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].OPT = VCP2_OUTPUT_PARAM_EDMA_OPT;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].SRC= (Uint32) &hVcp2Vbus->VCPOUT0;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].A_B_CNT= 
			CSL_EDMA3_CNT_MAKE (VCP2_NUM_OP<<2, 1);
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].DST=  vcp2_Edma3_Config->outParmAddr;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].SRC_DST_BIDX= 0;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(vcp2_Edma3_Config->nextDecisionPaRAMIndex<<5))&0xffff;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].CCNT= 1;
	}
	else
	{
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(vcp2_Edma3_Config->nextDecisionPaRAMIndex<<5))&0xffff;
	}
}


#if 1
/****************************************************************************\
 * tcp2SubmitEdma: The EDMA is programmed to service the user channels.*
 for this test, TCP2_CHANNELS_PER_TIME user channels are configured, and multiple user channel 
 use same input data,  but hard decision output buffer are different, only if  hard decision in all output 
 buffers are same as expected, then all user channel are correct decoded
The purpose of these test method are:
1, verify multiple user channels chain feature of TCP2
2, same data are process multiple times, then we can get more accurate average
\****************************************************************************/
void tcp2SubmitEdma(TCP2_Edma3_Config * tcp2_Edma3_Config)
{
	Uint32 i, uiPaRAM_Index, uiLastRxIndex, uiLastTxIndex;
	Uint32 uiGlobalL2Offset= 0x10000000+0x01000000*CSL_chipReadReg (CSL_CHIP_DNUM);

#if 0
	//Since EDMA can only access global address, if data are in L2, here convert the local address to global address
	if(tcp2_Edma3_Config->inputConfigAddr<0x10000000)
		tcp2_Edma3_Config->inputConfigAddr += uiGlobalL2Offset;		
	if(tcp2_Edma3_Config->sysParDataAddr< 0x10000000)
		tcp2_Edma3_Config->sysParDataAddr += uiGlobalL2Offset;		
	if(tcp2_Edma3_Config->interleaverTableAddr< 0x10000000)
		tcp2_Edma3_Config->interleaverTableAddr += uiGlobalL2Offset;		
	if(tcp2_Edma3_Config->hardDecisionAddr< 0x10000000)
		tcp2_Edma3_Config->hardDecisionAddr += uiGlobalL2Offset;		
	if(tcp2_Edma3_Config->outParmAddr< 0x10000000)
		tcp2_Edma3_Config->outParmAddr += uiGlobalL2Offset;		
#endif	
	/* clear the EDMA error registers */
	chErrClear.missed = TRUE;
	chErrClear.secEvt = TRUE;
	CSL_edma3HwChannelControl (hTCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_DISABLE, NULL);
	CSL_edma3HwChannelControl (hTCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_DISABLE, NULL);
	CSL_edma3HwChannelControl (hTCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_CLEARERR, &chErrClear);
	CSL_edma3HwChannelControl (hTCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_CLEARERR, &chErrClear);
	CSL_edma3HwChannelControl (hTCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_CLEAR, NULL);
	CSL_edma3HwChannelControl (hTCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_CLEAR, NULL);
	//Clear interrupt
	EDMA3CC_Regs->ICR= (1<<CSL_EDMA3_CHA_TCP2REVT);

	uiPaRAM_Index=TCP2_PARAM_OFFSET; 	

	//Config first user channel
	tcp2_Edma3_Config->inputConfigPaRAMIndex= CSL_EDMA3_CHA_TCP2XEVT;
	tcp2_Edma3_Config->sysParPaRAMIndex= uiPaRAM_Index++; 
	if(tcp2_Edma3_Config->intFlag)
		tcp2_Edma3_Config->interleavePaRAMIndex= uiPaRAM_Index++; 
	tcp2_Edma3_Config->decisionPaRAMIndex= CSL_EDMA3_CHA_TCP2REVT;
	if(tcp2_Edma3_Config->outParmFlag)
		tcp2_Edma3_Config->outPaRAMIndex= uiPaRAM_Index++;
	tcp2_Edma3_Config->nextInputPaRAMIndex= uiPaRAM_Index;
	if(tcp2_Edma3_Config->intFlag)
		tcp2_Edma3_Config->nextDecisionPaRAMIndex= uiPaRAM_Index+3;		
	else	
		tcp2_Edma3_Config->nextDecisionPaRAMIndex= uiPaRAM_Index+2;		
	//Config EDMA for one user channel
	Config_TCP2_EDMA3_PaRAM(tcp2_Edma3_Config);
	//Different hard decision buffer for different user channel, 
	//the purpose is to make sure every user channel are correct decoded
	tcp2_Edma3_Config->hardDecisionAddr+= ((MAX_TCP2_FRAME_LEN+7)/8+7)/8*8;

	//Config the rest user channels
	for(i=1; i< TCP2_CHANNELS_PER_TIME; i++) 
	{
		tcp2_Edma3_Config->inputConfigPaRAMIndex= uiPaRAM_Index++;
		tcp2_Edma3_Config->sysParPaRAMIndex= uiPaRAM_Index++; 
		if(tcp2_Edma3_Config->intFlag)
			tcp2_Edma3_Config->interleavePaRAMIndex= uiPaRAM_Index++; 
		tcp2_Edma3_Config->decisionPaRAMIndex= uiPaRAM_Index++;
		if(tcp2_Edma3_Config->outParmFlag)
			tcp2_Edma3_Config->outPaRAMIndex= uiPaRAM_Index++;
		tcp2_Edma3_Config->nextInputPaRAMIndex= uiPaRAM_Index;
		if(tcp2_Edma3_Config->intFlag)
			tcp2_Edma3_Config->nextDecisionPaRAMIndex= uiPaRAM_Index+3;		
		else	
			tcp2_Edma3_Config->nextDecisionPaRAMIndex= uiPaRAM_Index+2;		
		//Config EDMA for one user channel
		Config_TCP2_EDMA3_PaRAM(tcp2_Edma3_Config);
		tcp2_Edma3_Config->hardDecisionAddr+= ((MAX_TCP2_FRAME_LEN+7)/8+7)/8*8;
	}

	//Config the link and interrupt for last user channel
	if(tcp2_Edma3_Config->outParmFlag||TCP2_CHANNELS_PER_TIME>1)
		uiLastRxIndex= uiPaRAM_Index-1;
	else
		uiLastRxIndex= CSL_EDMA3_CHA_TCP2REVT;
	//linklast RX edma to NULL
	EDMA3CC_Regs->PARAMSET[uiLastRxIndex].LINK_BCNTRLD= CSL_EDMA3_LINK_NULL;  
	//last RX edma generate interrupt
	EDMA3CC_Regs->PARAMSET[uiLastRxIndex].OPT|= (1<<CSL_EDMA3CC_OPT_TCINTEN_SHIFT) ;  

	if(tcp2_Edma3_Config->outParmFlag)
	{
		if((TCP2_CHANNELS_PER_TIME>1))
		 	uiLastTxIndex= uiPaRAM_Index-3;
		else
		 	uiLastTxIndex= uiPaRAM_Index-2;
	}
	else		
	{
		if((TCP2_CHANNELS_PER_TIME>1))
		 	uiLastTxIndex= uiPaRAM_Index-2;
		else
		 	uiLastTxIndex= uiPaRAM_Index-1;
	}
	//Link last TX edma to DUMMY
	EDMA3CC_Regs->PARAMSET[uiLastTxIndex].LINK_BCNTRLD=
		( (Uint32)EDMA3CC_Regs->PARAMSET+(DUMMY_PARAM_NUM<<5))&0xffff;		

	/* Enable Channel */
	CSL_edma3HwChannelControl (hTCP2TxChannel, CSL_EDMA3_CMD_CHANNEL_ENABLE, NULL);
	CSL_edma3HwChannelControl (hTCP2RxChannel, CSL_EDMA3_CMD_CHANNEL_ENABLE, NULL);
}

/*
 * ============================================================================
 *   @func   Config_TCP2_EDMA3_PaRAM
 *
 *   @desc
 *     This function Setup EDMA PaRAM for one TCP2 processing task.
 *
 *  @arg  
 *      configration paramter for EDMA3 of TCP2
 *
 *  @return
 *      None
 * ============================================================================
*/
void Config_TCP2_EDMA3_PaRAM(TCP2_Edma3_Config * tcp2_Edma3_Config)
{
	Uint32 inputConfigPaRAMIndex, sysParPaRAMIndex, interleavePaRAMIndex; 
	Uint32 decisionPaRAMIndex, outPaRAMIndex;
	Uint32 uiPaRAMBaseAddr= (Uint32)EDMA3CC_Regs->PARAMSET;
	
	inputConfigPaRAMIndex= tcp2_Edma3_Config->inputConfigPaRAMIndex;
	sysParPaRAMIndex= tcp2_Edma3_Config->sysParPaRAMIndex;
 	interleavePaRAMIndex= tcp2_Edma3_Config->interleavePaRAMIndex;
 	decisionPaRAMIndex= tcp2_Edma3_Config->decisionPaRAMIndex;
 	outPaRAMIndex= tcp2_Edma3_Config->outPaRAMIndex;

	//Setup Input Configuration PaRAM
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].OPT = TCP2_INPUT_CONFIG_EDMA_OPT;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].SRC= tcp2_Edma3_Config->inputConfigAddr;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].A_B_CNT= 
		CSL_EDMA3_CNT_MAKE (TCP2_NUM_IC<<2, 1);
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].DST= TCP_ICMEM_ADDR;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].SRC_DST_BIDX= 0;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].LINK_BCNTRLD= 
		(uiPaRAMBaseAddr+(sysParPaRAMIndex<<5))&0xffff;
	EDMA3CC_Regs->PARAMSET[inputConfigPaRAMIndex].CCNT= 1;

 	//Setup  Systematics and Parities PaRAM
	EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].OPT = TCP2_SYSTEM_PARITY_EDMA_OPT;
	EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].SRC= tcp2_Edma3_Config->sysParDataAddr;
	if(tcp2_Edma3_Config->frameLength<(65536/4-1))		//ACNT can't be larger than 65535
		EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].A_B_CNT= 
			CSL_EDMA3_CNT_MAKE (((tcp2_Edma3_Config->frameLength+1)/2)*8, 1);
	else
		EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].A_B_CNT= 
		//ACNT is set to 64 to fully utilized the EDMA3 bandwidth, 
		//test shows larger ACNT can't improve performance further
			CSL_EDMA3_CNT_MAKE (64, (tcp2_Edma3_Config->frameLength+15)/16);
	EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].DST=  CSL_TCP2_X0_MEM;
	EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].SRC_DST_BIDX= 
		CSL_EDMA3_BIDX_MAKE(64, 64) ;
	EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].CCNT= 1;

	if(tcp2_Edma3_Config->intFlag)
	{
		EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(interleavePaRAMIndex<<5))&0xffff;

	 	//Setup Interleaver PaRAM
		EDMA3CC_Regs->PARAMSET[interleavePaRAMIndex].OPT = TCP2_INTERLEAVER_EDMA_OPT;
		EDMA3CC_Regs->PARAMSET[interleavePaRAMIndex].SRC= tcp2_Edma3_Config->interleaverTableAddr;
		EDMA3CC_Regs->PARAMSET[interleavePaRAMIndex].A_B_CNT= 
			CSL_EDMA3_CNT_MAKE (((tcp2_Edma3_Config->frameLength+6)/4)*8, 1);
		EDMA3CC_Regs->PARAMSET[interleavePaRAMIndex].DST=  CSL_TCP2_I0_MEM;
		EDMA3CC_Regs->PARAMSET[interleavePaRAMIndex].SRC_DST_BIDX= 0;
		EDMA3CC_Regs->PARAMSET[interleavePaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(tcp2_Edma3_Config->nextInputPaRAMIndex<<5))&0xffff;
		EDMA3CC_Regs->PARAMSET[interleavePaRAMIndex].CCNT= 1;
	}
	else
		EDMA3CC_Regs->PARAMSET[sysParPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(tcp2_Edma3_Config->nextInputPaRAMIndex<<5))&0xffff;
	
 	//Setup Hard Decision PaRAM
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].OPT = TCP2_HARD_DECISION_EDMA_OPT;
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].SRC= CSL_TCP2_O0_MEM;
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].A_B_CNT= 
		CSL_EDMA3_CNT_MAKE (((tcp2_Edma3_Config->frameLength+63)/64)*8, 1);
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].DST=  tcp2_Edma3_Config->hardDecisionAddr;          /* Destination */
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].SRC_DST_BIDX= 0;
	EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].CCNT= 1;

	if(tcp2_Edma3_Config->outParmFlag) 	//if Read output parameter
	{
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(outPaRAMIndex<<5))&0xffff;
	 	//Setup Output Parameter PaRAM
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].OPT = TCP2_OUTPUT_PARAM_EDMA_OPT;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].SRC=  TCP_OPMEM_ADDR;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].A_B_CNT= 
			CSL_EDMA3_CNT_MAKE (TCP2_NUM_OP<<2, 1);
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].DST=  tcp2_Edma3_Config->outParmAddr;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].SRC_DST_BIDX= 0;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(tcp2_Edma3_Config->nextDecisionPaRAMIndex<<5))&0xffff;
		EDMA3CC_Regs->PARAMSET[outPaRAMIndex].CCNT= 1;
	}
	else
	{
		EDMA3CC_Regs->PARAMSET[decisionPaRAMIndex].LINK_BCNTRLD= 
			(uiPaRAMBaseAddr+(tcp2_Edma3_Config->nextDecisionPaRAMIndex<<5))&0xffff;
	}
}

#endif