/*  ===========================================================================
 *   Copyright (c) Texas Instruments Inc 2006
 *
 *   Use of this software is controlled by the terms and conditions found in
 *   the license agreement under which this software has been supplied.
 *   ==========================================================================
 */


/** ===========================================================================
 * @desc  header file for EDMA configuration for VCP2/TCP2 channel density test.
 *
 * ============================================================================
 */
/* ============================================================================
 *  Revision History
 *  ===============
 *  11-July-2006   Brighton     File Created.
 * ============================================================================
 */
 
#ifndef _VCP2_TCP2_EDMA_H_
#define _VCP2_TCP2_EDMA_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <csl_edma3.h>
#include <tistdtypes.h>
#include <csl_tcp2.h>

#define  TCP_ICMEM_ADDR           	0x50000000u
#define  TCP_OPMEM_ADDR			    0x50000040u


/* DUMMY PARAM Number*/
#define DUMMY_PARAM_NUM  127

//Multiple PaRAM are used to support decoding multiple user channels one time 
/* start of VCP2 PARAM */
#define VCP2_PARAM_OFFSET  128

/* Length of TCP2 PARAM */
#define VCP2_PARAM_LEN  64

/*How many user channels to be proccessed every time start VCP2*/
//Max value =VCP2_PARAM_LEN/5, since every user channels need 4 PaRAM
#define VCP2_CHANNELS_PER_TIME  (VCP2_PARAM_LEN/4)

#define VCP2_BM_FIFO_SIZE 	128
#define VCP2_HD_FIFO_SIZE 	256


/* start of TCP2 PARAM */
#define TCP2_PARAM_OFFSET  192

/* Length of TCP2 PARAM */
#define TCP2_PARAM_LEN  64

/*How many user channels to be proccessed every time start TCP2*/
//Max value =TCP2_PARAM_LEN/5, since every user channels need 5 PaRAM
#define TCP2_CHANNELS_PER_TIME  (TCP2_PARAM_LEN/5)



//following is the default EDMA configration for VCP2
//EDMA option for VCP2 Input Configuration
#define  VCP2_INPUT_CONFIG_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	0,\
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_NONE,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_INCR\
)

//EDMA option for VCP2 Branch Metrics
#define  VCP2_BRANCH_METRICS_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_VCP2XEVT , \
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_64BIT,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_CONST,\
	CSL_EDMA3_ADDRMODE_INCR\
)

//EDMA option for VCP2 Hard Decision
#define  VCP2_HARD_DECISION_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_VCP2REVT , \
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_64BIT,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_CONST\
)

//EDMA option for VCP2 Hard Decision in tailed mode
#define  VCP2_TAILED_HARD_DECISION_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_VCP2REVT , \
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_64BIT,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_AB,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_CONST\
)
//EDMA option for VCP2 Output Parameter
#define  VCP2_OUTPUT_PARAM_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_VCP2REVT , \
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_NONE,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_INCR\
)


//following is the default EDMA configration for TCP2
//EDMA option for TCP2 Input Configuration
#define  TCP2_INPUT_CONFIG_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	0,\
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_NONE,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_INCR\
)

//EDMA option for TCP2 Systematics and Parities data 
#define  TCP2_SYSTEM_PARITY_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_TCP2XEVT,\
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_NONE,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_AB,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_INCR\
)

//EDMA option for TCP2 Interleaver table 
#define  TCP2_INTERLEAVER_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_TCP2XEVT,\
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_NONE,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_INCR\
)

//EDMA option for TCP2 Hard Decision
#define  TCP2_HARD_DECISION_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_TCP2REVT,\
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_NONE,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_INCR\
)

//EDMA option for TCP2 Output Parameter
#define  TCP2_OUTPUT_PARAM_EDMA_OPT CSL_EDMA3_OPT_MAKE(\
	CSL_EDMA3_ITCCH_DIS,\
	CSL_EDMA3_TCCH_DIS,\
	CSL_EDMA3_ITCINT_DIS,\
	CSL_EDMA3_TCINT_DIS,\
	CSL_EDMA3_CHA_TCP2REVT , \
	CSL_EDMA3_TCC_NORMAL,\
	CSL_EDMA3_FIFOWIDTH_NONE,\
	CSL_EDMA3_STATIC_DIS,\
	CSL_EDMA3_SYNC_A,\
	CSL_EDMA3_ADDRMODE_INCR,\
	CSL_EDMA3_ADDRMODE_INCR\
)


/* Coprocessor specifics */
#define VCP2_NUM_IC 6
#define VCP2_NUM_OP 2

typedef struct VCP2_Edma3_Config {
	Uint32 inputConfigAddr;
	Uint32 branchMetricsAddr;
	Uint32 hardDecisionAddr;
	Uint32 outParmAddr;
	Uint32 outParmFlag; /**< Output parameters read flag */

	/** Traceback mode */
	Uint8 traceBack;
	/** Branch metrics buffer length in input FIFO */
	Uint32 bmBuffLen;
	/** Decisions buffer length in output FIFO */
	Uint32 decBuffLen;
	/** Number of branch metric frames*/
	Uint32 numBmFrames;
	/** Number of decision frames */
	Uint32 numDecFrames;

	Uint32 inputConfigPaRAMIndex;
	Uint32 branchMetricsPaRAMIndex; 
	Uint32 decisionPaRAMIndex;
	Uint32 outPaRAMIndex;
	Uint32 nextInputPaRAMIndex;
	Uint32 nextDecisionPaRAMIndex;		
}VCP2_Edma3_Config ;

#define TCP2_NUM_IC 16
#define TCP2_NUM_OP 3

typedef struct TCP2_Edma3_Config {
	Uint32 inputConfigAddr;
	Uint32 sysParDataAddr;
	Uint32 frameLength;
	Uint32 interleaverTableAddr;
	Uint32 intFlag;     /**< Interleaver write flag     */
	Uint32 hardDecisionAddr;
	Uint32 outParmAddr;
	Uint32 outParmFlag; /**< Output parameters read flag */
	Uint32 inputConfigPaRAMIndex;
	Uint32 sysParPaRAMIndex; 
	Uint32 interleavePaRAMIndex; 
	Uint32 decisionPaRAMIndex;
	Uint32 outPaRAMIndex;
	Uint32 nextInputPaRAMIndex;
	Uint32 nextDecisionPaRAMIndex;		
}TCP2_Edma3_Config ;

extern CSL_Edma3ccRegs  * EDMA3CC_Regs;

void Init_VCP2_TCP2_EDMA();
void Init_EDMA_PaRAM();
void Config_VCP2_TCP2_Edma();
void tcp2SubmitEdma(TCP2_Edma3_Config * tcp2_Edma3_Config);
void Config_VCP2_EDMA3_PaRAM(VCP2_Edma3_Config * vcp2_Edma3_Config);
void Config_TCP2_EDMA3_PaRAM(TCP2_Edma3_Config * tcp2_Edma3_Config);


#ifdef __cplusplus
}
#endif

#endif /*_VCP2_TCP2_EDMA_H_*/

