/******************************************************************************\
*           TEXAS INSTRUMENTS INCORPORATED PROPRIETARY INFORMATION           
*                                                                            
*  Property of Texas Instruments 
*  For  Unrestricted  Internal  Use  Only 
*  Unauthorized reproduction and/or distribution is strictly prohibited.  
*  This product is protected under copyright law and trade secret law 
*  as an unpublished work.  
*  Created 2004, (C) Copyright 2003 Texas Instruments.  All rights reserved.
*------------------------------------------------------------------------------
*  Filename       : tcp2_edma.c
*  Date Created   : Oct 18, 2004
*  Last Modified  : Mar 11, 2008
*  Description    : Common Functions for configuring the 3PCC DMA for TCP2
*  History        :
*  Project        : Ported for Curie from Himalaya
*  Author         : Lloyd D'cruz
\*****************************************************************************/

#include <csl_edma3.h>	// Changed csl_edma.h to csl_edma3.h by Thara
#include <csl_tcp2.h>
#include "val_util.h"
#include "tcp2_edma.h"
#include <csl_edma3Aux.h>	// changed csl_edmaAux.h to csl_edma3Aux.h by Thara
#define TEST_WIMODE_MASK 0x80000 // TODO 

// temporarily assigned
#define CURIE_TEST_GEM_DRAELO  0xffffffff
#define CURIE_TEST_GEM_DRAEHI  0x0

#define CSL_EDMA3_DMACHANNELSETUP_DEFAULT1   {       \
   {CSL_EDMA3_QUE_0,0}, \
   {CSL_EDMA3_QUE_0,1}, \
   {CSL_EDMA3_QUE_0,2}, \
   {CSL_EDMA3_QUE_0,3}, \
   {CSL_EDMA3_QUE_0,4}, \
   {CSL_EDMA3_QUE_0,5}, \
   {CSL_EDMA3_QUE_0,6}, \
   {CSL_EDMA3_QUE_0,7}, \
   {CSL_EDMA3_QUE_0,8}, \
   {CSL_EDMA3_QUE_0,9}, \
   {CSL_EDMA3_QUE_0,10}, \
   {CSL_EDMA3_QUE_0,11}, \
   {CSL_EDMA3_QUE_0,12}, \
   {CSL_EDMA3_QUE_0,13}, \
   {CSL_EDMA3_QUE_0,14}, \
   {CSL_EDMA3_QUE_0,15}, \
   {CSL_EDMA3_QUE_0,16}, \
   {CSL_EDMA3_QUE_0,17}, \
   {CSL_EDMA3_QUE_0,18}, \
   {CSL_EDMA3_QUE_0,19}, \
   {CSL_EDMA3_QUE_0,20}, \
   {CSL_EDMA3_QUE_0,21}, \
   {CSL_EDMA3_QUE_0,22}, \
   {CSL_EDMA3_QUE_0,23}, \
   {CSL_EDMA3_QUE_0,24}, \
   {CSL_EDMA3_QUE_0,25}, \
   {CSL_EDMA3_QUE_0,26}, \
   {CSL_EDMA3_QUE_0,27}, \
   {CSL_EDMA3_QUE_0,28}, \
   {CSL_EDMA3_QUE_0,29}, \
   {CSL_EDMA3_QUE_0,30}, \
   {CSL_EDMA3_QUE_0,31}, \
   {CSL_EDMA3_QUE_0,32}, \
   {CSL_EDMA3_QUE_0,33}, \
   {CSL_EDMA3_QUE_0,34}, \
   {CSL_EDMA3_QUE_0,35}, \
   {CSL_EDMA3_QUE_0,36}, \
   {CSL_EDMA3_QUE_0,37}, \
   {CSL_EDMA3_QUE_0,38}, \
   {CSL_EDMA3_QUE_0,39}, \
   {CSL_EDMA3_QUE_0,40}, \
   {CSL_EDMA3_QUE_0,41}, \
   {CSL_EDMA3_QUE_0,42}, \
   {CSL_EDMA3_QUE_0,43}, \
   {CSL_EDMA3_QUE_0,44}, \
   {CSL_EDMA3_QUE_0,45}, \
   {CSL_EDMA3_QUE_0,46}, \
   {CSL_EDMA3_QUE_0,47}, \
   {CSL_EDMA3_QUE_0,48}, \
   {CSL_EDMA3_QUE_0,49}, \
   {CSL_EDMA3_QUE_0,50}, \
   {CSL_EDMA3_QUE_0,51}, \
   {CSL_EDMA3_QUE_0,52}, \
   {CSL_EDMA3_QUE_0,53}, \
   {CSL_EDMA3_QUE_0,54}, \
   {CSL_EDMA3_QUE_0,55}, \
   {CSL_EDMA3_QUE_0,56}, \
   {CSL_EDMA3_QUE_0,57}, \
   {CSL_EDMA3_QUE_0,58}, \
   {CSL_EDMA3_QUE_0,59}, \
   {CSL_EDMA3_QUE_0,60}, \
   {CSL_EDMA3_QUE_0,61}, \
   {CSL_EDMA3_QUE_0,62}, \
   {CSL_EDMA3_QUE_0,63} \
}

/* EDMA module Handle */
CSL_Edma3Handle hModule;
CSL_Edma3Obj	edmaObj;
CSL_Edma3Context	contextMain;

/* EDMA Handles for Transmit and Receive Channels */
CSL_Edma3ChannelHandle hEdmaTcpXevt = NULL; /* EDMA channel for IC Values */ 
CSL_Edma3ChannelHandle hEdmaTcpRevt = NULL; /* EDMA channel for Decisions */
CSL_Edma3ChannelHandle hEdmaTcpRevt1 = NULL; /* EDMA channel for Decisions */

/* EDMA PaRAM Handles */
CSL_Edma3ParamHandle hParamIc; // Handler to the TCP2_ConfigIc structure Parameter entry
CSL_Edma3ParamHandle hParamSys; // Systematic and parity data (Input to the TCP)
CSL_Edma3ParamHandle hParamInt; // Interleaver coefficients (Input to the TCP)
CSL_Edma3ParamHandle hParamDec; // Hard Decision data (Output from the TCP)
CSL_Edma3ParamHandle hParamOp; // 
CSL_Edma3ParamHandle hParamDum; //

/* EDMA PaRAM Handles for Multichannel */
CSL_Edma3ParamHandle hParamIcArray[64], hParamSysArray[64],hParamIntArray[64],hParamDecArray[64], hParamOpArray[64];
CSL_Edma3ParamHandle hParamAprArray[64];
CSL_Edma3ParamHandle hParamDummyArray[64];

/* EDMA Channel Objects for Xmt and Rcv */
CSL_Edma3ChannelObj edmaTcpXmtObj, edmaTcpRcvObj, edmaTputRcvObj;

/* EDMA handles */
CSL_Edma3Context       edmaContext;
CSL_Edma3ChannelAttr   chAttr;
CSL_Edma3CmdIntr       CommonregionIntr;
CSL_Edma3CmdDrae       draeAttr;
CSL_Status             chStatus;
CSL_Status             status;

Uint32  intrEn[2];
Uint32  ParamTxno = 31;
Uint32  ParamRxno = 30;
Uint32 temp_hParamIcArray;
Uint32 temp_hParamDecArray;
/****************************************************************************\
 * setupTcpEdma: The EDMA is programmed to service single  channel scenario *
\****************************************************************************/
void setupTcpEdma(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 			
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length;

    frame_length = tcpParameters->frameLen; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;

        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,	
                            &chAttr,
                            &chStatus);
                            
	/* Xmt Channel Setup */
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT); //Channel to parameter entry mapping
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);	//Channel to Que mapping


	/* Acquire basic param and set it up Parameter entry Basic Setup */
	hParamIc  = CSL_edma3GetParamHandle(hEdmaTcpXevt, CSL_EDMA3_CHA_TCP2XEVT, &status);
	hParamSys = CSL_edma3GetParamHandle(hEdmaTcpXevt, CSL_EDMA3_CHA_TCP2XEVT+2, &status);
	hParamInt = CSL_edma3GetParamHandle(hEdmaTcpXevt, CSL_EDMA3_CHA_TCP2XEVT+4, &status);
	hParamDum = CSL_edma3GetParamHandle(hEdmaTcpXevt, CSL_EDMA3_CHA_TCP2XEVT+6, &status);
	        
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                     CSL_EDMA3_TCCH_DIS, \
                                     CSL_EDMA3_ITCINT_DIS, \
                                     CSL_EDMA3_TCINT_DIS,\
                                     CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                     CSL_EDMA3_FIFOWIDTH_NONE, \
                                     CSL_EDMA3_STATIC_DIS, \
                                     CSL_EDMA3_SYNC_A, \
                                     CSL_EDMA3_ADDRMODE_INCR, \
                                     CSL_EDMA3_ADDRMODE_INCR
                                   );           
	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIc, option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx,hParamSys);    
  
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/

    /* Acquire basic param and set it up Parameter entry Basic Setup */
    /* Systematic-Parity data */
    	Acnt =  2 * (frame_length/2); // Acnt Wordcount 
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
    	Bcnt = 0; 
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

	option = CSL_EDMA3_OPT_MAKE(CSL_EDMA3_ITCCH_DIS, \
                                    CSL_EDMA3_TCCH_DIS, \
                                    CSL_EDMA3_ITCINT_DIS, \
                                    CSL_EDMA3_TCINT_DIS,\
				    CSL_XMT_TCC,CSL_EDMA3_TCC_NORMAL,					
				    CSL_EDMA3_FIFOWIDTH_NONE,FALSE,				
				    CSL_EDMA3_SYNC_AB,CSL_EDMA3_ADDRMODE_INCR,	
				    CSL_EDMA3_ADDRMODE_INCR);	

    if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE)
    {	
	option = CSL_EDMA3_OPT_MAKE(CSL_EDMA3_ITCCH_DIS, \
                                    CSL_EDMA3_TCCH_DIS, \
                                    CSL_EDMA3_ITCINT_DIS, \
                                    CSL_EDMA3_TCINT_DIS,\
				    CSL_XMT_TCC,CSL_EDMA3_TCC_NORMAL,
				    CSL_EDMA3_FIFOWIDTH_NONE,CSL_EDMA3_STATIC_DIS,
				    CSL_EDMA3_SYNC_A,CSL_EDMA3_ADDRMODE_INCR, 
				    CSL_EDMA3_ADDRMODE_INCR);		

	option = (option|(TEST_WIMODE_MASK));  // Enabling WIMODE 
    }
    else
    {
	
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
       option = CSL_EDMA3_OPT_MAKE(CSL_EDMA3_ITCCH_DIS, \
                                   CSL_EDMA3_TCCH_DIS, \
                                   CSL_EDMA3_ITCINT_DIS, \
                                   CSL_EDMA3_TCINT_DIS,\
			           CSL_XMT_TCC,CSL_EDMA3_TCC_NORMAL,					
				   CSL_EDMA3_FIFOWIDTH_NONE,FALSE,				
				   CSL_EDMA3_SYNC_AB,CSL_EDMA3_ADDRMODE_INCR,	
				   CSL_EDMA3_ADDRMODE_INCR);						
    } 

	
       edmaParamSetup(hParamSys, option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx,hParamInt);
    
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

    /* Acquire basic param and set it up Parameter entry Basic Setup */
       option = CSL_EDMA3_OPT_MAKE(FALSE,FALSE,FALSE,TRUE,
			           CSL_XMT_TCC,CSL_EDMA3_TCC_NORMAL,			
				   CSL_EDMA3_FIFOWIDTH_NONE,						
				   FALSE,CSL_EDMA3_SYNC_A,CSL_EDMA3_ADDRMODE_INCR,	
				   CSL_EDMA3_ADDRMODE_INCR);						

    /* Interleaver data */
	Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
	if(((frame_length+3)%4)!=0)
	{
	Acnt =  8 * (((frame_length+3)/4)+1);
	}
        Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	
	
    edmaParamSetup(hParamInt, option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx,hParamDum);

   /*----------------------------------------------------------------------*/
   /* Setup the parameter entry for the Dummy pointer     	   	   */
   /*----------------------------------------------------------------------*/
 
       Bcnt = 0;
       Acnt = 0;
       Bidx = 0;
       Cidx = 0;	
       Ccnt = 1;
       option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                    CSL_EDMA3_TCCH_DIS, \
                                    CSL_EDMA3_ITCINT_DIS, \
                                    CSL_EDMA3_TCINT_DIS,\
                                    CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                    CSL_EDMA3_FIFOWIDTH_NONE, \
                                    CSL_EDMA3_STATIC_DIS, \
                                    CSL_EDMA3_SYNC_AB, \
                                    CSL_EDMA3_ADDRMODE_INCR, \
                                    CSL_EDMA3_ADDRMODE_INCR
                                  );
	 edmaParamSetup(hParamDum, option, 0,
                        0, Acnt, Bcnt,
                        Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

   /*----------------------------------------------------------------------*/
   /* Link SPMEM to ILMEM and ILMEM to NULL Table     	  	   	   */
   /*----------------------------------------------------------------------*/
   /* Interrupt enable (Bits 0-11)  for the shadow region 1 */
    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr   =  1 << CSL_XMT_TCC ;   
    CommonregionIntr.intrh  =  0 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); 
	
   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
   /* Rcv Channel Open */
    chAttr.regionNum = CSL_EDMA3_REGION_1;		
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,	
                            	       &chAttr,
                            	       &chStatus);

   /* Channel Setup */
   CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT); 
   CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
   
   /* Acquire basic param and set it up Parameter entry Basic Setup */
   hParamDec = CSL_edma3GetParamHandle(hEdmaTcpRevt, CSL_EDMA3_CHA_TCP2REVT, &status);
   hParamOp = CSL_edma3GetParamHandle(hEdmaTcpRevt, CSL_EDMA3_CHA_TCP2REVT-2, &status);

   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                CSL_EDMA3_TCCH_DIS, \
                                CSL_EDMA3_ITCINT_DIS, \
                                CSL_EDMA3_TCINT_DIS,\
                                CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                CSL_EDMA3_FIFOWIDTH_NONE, \
                                CSL_EDMA3_STATIC_DIS, \
                                CSL_EDMA3_SYNC_A, \
                                CSL_EDMA3_ADDRMODE_INCR, \
                                CSL_EDMA3_ADDRMODE_INCR
                              );     
                                                   
    /* Hard decision data */

    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	
    edmaParamSetup (hParamDec, option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx,hParamOp);
  
   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/
  
   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                CSL_EDMA3_TCCH_DIS, \
                                CSL_EDMA3_ITCINT_DIS, \
                                CSL_EDMA3_TCINT_EN,\
                                CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                CSL_EDMA3_FIFOWIDTH_NONE, \
                                CSL_EDMA3_STATIC_DIS, \
                                CSL_EDMA3_SYNC_A, \
                                CSL_EDMA3_ADDRMODE_INCR, \
                                CSL_EDMA3_ADDRMODE_INCR
                              );     

   Acnt = TCP_OP_CONF_SIZE;
   Bcnt = 1;
   Ccnt = 1;
   Bidx = 0;
   Cidx = 0;
	
   edmaParamSetup (hParamOp, option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx,
                    (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
   /*------------------------------------------------------------------*/
   /* (6) Link HDMEM to OPMEM and OPMEM to NULL Table                  */
   /*------------------------------------------------------------------*/          
   /* Interrupt enable (Bits 0-11)  for the shadow region 1 */
   CommonregionIntr.region =  CSL_EDMA3_REGION_1 ; 
   CommonregionIntr.intr  |=  1 << CSL_RCV_TCC ;   
   CommonregionIntr.intrh  =  0x0 ;   
   CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);	 
   CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);
}


/****************************************************************************\
 * SetupTcpMultiEdma: The EDMA is programmed to service multi
 * channel scenario
\****************************************************************************/
void SetupTcpMultiEdma(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 	
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms,Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length;
    Uint32 ChannelNum;

    frame_length = tcpParameters->frameLen; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/

   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;	
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,	
                            &chAttr,
                            &chStatus);
                            
	/* Xmt Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,PARAM_ENTRY_NUM_IC); 
	CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); //Enable transmit channel
		
	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
	hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, NULL);
	
     }	
     else
     {

    ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
    hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, NULL);

     }
	
	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 2);
	hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, NULL);
	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 4);
	hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, NULL);

        option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );           

	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSysArray[*numDec]);   //added hParamSysArray[*numDec] by Thara

    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/
       /* Systematic-Parity data */

        Acnt =  2 * (frame_length/2); /* Acnt Wordcount */
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
        Bcnt = 0;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

    if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE) 
    {	
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 
	
	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
    }
    else
    {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 
    }

	
       edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx,hParamIntArray[*numDec]);
    
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

	/* Acquire basic param and set it up Parameter entry Basic Setup */
	
        ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 4);
	hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, NULL);
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 

    /* Interleaver data */

	Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
	if(((frame_length+3)%4)!=0)
	{
	Acnt =  8 * (((frame_length+3)/4)+1);
	}
        Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	
	
    edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

	CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); //Enable transmit channel

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
	/* Rcv Channel Open */
    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;		
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,
                            	       &chAttr,
                            	       &chStatus);

	/* Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,PARAM_ENTRY_NUM_IC); 
	CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); //Enable transmit channel

    ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6) );
    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, NULL);
    }
    else
    {

     ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6) );
     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, NULL);

    }
       ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6)  + 2);
       hParamOpArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, NULL);
	
     option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );           
  	
    /* Hard decision data */

    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	
    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx,
		   hParamOpArray[*numDec]);
  
   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 
	
	Acnt = TCP_OP_CONF_SIZE;
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;
	
    edmaParamSetup (hParamOpArray[*numDec], option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx,
                    (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
}


/****************************************************************************\
 * SetupTcpMultiEdma_dummy_transfer: The EDMA is programmed to service multi
 * channel scenario for checking throughput numbers
\****************************************************************************/
void SetupTcpMultiEdma_dummy_transfer(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms,Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length;
    Uint32 ChannelNum;

    frame_length = tcpParameters->frameLen; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/
   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;		
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,	
                            &chAttr,
                            &chStatus);
                            
	/* Xmt Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT); 
			
	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
	hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	
     }	
     else
     {

       ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
       hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);

     }

	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 2);
	hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 4);
	hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );           

	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSysArray[*numDec]);   


     if (*numDec > 0)
     {
      hParamIntArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamIcArray[*numDec],(hParamIntArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     } 
  
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/
        /* Systematic-Parity data */

        Acnt =  2 * (frame_length/2); /* Acnt Wordcount */
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
        Bcnt = 0;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

    if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE) 
    {	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
    }
    else
    {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
       option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                    CSL_EDMA3_TCCH_DIS, \
                                    CSL_EDMA3_ITCINT_DIS, \
                                    CSL_EDMA3_TCINT_DIS,\
                                    CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                    CSL_EDMA3_FIFOWIDTH_NONE, \
                                    CSL_EDMA3_STATIC_DIS, \
                                    CSL_EDMA3_SYNC_AB, \
                                    CSL_EDMA3_ADDRMODE_INCR, \
                                    CSL_EDMA3_ADDRMODE_INCR
                                  );
    }

	
       edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamIntArray[*numDec]);
    
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

    /* Acquire basic param and set it up Parameter entry Basic Setup */
    option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                 CSL_EDMA3_TCCH_DIS, \
                                 CSL_EDMA3_ITCINT_DIS, \
                                 CSL_EDMA3_TCINT_EN,\
                                 CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                 CSL_EDMA3_FIFOWIDTH_NONE, \
                                 CSL_EDMA3_STATIC_DIS, \
                                 CSL_EDMA3_SYNC_A, \
                                 CSL_EDMA3_ADDRMODE_INCR, \
                                 CSL_EDMA3_ADDRMODE_INCR
                               );
    /* Interleaver data */
    Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
    if(((frame_length+3)%4)!=0)
	{
	Acnt =  8 * (((frame_length+3)/4)+1);
	}
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	
	
    edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx,(CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
   /* Rcv Channel Open */
    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;		
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,
                            	       &chAttr,
                            	       &chStatus);

    /* Channel Setup */
    CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
    CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT);

    ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
    hParamDummyArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);
    }
    else
    {

     ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
     hParamDummyArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);

    }

   ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6)  + 4);
   hParamOpArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);


	/* Acquire basic param and set it up Parameter entry Basic Setup */
    option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                 CSL_EDMA3_TCCH_DIS, \
                                 CSL_EDMA3_ITCINT_DIS, \
                                 CSL_EDMA3_TCINT_EN,\
                                 CSL_RCV_TCC_DUMMY, CSL_EDMA3_TCC_NORMAL,\
                                 CSL_EDMA3_FIFOWIDTH_NONE, \
                                 CSL_EDMA3_STATIC_DIS, \
                                 CSL_EDMA3_SYNC_A, \
                                 CSL_EDMA3_ADDRMODE_INCR, \
                                 CSL_EDMA3_ADDRMODE_INCR
                               );
    /* Dummy 1 byte transfer L2-L2 */
    Acnt = 1;
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	
    edmaParamSetup (hParamDummyArray[*numDec], option, (Uint32 *)interleaverTable,
		   (Uint32 *)interleaverTable, Acnt, Bcnt, Ccnt, Bidx, Cidx, hParamOpArray[*numDec]); 

    if(*numDec > 0)
    {
      hParamOpArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDummyArray[*numDec],(hParamOpArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
    } 

   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/

    /* Rcv Channel Open */
    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;		
    chAttr.chaNum = TCP_RCV_CHANNEL_NUM;

    hEdmaTcpRevt1 = CSL_edma3ChannelOpen(&edmaTputRcvObj,
                            	       CSL_EDMA3,	
                            	       &chAttr,
                            	       &chStatus);
    /* Channel Setup */
    CSL_edma3HwChannelSetupQue(hEdmaTcpRevt1,CSL_EDMA3_QUE_0);
    CSL_edma3HwChannelSetupParam(hEdmaTcpRevt1,TCP_RCV_CHANNEL_NUM);

    ChannelNum =TCP_RCV_CHANNEL_NUM;
    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt1, ChannelNum, &status);
    }
    else
    {
     ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6) + 2);
     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt1, ChannelNum, &status);
    }

   /* Acquire basic param and set it up Parameter entry Basic Setup */
    option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                 CSL_EDMA3_TCCH_DIS, \
                                 CSL_EDMA3_ITCINT_DIS, \
                                 CSL_EDMA3_TCINT_DIS,\
                                 CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                 CSL_EDMA3_FIFOWIDTH_NONE, \
                                 CSL_EDMA3_STATIC_DIS, \
                                 CSL_EDMA3_SYNC_A, \
                                 CSL_EDMA3_ADDRMODE_INCR, \
                                 CSL_EDMA3_ADDRMODE_INCR
                               );
    /* Hard decision data */
    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	
    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx, hParamOpArray[*numDec]);
 

     if(*numDec > 0)
     {
      hParamDecArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDecArray[*numDec],(hParamDecArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     }
 
   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/
   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                CSL_EDMA3_TCCH_DIS, \
                                CSL_EDMA3_ITCINT_DIS, \
                                CSL_EDMA3_TCINT_EN,\
                                CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                CSL_EDMA3_FIFOWIDTH_NONE, \
                                CSL_EDMA3_STATIC_DIS, \
                                CSL_EDMA3_SYNC_A, \
                                CSL_EDMA3_ADDRMODE_INCR, \
                                CSL_EDMA3_ADDRMODE_INCR
                              );

   Acnt = TCP_OP_CONF_SIZE;
   Bcnt = 1;
   Ccnt = 1;
   Bidx = 0;
   Cidx = 0;
	
   edmaParamSetup (hParamOpArray[*numDec], option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
}

/****************************************************************************\
 * SetupTcpMultiEdma_RuseIntTble: The EDMA is programmed to service the multi 
 * channel by reusing Interleaver table from second channel onwards 
\****************************************************************************/
void SetupTcpMultiEdma_RuseIntTble(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms,Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length;
    Uint32 ChannelNum;

    frame_length = tcpParameters->frameLen; 


   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/
   
   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;		
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,	
                            &chAttr,
                            &chStatus);
                            
	/* Xmt Channel Setup */
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT);
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);

    	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
	hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	}	
    else
    {
        ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
        hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
    }





    ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 6);
    hParamDummyArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);

    option = CSL_EDMA3_OPT_MAKE(FALSE,FALSE,FALSE,FALSE,
 				   CSL_XMT_TCC,CSL_EDMA3_TCC_NORMAL,
 				   CSL_EDMA3_FIFOWIDTH_NONE,FALSE,
 				   CSL_EDMA3_SYNC_A,CSL_EDMA3_ADDRMODE_INCR,
 				   CSL_EDMA3_ADDRMODE_INCR);
 							   
	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx,  hParamSysArray[*numDec]);   


    
	if (*numDec == 1)
        {
	
	temp_hParamIcArray = 0xFFFF & (Uint32)hParamIcArray[*numDec];
	CSL_edma3ParamWriteWord(hParamIntArray[*numDec - 1],5,temp_hParamIcArray);
	}

	if (*numDec > 1)
        {
	temp_hParamIcArray = 0xFFFF & (Uint32)hParamIcArray[*numDec];
	CSL_edma3ParamWriteWord(hParamSysArray[*numDec - 1],5,temp_hParamIcArray);
        }
			
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/

       /* Systematic-Parity data */
        ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 2);
        hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	
        Acnt =  2 * (frame_length/2); /* Acnt Wordcount */
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
        Bcnt = 0;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

    if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE)
    {	
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
    }
    else
    {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

    }


	 edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamDummyArray[*numDec]);
	
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

    /* Acquire basic param and set it up Parameter entry Basic Setup */

        if(*numDec == 0)
        {
    ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 4);
    hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	
		option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

    /* Interleaver data */

	   Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
	   if(((frame_length+3)%4)!=0)
	   {
	      Acnt =  8 * (((frame_length+3)/4)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
	
    edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx,hParamIcArray[*numDec]);
}
   /*----------------------------------------------------------------------*/
   /* Setup the parameter entry for the Dummy pointer       	   	   */
   /*----------------------------------------------------------------------*/
       Bcnt = 0;
       Acnt = 0;
       Bidx = 0;
       Cidx = 0;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	 edmaParamSetup(hParamDummyArray[*numDec], option, 0,
                     0, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

	CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr   =  1 << CSL_XMT_TCC ;   
    CommonregionIntr.intrh  =  0x0 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); // Enable transmit channel    
	   
   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
	/* Rcv Channel Open */

    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;		
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,		
                            	       &chAttr,
                            	       &chStatus);

    /* Channel Setup */
    CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT);
    CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
    CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); 	

    ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);
    }
    else
    {

     ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);

    }
	
	 ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6)  + 2);
     hParamOpArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);

	if (*numDec > 0)
       {
		temp_hParamDecArray = 0xFFFF & (Uint32)hParamDecArray[*numDec];
		CSL_edma3ParamWriteWord(hParamOpArray[*numDec - 1],5,temp_hParamDecArray);
	   }

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
    /* Hard decision data */

    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	

    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx, hParamOpArray[*numDec]);
  
   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/

		option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	
	Acnt = TCP_OP_CONF_SIZE;
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;
	
    edmaParamSetup (hParamOpArray[*numDec], option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx,
                    (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
    CommonregionIntr.region =  CSL_EDMA3_REGION_1 ; 
    CommonregionIntr.intr  |=  1 << CSL_RCV_TCC ;   
    CommonregionIntr.intrh  =  0x0 ;   

    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);	 
    CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);

}
/****************************************************************************\
 * setupTcpEdmaSpmode: The EDMA is programmed to service single  channel
 * scenario : Shared Processing mode
\****************************************************************************/

void setupTcpEdmaSpmode(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 
                  TCP2_Params *tcpParameters, Uint32 *AprioriTable,
                  Uint32 *ExtrinsicData, Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length,prolog_length;
    Uint32 ExtrinsicRamOffset,OperationMode;
    frame_length = tcpParameters->frameLen; 
    prolog_length = tcpParameters->prologSize; 

    OperationMode = configIc -> ic0;
    OperationMode = (OperationMode >>1)&3; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/

     
   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,		
                            &chAttr,
                            &chStatus);
                            
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT);
   }	

    hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	hParamAprArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	hParamDummyArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSysArray[*numDec]);   

	if (*numDec > 0)
    {
		temp_hParamIcArray = 0xFFFF & (Uint32)hParamIcArray[*numDec];
		CSL_edma3ParamWriteWord(hParamAprArray[*numDec - 1],5,temp_hParamIcArray);
	}

    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/

       /* Systematic-Parity data */

        if(OperationMode == TCP_SP_FIRST_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + prolog_length)/2); /* Acnt Wordcount */
	   if((((frame_length + prolog_length))%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_MIDDLE_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + 2*prolog_length)/2); /* Acnt Wordcount */
	   if(((frame_length + 2*prolog_length)%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + 2*prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_LAST_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + prolog_length)/2); /* Acnt Wordcount */
	   if((((frame_length + prolog_length))%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	

        }
        
       if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE)
       {	
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
       }
       else
       {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

      }

	
       edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamAprArray[*numDec]);
    
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );


     /* Apriori Data */
 
        if(OperationMode == TCP_SP_FIRST_SUBFRAME)
        {
	   Acnt =  8 * ((frame_length + prolog_length)/8); /* Byte count */
	   if(((frame_length + prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_MIDDLE_SUBFRAME)
        {
	   Acnt =  8 * (((frame_length + 2 * prolog_length)/8)); /* Byte count */
	   if(((frame_length + 2 * prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + 2 * prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;
        }
        else if(OperationMode == TCP_SP_LAST_SUBFRAME)
        {
	   Acnt =  8 * ((frame_length + prolog_length)/8); /* Byte count */
	   if(((frame_length + prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;
       }  

	
    edmaParamSetup(hParamAprArray[*numDec], option, (Uint32 *)AprioriTable,
 		  (Uint32 *)TCP_EXTRNSICMEM_W1_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, hParamDummyArray[*numDec]);

   /*----------------------------------------------------------------------*/
   /* Setup the parameter entry for the Dummy pointer       	   	   */
   /*----------------------------------------------------------------------*/
 
	   Bcnt = 0;
       Acnt = 0;
       Bidx = 0;
       Cidx = 0;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	 edmaParamSetup(hParamDummyArray[*numDec], option, 0,
                     0, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr   =  0xFFFFFFFF ;   
    CommonregionIntr.intrh  =  0x0 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); // Enable transmit channel    

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
   /* Rcv Channel Open */

    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,		
                            	       &chAttr,
                            	       &chStatus);

	/* Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT);

    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ParamRxno--, &status);
    }
    else
    {
     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ParamRxno--, &status);
    }
	
	if (*numDec > 0)
    {
		temp_hParamDecArray = 0xFFFF & (Uint32)hParamDecArray[*numDec];
		CSL_edma3ParamWriteWord(hParamDecArray[*numDec - 1],5,temp_hParamDecArray);
	}

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

    /* Extrinsic data */
    Acnt = 8 * frame_length/8;
    if((frame_length%8)!=0)
    {
    Acnt = 8 * ((frame_length/8)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
  
   if((OperationMode == TCP_SP_MIDDLE_SUBFRAME)||(OperationMode == TCP_SP_LAST_SUBFRAME))
   {
      ExtrinsicRamOffset = prolog_length;
   }
   else
   {

     ExtrinsicRamOffset = 0;

   }
  
    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)(TCP_EXTRNSICMEM_W0_ADDR + ExtrinsicRamOffset),
		   (Uint32 *)ExtrinsicData, Acnt, Bcnt, Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);
  
    CommonregionIntr.region =  CSL_EDMA3_REGION_1 ; 
    CommonregionIntr.intr  |=  1 << CSL_RCV_TCC ;   
    CommonregionIntr.intrh  =  0x0 ;   

    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);	 
    CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);

}

/*------------------------------------------------------------------*/
/* EDMA MODULE Initialization 
*------------------------------------------------------------------*/          
void edma3ModuleInit(Uint32 edma3_region_num)
{
  CSL_Edma3HwSetup hwSetup;
  CSL_Edma3QueryInfo 	info;
  CSL_Status	status;
  CSL_Edma3HwDmaChannelSetup dmahwSetup[CSL_EDMA3_NUM_DMACH ] = CSL_EDMA3_DMACHANNELSETUP_DEFAULT1;

  // Module Initialization
  CSL_edma3Init(&contextMain);

  // Module Level Open
  hModule = CSL_edma3Open(&edmaObj,CSL_EDMA3,NULL,&status);

  //Module Setup
  hwSetup.dmaChaSetup = &dmahwSetup[0];
  hwSetup.qdmaChaSetup = NULL;
  CSL_edma3HwSetup(hModule,&hwSetup);

  // Query Module revision and configuration.
  CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INFO,&info);

  // Setup the DRAE Masks
  draeAttr.region = edma3_region_num;    
  draeAttr.drae =   CURIE_TEST_GEM_DRAELO;
  draeAttr.draeh =  CURIE_TEST_GEM_DRAEHI; 
  CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_DMAREGION_ENABLE,&draeAttr);
}	


/*------------------------------------------------------------------*/
/* Populating the EDMA Parameters 
*------------------------------------------------------------------*/          

void edmaParamSetup (CSL_Edma3ParamHandle hParam, Uint32 option, 
                     Uint32 *srcAddr, Uint32 *dstAddr, Uint32 Acnt,
                     Uint32 Bcnt, Uint32 Ccnt,Uint32 Bidx, Uint32 Cidx,CSL_Edma3ParamHandle link) 
{
    CSL_Edma3ParamSetup     myParamSetup;

    myParamSetup.option = option;
    myParamSetup.srcAddr = (Uint32) srcAddr;
    myParamSetup.aCntbCnt = CSL_EDMA3_CNT_MAKE(Acnt, Bcnt);
    myParamSetup.dstAddr = (Uint32) dstAddr;
    myParamSetup.srcDstBidx = CSL_EDMA3_BIDX_MAKE(Bidx, Bidx);
    myParamSetup.linkBcntrld = CSL_EDMA3_LINKBCNTRLD_MAKE(link,0); 
    myParamSetup.srcDstCidx = CSL_EDMA3_CIDX_MAKE(Cidx, Cidx);
    myParamSetup.cCnt = Ccnt;
    CSL_edma3ParamSetup(hParam, &myParamSetup);
}

void edmaCloseHandles_tput()
{
	CSL_edma3ChannelClose(hEdmaTcpXevt);
	CSL_edma3ChannelClose(hEdmaTcpRevt);
	CSL_edma3ChannelClose(hEdmaTcpRevt1);
	CSL_edma3Close(hModule);
}

void edmaCloseHandles()
{
	CSL_edma3ChannelClose(hEdmaTcpXevt);
	CSL_edma3ChannelClose(hEdmaTcpRevt);
	CSL_edma3Close(hModule);
}

void edmaMultChnlEnable_tput()
{
    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  =   1 << CSL_XMT_TCC ;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);	
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);

    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  |= 1 << CSL_RCV_TCC_DUMMY ;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);	
    CSL_edma3HwChannelControl(hEdmaTcpRevt1,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);

    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  |= 1 << CSL_RCV_TCC ;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);
}

void edmaMultChnlEnable()
{
    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  = 0x80000000;
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);

    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  |= 0x40000000;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);
}

/****************************************************************************\
 * setupTcpEdma_MemAccess: The EDMA is programmed to service 
 * single  channel scenario                                                *
\****************************************************************************/

void setupTcpEdma_MemAccess(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 	
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length;

    frame_length = tcpParameters->frameLen; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
    hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,	
                            &chAttr,
                            &chStatus);
                            
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,PARAM_ENTRY_NUM_IC);
	
	/* Acquire basic param and set it up Parameter entry Basic Setup */
	hParamIc = CSL_edma3GetParamHandle(hEdmaTcpXevt, PARAM_ENTRY_NUM_IC, &status);
	hParamSys = CSL_edma3GetParamHandle(hEdmaTcpXevt, PARAM_ENTRY_NUM_SP, &status);
	hParamInt = CSL_edma3GetParamHandle(hEdmaTcpXevt, PARAM_ENTRY_NUM_IL, &status);

 	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIc, option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSys);    

	CommonregionIntr.intr  |= 1 << CSL_XMT_TCC ;   
    CommonregionIntr.intrh  =  0x0000 ;

    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/

       /* Systematic-Parity data */

        Acnt =  2 * (frame_length/2); /* Acnt Wordcount */
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
        Bcnt = 0;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

        if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE)
        {	
	
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC + 1, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
        }
        else
        {
           Bcnt = (Acnt/2);
           Acnt = 8;
           Bidx = 8;
           Cidx = 8;	
           Ccnt = 1;
           option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC + 1, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

       }

	
       edmaParamSetup(hParamSys, option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamInt);
    
   /*------------------------------------------------------------------*/
   /*                          Link ICMEM to SPMEM                     */
   /*------------------------------------------------------------------*/
  
	CommonregionIntr.intr  |= 1 << (CSL_XMT_TCC + 1) ;   
    CommonregionIntr.intrh  =  0x0000 ;

    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC + 2, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

    /* Interleaver data */

	Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
	if(((frame_length+3)%4)!=0)
	{
	Acnt =  8 * (((frame_length+3)/4)+1);
	}
        Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	
	
    edmaParamSetup(hParamInt, option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

   /*----------------------------------------------------------------------*/
   /* Link SPMEM to ILMEM and ILMEM to NULL Table     	  	   	   */
   /*----------------------------------------------------------------------*/
  
    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  |= 1 << (CSL_XMT_TCC + 2) ;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);
	
   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
   /* Rcv Channel Open */
    chAttr.regionNum = CSL_EDMA3_REGION_1;
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,		
                            	       &chAttr,
                            	       &chStatus);

    /* Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,PARAM_ENTRY_NUM_IC);
	
    /* Acquire basic param and set it up Parameter entry Basic Setup */
    hParamDec = CSL_edma3GetParamHandle(hEdmaTcpRevt, PARAM_ENTRY_NUM_HD, &status);
    hParamOp = CSL_edma3GetParamHandle(hEdmaTcpRevt, PARAM_ENTRY_NUM_OP, &status);

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

    /* Hard decision data */

    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	
    edmaParamSetup (hParamDec, option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx, hParamOp);
  


	intrEn[0] |= 1 << CSL_RCV_TCC;
	intrEn[1] = 0;

   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/
  
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC + 1, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	
	Acnt = TCP_OP_CONF_SIZE;
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;
	
        edmaParamSetup (hParamOp, option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx,
                    (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
   /*------------------------------------------------------------------*/
   /* (6) Link HDMEM to OPMEM and OPMEM to NULL Table                  */
   /*------------------------------------------------------------------*/          
   
	CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  |= 1 << (CSL_RCV_TCC + 1) ;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);
}


/****************************************************************************\
 * SetupTcp_Sa_Sp: The EDMA is programmed to service Standalone and Shared 
 * Processing frames
\****************************************************************************/
void SetupTcp_Sa_Sp(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms,Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length,prolog_length;
    Uint32 ExtrinsicRamOffset,OperationMode;

    frame_length = tcpParameters->frameLen; 
    prolog_length = tcpParameters->prologSize; 

    OperationMode = configIc -> ic0;
    OperationMode = (OperationMode >>1)&3; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/

   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
    hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                                        CSL_EDMA3,	
                                        &chAttr,
                                        &chStatus);
                            
	/* Xmt Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT);

	hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	
     }	
     else
     {

       hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);

     }
	
	hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);

 	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
 	
	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSysArray[*numDec]);   


      if(*numDec > 0)
	 {
	  hParamIntArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamIcArray[*numDec],(hParamIntArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
         }

       	
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/

       /* Systematic-Parity data */

        if(OperationMode == 0) /* for SA mode */
        {

        Acnt =  2 * (frame_length/2); /* Acnt Wordcount */
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
        Bcnt = 0;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	
        }
        else
        {

        /* SP Mode */

         if(OperationMode == TCP_SP_FIRST_SUBFRAME)
         {
           Acnt =  2 * ((frame_length + prolog_length)/2); /* Acnt Wordcount */
	   if((((frame_length + prolog_length))%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_MIDDLE_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + 2*prolog_length)/2); /* Acnt Wordcount */
	   if(((frame_length + 2*prolog_length)%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + 2*prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_LAST_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + prolog_length)/2); /* Acnt Wordcount */
	   if((((frame_length + prolog_length))%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	

        }
        }

       if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE) 
       {	
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	
	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
       }
       else
       {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
       
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
       
       }

	
       edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamIntArray[*numDec]);
    
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	

    /* Interleaver data */
     
        if(OperationMode == 0) /* Standalone Mode */
        {
	Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
	if(((frame_length+3)%4)!=0)
	{
	Acnt =  8 * (((frame_length+3)/4)+1);
	}
    Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

        edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);
        }
        else
        {

       /* Apriori Data for Shared Processing Mode */
 
        if(OperationMode == TCP_SP_FIRST_SUBFRAME)
        {
	   Acnt =  8 * ((frame_length + prolog_length)/8); /* Byte count */
	   if(((frame_length + prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	

         edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_EXTRNSICMEM_W1_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

        }
        else if(OperationMode == TCP_SP_MIDDLE_SUBFRAME)
        {
	   Acnt =  8 * (((frame_length + 2 * prolog_length)/8)); /* Byte count */
	   if(((frame_length + 2 * prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + 2 * prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;

	 edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_EXTRNSICMEM_W1_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

        }
        else if(OperationMode == TCP_SP_LAST_SUBFRAME)
        {
	   Acnt =  8 * ((frame_length + prolog_length)/8); /* Byte count */
	   if(((frame_length + prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;

         edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_EXTRNSICMEM_W1_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);
       }  
       }
	
   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
	/* Rcv Channel Open */
    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,		
                            	       &chAttr,
                            	       &chStatus);

	/* Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT);

    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ParamRxno--, &status);
    }
    else
    {

     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ParamRxno--, &status);

    }
	 hParamOpArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ParamRxno--, &status);

    if(OperationMode == 0) /* Standalone Mode */
    {
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	
    /* Hard decision data */

    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	
    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx, hParamOpArray[*numDec]);
  
   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/

     if(*numDec > 0)
     {
       hParamDecArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDecArray[*numDec],(hParamDecArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     }

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	
	
	Acnt = TCP_OP_CONF_SIZE;
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;
	
    edmaParamSetup (hParamOpArray[*numDec], option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx,
                    (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
   /*------------------------------------------------------------------*/
   /* (6) Link HDMEM to OPMEM and OPMEM to NULL Table          */
   /*------------------------------------------------------------------*/          
   }
   else
   {

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	

 /* Shared Prcoessing Mode */
 /* Extrinsic data */

    Acnt = 8 * frame_length/8;
    if((frame_length%8)!=0)
    {
    Acnt = 8 * ((frame_length/8)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
  
   if((OperationMode == TCP_SP_MIDDLE_SUBFRAME)||(OperationMode == TCP_SP_LAST_SUBFRAME))
   {
      ExtrinsicRamOffset = prolog_length;
   }
   else
   {

     ExtrinsicRamOffset = 0;

   }
  
    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)(TCP_EXTRNSICMEM_W0_ADDR + ExtrinsicRamOffset),
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx,
		   (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);
  
    if(*numDec > 1)
     {
       hParamDecArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDecArray[*numDec],(hParamDecArray[*numDec - 1]->LINK_BCNTRLD) >> 16); 
     }
     else
     {
       hParamOpArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDecArray[*numDec],(hParamOpArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     }
   }

}

/****************************************************************************\
 * SetupTcpMultiEdma_RuseIntTble: The EDMA is programmed to service the multi 
 * channel by reusing Interleaver table from second channel onwards 
\****************************************************************************/
void SetupTcpMultiChannelEdma_RuseIntTble(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms,Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length;
    Uint32 ChannelNum;

    frame_length = tcpParameters->frameLen; 


   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/
   
   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;		
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,	
                            &chAttr,
                            &chStatus);
                            
	/* Xmt Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT);

   	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
	hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	}	
    else
    {
        ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
        hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
    }

    	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 2);
		hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);

		ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 4);
    	hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);

		ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 6);
    	hParamDummyArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
		
    if(*numDec == 1)
	{
	hParamIntArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamIcArray[*numDec],(hParamIntArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
    }
    
    if(*numDec > 1)
	{
	hParamSysArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamIcArray[*numDec],(hParamSysArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
    }
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

 							   
	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

    edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSysArray[*numDec]);   


   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - SYSMEM                             */
   /*------------------------------------------------------------------*/
 
       /* Systematic-Parity data */

        Acnt =  2 * (frame_length/2); /* Acnt Wordcount */
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
    Bcnt = 0;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

    if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE)
    {	
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );



	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
    }
    else
    {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );


    }

    edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                    (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamIntArray[*numDec]);
    if(*numDec > 0)
	{
    edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                    (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamDummyArray[*numDec]);
 	}
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

       option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );


    /* Interleaver data */

	   Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
	   if(((frame_length+3)%4)!=0)
	   {
	      Acnt =  8 * (((frame_length+3)/4)+1);
	   }
       Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
	
    edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx,hParamDummyArray[*numDec]);

   
   /*----------------------------------------------------------------------*/
   /* Setup the parameter entry for the Dummy pointer     	  	   	   */
   /*----------------------------------------------------------------------*/
 
	   Bcnt = 0;
       Acnt = 0;
       Bidx = 0;
       Cidx = 0;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	 edmaParamSetup(hParamDummyArray[*numDec], option, 0,
                     0, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);    	 
 

   
   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
	/* Rcv Channel Open */

    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;		
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,		
                            	       &chAttr,
                            	       &chStatus);

	
	CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT);
    CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL); 	

    ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);
    }
    else
    {

     ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);

    }
	
	 ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6) + 2);
     hParamOpArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

    /* Hard decision data */

    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	

    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx, hParamOpArray[*numDec]);


   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/

		option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	
	Acnt = TCP_OP_CONF_SIZE;
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;
	
    edmaParamSetup (hParamOpArray[*numDec], option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx,
                    (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
    if(*numDec > 0)
	 {
       hParamOpArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDecArray[*numDec],(hParamOpArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     }
}

/****************************************************************************\
 * setupTcpEdmaSpmode: The EDMA is programmed to service single  channel
 * scenario : Shared Processing mode
\****************************************************************************/

void setupTcpMultiChannelEdmaSpmode(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 
                  TCP2_Params *tcpParameters, Uint32 *AprioriTable,
                  Uint32 *ExtrinsicData, Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length,prolog_length;
    Uint32 ExtrinsicRamOffset,OperationMode;
    frame_length = tcpParameters->frameLen; 
    prolog_length = tcpParameters->prologSize; 

    OperationMode = configIc -> ic0;
    OperationMode = (OperationMode >>1)&3; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/

     
   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,		
                            &chAttr,
                            &chStatus);
                            
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT);
   }	

    hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	hParamAprArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);
	hParamDummyArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ParamTxno++, &status);

	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );

	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSysArray[*numDec]);   

	
  
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - SPMEM                          */
    /*------------------------------------------------------------------*/

       /* Systematic-Parity data */

        if(OperationMode == TCP_SP_FIRST_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + prolog_length)/2); /* Acnt Wordcount */
	   if((((frame_length + prolog_length))%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_MIDDLE_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + 2*prolog_length)/2); /* Acnt Wordcount */
	   if(((frame_length + 2*prolog_length)%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + 2*prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_LAST_SUBFRAME)
        {
           Acnt =  2 * ((frame_length + prolog_length)/2); /* Acnt Wordcount */
	   if((((frame_length + prolog_length))%2)!=0)
	   {
	      Acnt = 2 * (((frame_length + prolog_length)/2) + 1);
	   }
           Bcnt = 0;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	

        }
        
       if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE)
       {	
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );


				   						
	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */
       }
       else
       {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );


      }

	
       edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, hParamAprArray[*numDec]);
    
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );



     /* Apriori Data */
 
        if(OperationMode == TCP_SP_FIRST_SUBFRAME)
        {
	   Acnt =  8 * ((frame_length + prolog_length)/8); /* Byte count */
	   if(((frame_length + prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;	
        }
        else if(OperationMode == TCP_SP_MIDDLE_SUBFRAME)
        {
	   Acnt =  8 * (((frame_length + 2 * prolog_length)/8)); /* Byte count */
	   if(((frame_length + 2 * prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + 2 * prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;
        }
        else if(OperationMode == TCP_SP_LAST_SUBFRAME)
        {
	   Acnt =  8 * ((frame_length + prolog_length)/8); /* Byte count */
	   if(((frame_length + prolog_length)%8)!=0)
	   {
	      Acnt =  8 * (((frame_length + prolog_length)/8)+1);
	   }
           Bcnt = 1;
	   Ccnt = 1;
	   Bidx = 0;
	   Cidx = 0;
       }  

	
    edmaParamSetup(hParamAprArray[*numDec], option, (Uint32 *)AprioriTable,
 		  (Uint32 *)TCP_EXTRNSICMEM_W1_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx, hParamDummyArray[*numDec]);


  if(*numDec > 0)
	{
	hParamAprArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamIcArray[*numDec],(hParamAprArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
    }
   /*----------------------------------------------------------------------*/
   /* Setup the parameter entry for the Dummy pointer     	  	   	   */
   /*----------------------------------------------------------------------*/
 
	   Bcnt = 0;
       Acnt = 0;
       Bidx = 0;
       Cidx = 0;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	 edmaParamSetup(hParamDummyArray[*numDec], option, 0,
                     0, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

	
   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
   /* Rcv Channel Open */

    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,		
                            	       &chAttr,
                            	       &chStatus);

	/* Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT);


    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ParamRxno--, &status);
    }
    else
    {
     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ParamRxno--, &status);
    }
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );



    Acnt = 8 * frame_length/8;
    if((frame_length%8)!=0)
    {
    Acnt = 8 * ((frame_length/8)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
  
   if((OperationMode == TCP_SP_MIDDLE_SUBFRAME)||(OperationMode == TCP_SP_LAST_SUBFRAME))
   {
      ExtrinsicRamOffset = prolog_length;
   }
   else
   {

     ExtrinsicRamOffset = 0;

   }
  
   if(*numDec > 0)
	 {
       hParamDecArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDecArray[*numDec],(hParamDecArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     }
    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)(TCP_EXTRNSICMEM_W0_ADDR + ExtrinsicRamOffset),
		   (Uint32 *)ExtrinsicData, Acnt, Bcnt, Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);
}



/****************************************************************************\
 * SetupTcpMultiEdma: The EDMA is programmed to service multi
 * channel scenario
\****************************************************************************/
void SetupTcpMultiChannelEdma(TCP2_ConfigIc *configIc, TCP2_UserData *TCP_UserData, 	// changed TCP to TCP2 by Thara
                  TCP2_Params *tcpParameters, Uint32 *interleaverTable,
                  Uint32 *hardDecisions, Uint32 *tcpOutParms,Uint32 *numDec)
{

    Uint32 option;
    Uint32 Acnt, Bcnt, Ccnt, Bidx, Cidx;        
    Uint32 frame_length;
    Uint32 ChannelNum;

    frame_length = tcpParameters->frameLen; 

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR TRANSMIT CHANNEL                    */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /*  EDMA Channel Setup for XEVT - ICMEM                             */
   /*------------------------------------------------------------------*/
   if(*numDec == 0)
   {
   
	/* Xmt Channel Open */
	chAttr.regionNum = CSL_EDMA3_REGION_1;		
	chAttr.chaNum = CSL_EDMA3_CHA_TCP2XEVT;
        hEdmaTcpXevt = CSL_edma3ChannelOpen(&edmaTcpXmtObj,
                            CSL_EDMA3,	
                            &chAttr,
                            &chStatus);
                            
	/* Xmt Channel Setup */
	CSL_edma3HwChannelSetupQue(hEdmaTcpXevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpXevt,CSL_EDMA3_CHA_TCP2XEVT);

   	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
	hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	}	
    else
    {
        ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8));
        hParamIcArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
    }

    	ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 2);
		hParamSysArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);

		ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 4);
    	hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);

		ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 6);
    	hParamDummyArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
		
			
  

	
        option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );           

  	
							   
	Acnt = TCP_IC_CONF_SIZE; 
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;

        edmaParamSetup (hParamIcArray[*numDec], option, (Uint32 *)configIc,
   		        (Uint32 *)TCP_ICMEM_ADDR, Acnt, Bcnt,
   			 Ccnt, Bidx, Cidx, hParamSysArray[*numDec]);

	
       /* Systematic-Parity data */

        Acnt =  2 * (frame_length/2); /* Acnt Wordcount */
	if((frame_length%2)!=0)
	{
	Acnt = 2 * ((frame_length/2) + 1);
	}
        Bcnt = 0;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	

    if(Acnt <= MAX_ACNT_IN_WI_BACKWARD_COMPATIBILITY_MODE) 
    {	
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 
	
	option = (option|(TEST_WIMODE_MASK));  /* Enabling WIMODE */

    }
    else
    {
       Bcnt = (Acnt/2);
       Acnt = 8;
       Bidx = 8;
       Cidx = 8;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 

    }

	
       edmaParamSetup(hParamSysArray[*numDec], option, (Uint32 *)TCP_UserData,
                     (Uint32 *)TCP_SPMEM_ADDR, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx,hParamIntArray[*numDec]);
    
    /*------------------------------------------------------------------*/
    /*     EDMA Channel Setup for XEVT - ILMEM                          */
    /*------------------------------------------------------------------*/

	/* Acquire basic param and set it up Parameter entry Basic Setup */
	
        ChannelNum = (CSL_EDMA3_CHA_TCP2XEVT + (*numDec * 8) + 4);
	hParamIntArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpXevt, ChannelNum, &status);
	
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 


    /* Interleaver data */

	Acnt =  8 * (((frame_length+3)/4)); /* Byte count */
	if(((frame_length+3)%4)!=0)
	{
	Acnt =  8 * (((frame_length+3)/4)+1);
	}
        Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;	
	
    edmaParamSetup(hParamIntArray[*numDec], option, (Uint32 *)interleaverTable,
 		  (Uint32 *)TCP_ILMEM_ADDR, Acnt, Bcnt, Ccnt,
    		   Bidx, Cidx,hParamDummyArray[*numDec]);

   if(*numDec > 0)
	 {
	  hParamIntArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamIcArray[*numDec],(hParamIntArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     }
   /*----------------------------------------------------------------------*/
   /* Setup the parameter entry for the Dummy pointer     	    	   */
   /*----------------------------------------------------------------------*/
 
       Bcnt = 0;
       Acnt = 0;
       Bidx = 0;
       Cidx = 0;	
       Ccnt = 1;
	   option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_XMT_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_AB, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );
	 edmaParamSetup(hParamDummyArray[*numDec], option, 0,
                     0, Acnt, Bcnt,
                     Ccnt, Bidx, Cidx, (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);

   /*------------------------------------------------------------------*/
   /*                    SETUP FOR RECEIVE CHANNEL                     */
   /*------------------------------------------------------------------*/

   /*------------------------------------------------------------------*/
   /* (4) EDMA Channel Setup for REVT - HDMEM    		       */
   /*------------------------------------------------------------------*/
   
	/* Rcv Channel Open */
    if(*numDec == 0)
    {
    chAttr.regionNum = CSL_EDMA3_REGION_1;		
    chAttr.chaNum = CSL_EDMA3_CHA_TCP2REVT;

    hEdmaTcpRevt = CSL_edma3ChannelOpen(&edmaTcpRcvObj,
                            	       CSL_EDMA3,		
                            	       &chAttr,
                            	       &chStatus);

	
	CSL_edma3HwChannelSetupQue(hEdmaTcpRevt,CSL_EDMA3_QUE_0);
	CSL_edma3HwChannelSetupParam(hEdmaTcpRevt,CSL_EDMA3_CHA_TCP2REVT);

    ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
    hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);
    }
    else
    {

     ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6));
     hParamDecArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);

    }
	
	 ChannelNum = (CSL_EDMA3_CHA_TCP2REVT - (*numDec * 6)  + 2);
     hParamOpArray[*numDec] = CSL_edma3GetParamHandle(hEdmaTcpRevt, ChannelNum, &status);

	/* Acquire basic param and set it up Parameter entry Basic Setup */
	
     option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_DIS,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             );           
  	
					            	
    /* Hard decision data */

    Acnt = 8 * (frame_length/64);
    if((frame_length%64)!=0)
    {
    Acnt = 8 * ((frame_length/64)+1);
    }	
    Bcnt = 1;
    Ccnt = 1;
    Bidx = 0;
    Cidx = 0;	    
	
    edmaParamSetup (hParamDecArray[*numDec], option, (Uint32 *)TCP_HDMEM_ADDR,
		   (Uint32 *)hardDecisions, Acnt, Bcnt, Ccnt, Bidx, Cidx,
		   hParamOpArray[*numDec]);
     if(*numDec > 0)
	 {
       hParamOpArray[*numDec - 1]->LINK_BCNTRLD = CSL_EDMA3_LINKBCNTRLD_MAKE(hParamDecArray[*numDec],(hParamOpArray[*numDec - 1]->LINK_BCNTRLD) >> 16);
     }
   /*------------------------------------------------------------------*/
   /* (5) EDMA Channel Setup for REVT - OPMEM                          */
   /*------------------------------------------------------------------*/
	option = CSL_EDMA3_OPT_MAKE( CSL_EDMA3_ITCCH_DIS, \
                                              CSL_EDMA3_TCCH_DIS, \
                                              CSL_EDMA3_ITCINT_DIS, \
                                              CSL_EDMA3_TCINT_EN,\
                                              CSL_RCV_TCC, CSL_EDMA3_TCC_NORMAL,\
                                              CSL_EDMA3_FIFOWIDTH_NONE, \
                                              CSL_EDMA3_STATIC_DIS, \
                                              CSL_EDMA3_SYNC_A, \
                                              CSL_EDMA3_ADDRMODE_INCR, \
                                              CSL_EDMA3_ADDRMODE_INCR
                                             ); 

	
	Acnt = TCP_OP_CONF_SIZE;
	Bcnt = 1;
	Ccnt = 1;
	Bidx = 0;
	Cidx = 0;
	
    edmaParamSetup (hParamOpArray[*numDec], option, (Uint32 *)TCP_OPMEM_ADDR,
                    (Uint32 *)tcpOutParms, Acnt, Bcnt, Ccnt, Bidx, Cidx,
                    (CSL_Edma3ParamHandle)CSL_EDMA3_LINK_NULL);  
    
}

void edma2MultChnlEnable()
{
    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  = 1 << CSL_XMT_TCC;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpXevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);

    CommonregionIntr.region =  CSL_EDMA3_REGION_1  ;   
    CommonregionIntr.intr  |= 1 << CSL_RCV_TCC;   
    CommonregionIntr.intrh  =  0x0000 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE, &CommonregionIntr);
    CSL_edma3HwChannelControl(hEdmaTcpRevt,CSL_EDMA3_CMD_CHANNEL_ENABLE,NULL);
}

