/*****************************************************************************\
*           TEXAS INSTRUMENTS INCORPORATED PROPRIETARY INFORMATION           
*                                                                            
*  Property of Texas Instruments 
*  For  Unrestricted  Internal  Use  Only 
*  Unauthorized reproduction and/or distribution is strictly prohibited.  
*  This product is protected under copyright law and trade secret law 
*  as an unpublished work.  
*  Created 2004, (C) Copyright 2003 Texas Instruments.  All rights reserved.
*------------------------------------------------------------------------------
*  Filename       : tcp2_interrupt.c
*  Date Created   : Oct 18, 2004
*  Last Modified  : Feb 24, 2005
*  Description    : Performs Interrupt Hookup for EDMA and TCP2
*  History        :
*  Project        : Himalaya
*  Author         : Manohar
\*****************************************************************************/


#include <stdio.h>
#include <csl.h>
#include <csl_edma3.h>	// changed TCP to TCP2 by Thara
#include <csl_intc.h>
#include "tcp2_edma.h"
#include "val_util.h"

/*****************************************************************************\
* Global data and variable declarations for Interrupt Controller              *
******************************************************************************/

Uint32 UserNumber = 1;
Uint32 profileStartLow[12][3];
Uint32 profileStartHigh[12][3];
Uint32 profileStopLow[12][3];
Uint32 profileStopHigh[12][3];
Uint32 Channel_Count = 0;

extern Uint32 ipr[];
extern volatile Uint32 TxEdmaCount;
extern volatile Uint32 RxEdmaCount;
extern volatile Uint32 RxEdmaDummyCount;
extern Uint32 tcpError;
extern CSL_Edma3CmdIntr            regionIntr;
volatile unsigned long long int Profile64;

CSL_IntcContext context; 
CSL_IntcEventHandlerRecord Record[3];
CSL_IntcHandle hIntcEdma;
CSL_IntcHandle hIntcTcp;
CSL_IntcObj intcEdmaObj;
CSL_IntcObj intcTcpObj;
CSL_Edma3CmdIntr Intr;

extern CSL_Edma3Handle    hModule;
extern CSL_Edma3ChannelHandle      hEdmaTcpRevt1;	//changed CSL_EdmaChanHandle to CSL_Edma3ChannelHandle by Thara

void Setup_Interrupt(void);
void Setup_Interrupt_TimeStamp(void);
void Setup_Interrupt_SpmemAccess(void);
interrupt void EDMA_ISR_TimeStamp(void);
interrupt void EDMA_ISR(void);
interrupt void EDMA_ISR_SpmemAccess(void);
interrupt void TCP2_ISR(void);


/*************************************************************************\
*		       Setup_Interrupt
\*************************************************************************/

void Setup_Interrupt()
{
//    CSL_IntcContext context;
    CSL_IntcParam vectId1,vectId2;
    CSL_IntcGlobalEnableState state;
//    CSL_IntcEventHandlerRecord Record[3];

    /* Setup the global Interrupt */
    context.numEvtEntries = 3;
    context.eventhandlerRecord = Record; 
    CSL_intcInit(&context);
	
    /* Enable NMIs  */
    CSL_intcGlobalNmiEnable();

    /* Enable Global Interrupts  */
    CSL_intcGlobalEnable(&state);

    /* Opening a handle for the Global Edma Event */
    vectId1 = CSL_INTC_VECTID_8;

    hIntcEdma = CSL_intcOpen(&intcEdmaObj,
                             CSL_INTC_EVENTID_EDMA3CC_INT1,
                             &vectId1,
                             NULL);
    
    /* Opening a handle for the Global Tcp Event */
    vectId2 = CSL_INTC_VECTID_9;

    hIntcTcp = CSL_intcOpen(&intcTcpObj,
                            CSL_INTC_EVENTID_TCP2_INT,
                            &vectId2,
                            NULL); 
    
    /* Hook the EDMA ISR */
    CSL_intcHookIsr(vectId1,&EDMA_ISR);

    /* Hook the TCP ISR */
    CSL_intcHookIsr(vectId2,&TCP2_ISR);

    /* Clear the Interrupt */
    CSL_intcHwControl(hIntcEdma,CSL_INTC_CMD_EVTCLEAR,NULL);
    CSL_intcHwControl(hIntcTcp,CSL_INTC_CMD_EVTCLEAR,NULL);

    /* Enable the Interrupt */
    CSL_intcHwControl(hIntcEdma,CSL_INTC_CMD_EVTENABLE,NULL);
    CSL_intcHwControl(hIntcTcp,CSL_INTC_CMD_EVTENABLE,NULL);
    Intr.region = CSL_EDMA3_REGION_1 ;
    Intr.intr   = 0xFFFFFFFF ;
    Intr.intrh  = 0x0 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTR_ENABLE,&Intr);    

}

/*************************************************************************\
*		       Setup_Interrupt
\*************************************************************************/

void Setup_Interrupt_SpmemAccess()
{
    CSL_IntcContext context;
    CSL_IntcParam vectId1,vectId2;
    CSL_IntcGlobalEnableState state;
//	CSL_Status intStat;

    /* Setup the global Interrupt */
    context.numEvtEntries = 3;
    context.eventhandlerRecord = Record; 
    CSL_intcInit(&context);
	
    /* Enable NMIs  */
    CSL_intcGlobalNmiEnable();

    /* Enable Global Interrupts  */
    CSL_intcGlobalEnable(&state);

    /* Opening a handle for the Global Edma Event */
    vectId1 = CSL_INTC_VECTID_8;

    hIntcEdma = CSL_intcOpen(&intcEdmaObj,
                                     CSL_INTC_EVENTID_EDMA3CC_INT1,
                                     &vectId1,
                                     NULL);
    
    /* Opening a handle for the Global Tcp Event */
    vectId2 = CSL_INTC_VECTID_9;

    hIntcTcp = CSL_intcOpen(&intcTcpObj,
                                     CSL_INTC_EVENTID_TCP2_INT,
                                     &vectId2,
                                     NULL); 
    
    /* Hook the EDMA ISR */
    CSL_intcHookIsr(vectId1,&EDMA_ISR_SpmemAccess);

    /* Hook the TCP ISR */
    CSL_intcHookIsr(vectId2,&TCP2_ISR);

    /* Clear the Interrupt */
    CSL_intcHwControl(hIntcEdma,CSL_INTC_CMD_EVTCLEAR,NULL);
    CSL_intcHwControl(hIntcTcp,CSL_INTC_CMD_EVTCLEAR,NULL);

    /* Enable the Interrupt */
    CSL_intcHwControl(hIntcEdma,CSL_INTC_CMD_EVTENABLE,NULL);
    CSL_intcHwControl(hIntcTcp,CSL_INTC_CMD_EVTENABLE,NULL);
}

/*************************************************************************\
*		       Setup_Interrupt_TimeStamp
\*************************************************************************/


void Setup_Interrupt_TimeStamp()
{
    CSL_IntcContext context;
    CSL_IntcParam vectId1,vectId2;
    CSL_IntcGlobalEnableState state;

    /* Setup the global Interrupt */
    context.numEvtEntries = 3;
    context.eventhandlerRecord = Record; 
    CSL_intcInit(&context);
	
    /* Enable NMIs  */
    CSL_intcGlobalNmiEnable();

    /* Enable Global Interrupts  */
    CSL_intcGlobalEnable(&state);

    /* Opening a handle for the Global Edma Event */
    vectId1 = CSL_INTC_VECTID_8;

    hIntcEdma = CSL_intcOpen(&intcEdmaObj,
                                     CSL_INTC_EVENTID_EDMA3CC_INT1,
                                     &vectId1,
                                     NULL);
    
    /* Opening a handle for the Global Tcp Event */
    vectId2 = CSL_INTC_VECTID_9;

    hIntcTcp = CSL_intcOpen(&intcTcpObj,
                                     CSL_INTC_EVENTID_TCP2_INT,
                                     &vectId2,
                                     NULL);
    
    /* Hook the EDMA ISR */
    CSL_intcHookIsr(vectId1,&EDMA_ISR_TimeStamp); 

    /* Hook the TCP ISR */
    CSL_intcHookIsr(vectId2,&TCP2_ISR);

    /* Clear the Interrupt */
    CSL_intcHwControl(hIntcEdma,CSL_INTC_CMD_EVTCLEAR,NULL);
    CSL_intcHwControl(hIntcTcp,CSL_INTC_CMD_EVTCLEAR,NULL);

    /* Enable the Interrupt */
    CSL_intcHwControl(hIntcEdma,CSL_INTC_CMD_EVTENABLE,NULL);
    CSL_intcHwControl(hIntcTcp,CSL_INTC_CMD_EVTENABLE,NULL);
}

/*************************************************************************\
*		        EDMA_ISR
\*************************************************************************/
interrupt void EDMA_ISR(void)
{
  CSL_Edma3CmdIntr            regionIntr;

  regionIntr.region = CSL_EDMA3_REGION_1;
  regionIntr.intr = 0;
  regionIntr.intrh = 0;
  CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&regionIntr);
    
  VAL_TRACE(0x44444444);

    do
    {
        if(regionIntr.intr & (1 << CSL_XMT_TCC))
        {
            TxEdmaCount++;
        }

        if(regionIntr.intr & (1 << CSL_RCV_TCC))
        {
            RxEdmaCount++;
        }

	CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTRPEND_CLEAR,&regionIntr);
	CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&regionIntr);

    }while(regionIntr.intr != 0);

    regionIntr.region = CSL_EDMA3_REGION_1 ;
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTRPEND_CLEAR,&regionIntr);
    CSL_edma3HwControl(hModule, CSL_EDMA3_CMD_INTR_EVAL, &regionIntr.region); 
}

/*************************************************************************\
*		        EDMA_ISR_SpmemAcess
\*************************************************************************/

interrupt void EDMA_ISR_SpmemAccess(void)
{
	CSL_Edma3CmdIntr            regionIntr;
  // CSL_edmaGetHwRegionStatus(shHandle, CSL_EDMA_QUERY_REGION_INTERRUPT_PENDSTATUS, ipr);
   regionIntr.region = CSL_EDMA3_REGION_1;
    regionIntr.intr = 0;
    regionIntr.intrh = 0;
    CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&regionIntr);
    
       /* CSL_edmaGetHwRegionStatus(shHandle, CSL_EDMA_QUERY_REGION_INTERRUPT_PENDSTATUS, ipr);*/
    VAL_TRACE(0x44444444);

    do
    {
        if(regionIntr.intr & (1 << CSL_XMT_TCC))
        {
            TxEdmaCount++;
        }

      	if(regionIntr.intr & (1 << (CSL_XMT_TCC + 1)))
        {
            TxEdmaCount++;
        }

 	if(regionIntr.intr & (1 << (CSL_XMT_TCC + 2)))
        {
            TxEdmaCount++;
        }

        if(regionIntr.intr & (1 << CSL_RCV_TCC))
        {
            RxEdmaCount++;
        }

  	if(regionIntr.intr & (1 << (CSL_RCV_TCC+1)))
        {
            RxEdmaCount++;
        }

		CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTRPEND_CLEAR,&regionIntr);
		CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&regionIntr);

       // CSL_edmaHwRegionControl(shHandle,CSL_EDMA_CMD_REGION_INTERRUPT_CLEAR,ipr);
       // CSL_edmaGetHwRegionStatus(shHandle, CSL_EDMA_QUERY_REGION_INTERRUPT_PENDSTATUS, ipr);

    }while(regionIntr.intr != 0);
     CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTRPEND_CLEAR,&regionIntr);

     

   /* }while(ipr[0] != 0);

    CSL_edmaHwRegionControl(shHandle,CSL_EDMA_CMD_REGION_INTERRUPT_CLEAR,ipr);*/
    /* Set Interrupt EVAL */
/*    CSL_edmaHwRegionControl(shHandle, CSL_EDMA_CMD_REGION_INTERRUPT_EVAL, NULL); */
	

}

/*************************************************************************\
*		        EDMA_ISR_TimeStamp
\*************************************************************************/

interrupt void EDMA_ISR_TimeStamp(void)
{

    CSL_Edma3CmdIntr            regionIntr;

    regionIntr.region = CSL_EDMA3_REGION_1;
    regionIntr.intr = 0;
    regionIntr.intrh = 0;
    CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&regionIntr);
    
    VAL_TRACE(0x44444444);

    do
    {
        if(regionIntr.intr & (1 << CSL_XMT_TCC))
        {
            TxEdmaCount++;

            // Populate the Time for XMT completion of Channel "i" 
			Profile64=CSL_tscRead();
            profileStopLow[UserNumber][0] = Profile64;
            profileStopHigh[UserNumber][0] = (Profile64 >> 32); 
        }

        if(regionIntr.intr & (1 << CSL_RCV_TCC_DUMMY))
        {
            RxEdmaDummyCount++;

            // Populate the Time for TCP Process completion of Channel "i" 
			Profile64=CSL_tscRead();
            profileStopLow[UserNumber][1] = Profile64;
            profileStopHigh[UserNumber][1] = (Profile64 >> 32);
            CSL_edma3HwChannelControl(hEdmaTcpRevt1,CSL_EDMA3_CMD_CHANNEL_SET,NULL);
        }
       
        if(regionIntr.intr & (1 << CSL_RCV_TCC))
        {
            RxEdmaCount++;

            // Populate the Time for RCV completion of Channel "i" 
			Profile64=CSL_tscRead();
            profileStopLow[UserNumber][2] = Profile64;
            profileStopHigh[UserNumber][2] = (Profile64 >> 32);

            UserNumber++;
      
            // Populate the Time before next Channel is started 
			Profile64=CSL_tscRead();
            profileStartLow[UserNumber][0] = Profile64;
            profileStartHigh[UserNumber][0] = (Profile64 >> 32);
        }

	CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTRPEND_CLEAR,&regionIntr);
	CSL_edma3GetHwStatus(hModule,CSL_EDMA3_QUERY_INTRPEND,&regionIntr);
    } while(regionIntr.intr != 0);
    regionIntr.region = CSL_EDMA3_REGION_1;    
    CSL_edma3HwControl(hModule,CSL_EDMA3_CMD_INTRPEND_CLEAR,&regionIntr);
    CSL_edma3HwControl(hModule, CSL_EDMA3_CMD_INTR_EVAL, &regionIntr.region); 
}


/*************************************************************************\
*		        	TCP2 ISR
\*************************************************************************/
interrupt void TCP2_ISR(void)
{
    VAL_TRACE(0xAAAA1100);
    tcpError++;
}
