/*****************************************************************************\
*           TEXAS INSTRUMENTS INCORPORATED PROPRIETARY INFORMATION           
*                                                                            
*  Property of Texas Instruments 
*  For  Unrestricted  Internal  Use  Only 
*  Unauthorized reproduction and/or distribution is strictly prohibited.  
*  This product is protected under copyright law and trade secret law 
*  as an unpublished work.  
*  Created 2004, (C) Copyright 2003 Texas Instruments.  All rights reserved.
*------------------------------------------------------------------------------
*  Filename       : tcp2_util.c
*  Date Created   : Oct 09, 2004
*  Last Modified  : Jun 14, 2005
*  Description    : Utility functions for the TCP2 module
*  History        :
*  Project        : Himalaya
*  Author         : Manohar
\*****************************************************************************/

#include <csl.h>
#include "val_util.h"
#include <csl_tcp2.h>
#include "tcp2_edma.h"

Int32 tcpCheckResults(Uint32 *actual, Uint32 *reference, Uint32 size);
Int32 tcpCheckparams(Uint32 *actual, Uint32 *reference, Uint32 size);
Int32 tcpCheckResults_Spmode(Uint32 *actual, Uint32 *reference, Uint32 size); 
Int32 tcpCheckResults_Spmode_Bigend(Uint32 *actual, Uint32 *reference, Uint32 size); 
//void TCP2_start(void);
void TCP2_PowerSaverEnable(void);
void TCP2_storeTailbits(Uint32 *TailConfig, TCP2_ConfigIc *restrict configIc);

/****************************************************************************\
 * tcpCheckResults: Verify that the hard decisions and output parameters sent*
 * by the TCP are correct. If the hard decisions are not correct, the index *
 * of the first incorrect value is returned. If the output parameters (the  *
 * number of iterations) is not correct, then a -1 is returned. If all      *
 * values are correct then a 0 is returned.                                 *
\****************************************************************************/

Int32 tcpCheckResults(Uint32 *actual, Uint32 *reference, Uint32 size)
{
    Uint32 i;
    Uint32 mismatch=0;
    Uint32 numWords;
	
   numWords = 2 * (size/64);
   if((size%64)!=0)
   numWords = 2 * ((size/64) + 1 );

   for (i=0; i < numWords; i++)
   {
	if (actual[i] != reference[i])
       {
       mismatch++;
       VAL_TRACE(0xDEAD2000);        
       VAL_TRACE(i);
       VAL_TRACE(0xDEAD2001);
       VAL_TRACE(reference[i]);   
       VAL_TRACE(0xDEAD2002); 
       VAL_TRACE(actual[i]);
       break;
       } /* end if actual != reference */
   } /* end for i */
    VAL_TRACE(mismatch);          
    if (mismatch)
    {    
    VAL_TRACE(0xDEAD0BAD);   	/* return index for 1st error that occured. */
    VAL_TRACE(i+1);             /* Index will range from 1 to framelength   */            
    return(FAIL);    
    }
    else
    {
    return(PASS);              /* no error */
    }
} /* end checkResults() */

Int32 tcpCheckparams(Uint32 *actual, Uint32 *reference, Uint32 size)
{
    Uint32 i;
    Uint32 mismatch=0;
    Uint32 numWords = size;

    for (i=0; i<numWords; i++)
    {
	if (actual[i] != reference[i])
       {
       mismatch++;
       VAL_TRACE(0xDEAD1000);        
       VAL_TRACE(i);
       VAL_TRACE(0xDEAD1001);
       VAL_TRACE(reference[i]);   
       VAL_TRACE(0xDEAD1002); 
       VAL_TRACE(actual[i]);
       break;
       } /* end if actual != reference */
   } /* end for i */

    VAL_TRACE(mismatch);          
    if (mismatch)
    {    
    VAL_TRACE(0xDEAD0BAD);   	/* return index for 1st error that occured. */
    VAL_TRACE(i+1);             /* Index will range from 1 to framelength   */            
    return(FAIL);    
    }
    else
    {
    return(PASS);              /* no error */
    }

} /* end checkResults() */

//void TCP2_start()
//{
//   (*(volatile Uint32*)(TCP_ENDIAN_MODE_REG_ADDRESS)) = (Uint32) TCP_ENDIAN_WORD;    
//   (*(volatile Uint32*)(TCP_EMULATION_CONTROL_REG_ADDRESS)) = (Uint32) TCP_EMULATION_WORD;    
//   (*(volatile Uint32*)(TCP_EXECUTION_REG_ADDRESS)) = (Uint32) TCP_START;  
//}

/****************************************************************************\
 * tcpCheckResults: Verify that the hard decisions and output parameters sent*
 * by the TCP are correct. If the hard decisions are not correct, the index *
 * of the first incorrect value is returned. If the output parameters (the  *
 * number of iterations) is not correct, then a -1 is returned. If all      *
 * values are correct then a 0 is returned.                                 *
\****************************************************************************/

Int32 tcpCheckResults_Spmode(Uint32 *actual, Uint32 *reference, Uint32 size) 
{

  	Int32 i;
        Int32 mismatch=0;
        Int32 paddedWordIndex = 0;
        Int32 paddedHigherWordIndex = -1;
        Int32 paddedWordMask = 0 ;
        Int32 paddedHigherWordMask = 0 ;
        Int32 maskedComparedData = 0;
        Int32 maskedRefData = 0;
        Uint32 dataLength,endianess,extrinsicFormat,frameLength;	

        endianess = 1; /* LittleEndian */
        extrinsicFormat = (TCP_ENDIAN_WORD >>1) &1;

        frameLength = size;
        dataLength = 2 * (frameLength/8);
        if((frameLength%8)!= 0)
        dataLength = 2 * ((frameLength/8) + 1);

        /* paddedWordIndex is either (dataLength-1) or (dataLength-2) */
        /* number of extrinsic bytes = frame length */

        if ((frameLength % 8) == 0 )
        {
            paddedWordIndex = -1;
            paddedHigherWordIndex = -1;
        }
        else
        {
            
            paddedWordIndex = (frameLength % 8) >= 4 ? (dataLength-1) : (dataLength-2) ;
            if( (frameLength % 8) < 4 )
            {
                paddedHigherWordIndex = (dataLength-1);
            }
        }
        /* paddedWordMask depends on endianness and extrinsic format (native or 32 bit)
           In case of big endian - 32 bit format words are stored as 3210
                                                                     7654
                      big endian - 8 bit format words are stored as  0123
                                                                     4567 ... problem 

                      in case of little endian, words are stored as  3210
                                                                     7654
             
           so padded mask depends on these..                                                          
         */

        paddedWordMask = (1<< (8*(frameLength % 4) )) - 1;
        
        if( ( endianess == BIG_ENDIAN) && (extrinsicFormat == NATIVE_FORMAT) )
        {
                /* the useful bits in padded word are at the left end */
                paddedWordMask = paddedWordMask << ( 32 - (8*(frameLength % 4)) );
        }

            
        
        for (i=0; i< dataLength ; i++)
        {
                if ( i == paddedWordIndex)
                {
                    maskedComparedData = actual[i] & paddedWordMask;
                    maskedRefData = reference[i] & paddedWordMask;
                }
                else if ( i == paddedHigherWordIndex ) 
                {
                    maskedComparedData = actual[i] & paddedHigherWordMask;
                    maskedRefData = reference[i] & paddedHigherWordMask ;
                }
                else 
                {
                    maskedComparedData = actual[i];
                    maskedRefData = reference[i] ;
                }

                if (maskedComparedData != maskedRefData)
	        {
                    mismatch++;
       		    VAL_TRACE(0xDEAD2000);        
       		    VAL_TRACE(i);
       		    VAL_TRACE(0xDEAD2001);
       		    VAL_TRACE(reference[i]);   
                    VAL_TRACE(0xDEAD2002); 
                    VAL_TRACE(actual[i]);
                    break;
               	}

   
      }

	VAL_TRACE(mismatch);          
    	if (mismatch)
    	{    
    	   VAL_TRACE(0xDEAD0BAD);   	/* return index for 1st error that occured. */
           VAL_TRACE(i+1);              /* Index will range from 1 to framelength   */            
           return(FAIL);    
        }
        else
        {
           return(PASS);               /* no error */
        }


   }


Int32 tcpCheckResults_Spmode_Bigend(Uint32 *actual, Uint32 *reference, Uint32 size) 
{

  	Int32 i;
        Int32 mismatch=0;
        Int32 paddedWordIndex = 0;
        Int32 paddedHigherWordIndex = -1;
        Int32 paddedWordMask = 0 ;
        Int32 paddedHigherWordMask = 0 ;
        Int32 maskedComparedData = 0;
        Int32 maskedRefData = 0;
        Uint32 dataLength,endianess,extrinsicFormat,frameLength;	

        endianess = 0; /* Big Endain */
        extrinsicFormat = (TCP_BIG_ENDIAN_WORD >> 1) & 1;

        frameLength = size;
        dataLength = 2 * (frameLength/8);
        if((frameLength%8)!= 0)
        dataLength = 2 * ((frameLength/8) + 1);

        /* paddedWordIndex is either (dataLength-1) or (dataLength-2) */
        /* number of extrinsic bytes = frame length */

        if ((frameLength % 8) == 0 )
        {
            paddedWordIndex = -1;
            paddedHigherWordIndex = -1;
        }
        else
        {
            
            paddedWordIndex = (frameLength % 8) >= 4 ? (dataLength-1) : (dataLength-2) ;
            if( (frameLength % 8) < 4 )
            {
                paddedHigherWordIndex = (dataLength-1);
            }
        }
        /* paddedWordMask depends on endianness and extrinsic format (native or 32 bit)
           In case of big endian - 32 bit format words are stored as 3210
                                                                     7654
                      big endian - 8 bit format words are stored as  0123
                                                                     4567 ... problem 

                      in case of little endian, words are stored as  3210
                                                                     7654
             
           so padded mask depends on these..                                                          
         */

        paddedWordMask = (1<< (8*(frameLength % 4) )) - 1;
        
        if( ( endianess == BIG_ENDIAN) && (extrinsicFormat == NATIVE_FORMAT) )
        {
                /* the useful bits in padded word are at the left end */
                paddedWordMask = paddedWordMask << ( 32 - (8*(frameLength % 4)) );
        }

        
        for (i=0; i< dataLength ; i++)
        {
                if ( i == paddedWordIndex)
                {
                    maskedComparedData = actual[i] & paddedWordMask;
                    maskedRefData = reference[i] & paddedWordMask;
                }
                else if ( i == paddedHigherWordIndex ) 
                {
                    maskedComparedData = actual[i] & paddedHigherWordMask;
                    maskedRefData = reference[i] & paddedHigherWordMask ;
                }
                else 
                {
                    maskedComparedData = actual[i];
                    maskedRefData = reference[i] ;
                }

                if (maskedComparedData != maskedRefData)
	        {
                    mismatch++;
       		    VAL_TRACE(0xDEAD2000);        
       		    VAL_TRACE(i);
       		    VAL_TRACE(0xDEAD2001);
       		    VAL_TRACE(reference[i]);   
                    VAL_TRACE(0xDEAD2002); 
                    VAL_TRACE(actual[i]);
                    break;
               	}

   
      }

	VAL_TRACE(mismatch);          
    	if (mismatch)
    	{    
    	   VAL_TRACE(0xDEAD0BAD);   	/* return index for 1st error that occured. */
           VAL_TRACE(i+1);              /* Index will range from 1 to framelength   */            
           return(FAIL);    
        }
        else
        {
           return(PASS);               /* no error */
        }


   }

/****************************************************************/
// Function to store Tail bit values from .asm file into Input configuration register memory 
/****************************************************************/
void TCP2_storeTailbits(
		Uint32	          *TailConfig,
		TCP2_ConfigIc    *restrict configIc
) 
{
	Uint32 gie;
	gie =  _disable_interrupts ();
	    
    configIc->ic6  = *(TailConfig);
    configIc->ic7  = *(TailConfig+1);
    configIc->ic8  = *(TailConfig+2);
    configIc->ic9  = *(TailConfig+3);
    configIc->ic10 = *(TailConfig+4);
    configIc->ic11 = *(TailConfig+5);
    
	_restore_interrupts (gie);
}
