/****************************************************************************\
 *           Copyright (C) 2000 Texas Instruments Incorporated.             *
 *                           All Rights Reserved                            *
 *                                                                          *
 * GENERAL DISCLAIMER                                                       *
 * ------------------                                                       *
 * All software and related documentation is provided "AS IS" and without   *
 * warranty or support of any kind and Texas Instruments expressly disclaims*
 * all other warranties, express or implied, including, but not limited to, *
 * the implied warranties of merchantability and fitness for a particular   *
 * purpose.  Under no circumstances shall Texas Instruments be liable for   *
 * any incidental, special or consequential damages that result from the    *
 * use or inability to use the software or related documentation, even if   *
 * Texas Instruments has been advised of the liability.                     *
 ****************************************************************************
 *                                                                          *
 * Written by :                                                             *
 *            Sebastien Tomas  (VCP2/TCP2)                                  *
 *            Texas Instruments                                             * 
 *            12 May, 2005                                                  *
 *                                                                          *
 * The purpose of this test is to evaluate Bit-Error Rate performance for   *
 * both VCP2 and TCP2 co-processors in the TCI6482 (Himalaya)               *
 *									                                        *
 * This test should not be redistributed for any reason without             *
 * permission.                                                              *
 *                                                                          *
 Modified by Brighton Feng 12 July, 2006 for VCP2/TCP2 channel density test.
\***************************************************************************/

#include "turboEncode.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include "csl_tcp2Aux.h"
//#include "TCP2_Channel_Density.h"


#define VERBOSE_MUX 0
#define VERBOSE_RSC 0
#define VERBOSE_ININTER 0

Int32 mux3GPP(const Uint8 *outrsc1,const Uint8 *outrsc2, const Uint32 size, const Uint32 K,const Uint32 rate, const Uint32 tails, Uint32 *out)
{
	Uint32 i,n=0,shift=0;
	Uint32 nMax=TCP2_normalCeil((size*rate+tails),32)-1;
	Uint32 value;
	
	
	out[0]=0;
	
	/* info bits */
	for (i=0;i<size;i++)
	{
		/* x */
		value=outrsc1[i]&0x1;
		out[n]|= value<<shift;
		shift++;
#if VERBOSE_MUX
		printf("\n%d ",value);
#endif
		if (shift==32)
		{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
		}

		/* p0 */
		value=(outrsc1[i]>>1)&0x1;
		out[n]|= value<<shift;
		shift++;
#if VERBOSE_MUX
		printf("%d ",value);
#endif
		if (shift==32)
		{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
		}

		/* po' */
		value=(outrsc2[i]>>1)&0x1;
		out[n]|= value<<shift;
		shift++;
#if VERBOSE_MUX
		printf("%d ",value);
#endif
		if (shift==32)
		{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
		}
	}
	
	/* tail bits */
	/* x */
	value=(outrsc1[i]&0x1);
	out[n]|=value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("\n%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* p0 */
	value=(outrsc1[i]>>1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* x */
	value=(outrsc1[i+1]&0x1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d  ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* p0 */
	value=(outrsc1[i+1]>>1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}
		
	/* x */
	value=(outrsc1[i+2]&0x1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* p0 */
	value=(outrsc1[i+2]>>1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* x' */
	value=(outrsc2[i]&0x1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* p0' */
	value=(outrsc2[i]>>1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* x' */
	value=(outrsc2[i+1]&0x1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* p0' */
	value=(outrsc2[i+1]>>1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}
		
	/* x' */
	value=(outrsc2[i+2]&0x1);
	out[n]|= value<<shift;
	shift++;
#if VERBOSE_MUX
	printf("%d ",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}

	/* p0' */
	value=(outrsc2[i+2]>>1);
	out[n]|= value<<shift;
#if VERBOSE_MUX
	printf("%d\n",value);
#endif
	if (shift==32)
	{
			if (n<nMax)
			{
				out[++n]=0;
				shift=0;
			}
	}
	
	return 0;
	
}

Int32 mux3GPP2(const Uint8 *outrsc1, const Uint8 *outrsc2, const Uint32 size, const Uint32 K,const Uint32 rate, const Uint32 tails, Uint32 *out)
{
	Uint32 i,n=0,shift=0;
	Uint32 nMax=TCP2_normalCeil((size*rate+tails),32)-1;
	Uint32 value;
	Uint32 uiRsc1, uiRsc2, uiOut;
	Uint32 uiOutBits;
	Uint16 *output= (Uint16 *)out; 

	out[0]=0;
	
	switch (rate)
	{
		case 2:
			/* info bits */
			for (i=0;i<size;i++)
			{
				/* x */
				value=(outrsc1[i]&0x1);
				out[n]|= value<<shift;
				shift++;
#if VERBOSE_MUX
				printf("\n%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}

				/* p0 or p0'*/
				if (!(i%2))
				{
					value=(outrsc1[i]>>1)&0x1;
					out[n]|= value<<shift;
				}
				else
				{
					value= (outrsc2[i]>>1)&0x1;
					out[n]|= value<<shift;
				}
#if VERBOSE_MUX
				printf("%d ",value);
#endif
				shift++;
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}			
		
			}
	
			/* tail bits */
			/* x */
			value=(outrsc1[i]&0x1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("\n%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0 */
			value=(outrsc1[i]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* x */
			value=(outrsc1[i+1]&0x1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0 */
			value=(outrsc1[i+1]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
			
			/* x */
			value=(outrsc1[i+2]&0x1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	
	
			/* p0 */
			value=(outrsc1[i+2]>>1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* x' */
			value=(outrsc2[i]&0x1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value=(outrsc2[i]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i+1]&0x1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value=(outrsc2[i+1]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			/* x' */
			value=(outrsc2[i+2]&0x1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value= (outrsc2[i+2]>>1)&0x1;
			out[n]|= value<<shift;
#if VERBOSE_MUX
			printf("%d\n",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			break;
	
		case 3:
			/* info bits */
			for (i=0;i<size;i++)
			{
				/* x */
				value= (outrsc1[i]&0x1);
				out[n]|= value<<shift;
				shift++;
#if VERBOSE_MUX
				printf("\n%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}

				/* p0 */
				value=(outrsc1[i]>>1)&0x1;
				out[n]|= value<<shift;
				shift++;
#if VERBOSE_MUX
				printf("%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}			
		
				/* po' */	
				value= (outrsc2[i]>>1)&0x1;
				out[n]|= value<<shift;
				shift++;
#if VERBOSE_MUX
				printf("%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}
			}
	
			/* tail bits */
			/* x */
			value=(outrsc1[i]&0x1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("\n%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x */
			value=(outrsc1[i]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0 */
			value=(outrsc1[i]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("\n%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* x */
			value=(outrsc1[i+1]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x */
			value=(outrsc1[i+1]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0 */
			value=(outrsc1[i+1]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
			
			/* x */
			value=(outrsc1[i+2]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	
	
			/* x */
			value= (outrsc1[i+2]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	
	
			/* p0 */
			value=(outrsc1[i+2]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* x' */
			value=(outrsc2[i]&0x1);
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value=(outrsc2[i]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value= (outrsc2[i+1]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i+1]&0x1);
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value= (outrsc2[i+1]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			/* x' */
			value=(outrsc2[i+2]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i+2]&0x1);
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value=(outrsc2[i+2]>>1)&0x1;
			out[n]|= value<<shift;
#if VERBOSE_MUX
			printf("%d\n",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			break;
		
		case 4:
			/* info bits */
			for (i=0;i<size;i++)
			{
				/* x */
				value=(outrsc1[i]&0x1);
				out[n]|= value<<shift;
				shift++;
#if VERBOSE_MUX
				printf("\n%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}

				/* p0 */
				value=(outrsc1[i]>>1)&0x1;
				out[n]|= value<<shift;
				shift++;
#if VERBOSE_MUX
				printf("%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}			
		
				/* p0' or p1 */	
				if (!(i%2))
				{
					value=(outrsc1[i]>>2)&0x1;
					out[n]|= value<<shift;
				}
				else
				{
					value= (outrsc2[i]>>1)&0x1;
					out[n]|= value<<shift;
				}
				shift++;
#if VERBOSE_MUX
				printf("%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}

				/* p1' */	
				value=(outrsc2[i]>>2)&0x1;
				out[n]|= value<<shift;
				shift++;
#if VERBOSE_MUX
				printf("%d ",value);
#endif
				if (shift==32)
				{
					if (n<nMax)
					{
						out[++n]=0;
						shift=0;
					}
				}
			}
	
			/* tail bits */
			/* x */
			value= (outrsc1[i]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("\n%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x */
			value=(outrsc1[i]&0x1);
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0 */
			value=(outrsc1[i]>>1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* p1 */
			value= (outrsc1[i]>>2)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* x */
			value=(outrsc1[i+1]&0x1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x */
			value=(outrsc1[i+1]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0 */
			value=(outrsc1[i+1]>>1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
			
			/* p1 */
			value= (outrsc1[i+1]>>2)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
			
			/* x */
			value=(outrsc1[i+2]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	
	
			/* x */
			value=(outrsc1[i+2]&0x1);
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	
	
			/* p0 */
			value=(outrsc1[i+2]>>1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* p1 */
			value=(outrsc1[i+2]>>2)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}	

			/* x' */
			value= (outrsc2[i]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value=(outrsc2[i]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value= (outrsc2[i]>>2)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i+1]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i+1]&0x1);
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value=(outrsc2[i+1]>>1)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			/* p1' */
			value=(outrsc2[i+1]>>2)&0x1;
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			/* x' */
			value=(outrsc2[i+2]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* x' */
			value=(outrsc2[i+2]&0x1);
			out[n]|= value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}

			/* p0' */
			value=(outrsc2[i+2]>>1)&0x1;
			out[n]|=value<<shift;
			shift++;
#if VERBOSE_MUX
			printf("%d ",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			/* p1' */
			value= (outrsc2[i+2]>>2)&0x1;
			out[n]|=value<<shift;
#if VERBOSE_MUX
			printf("%d\n",value);
#endif
			if (shift==32)
			{
				if (n<nMax)
				{
					out[++n]=0;
					shift=0;
				}
			}
		
			break;
		case 5:
			/* info bits */
			value=0;
			uiOut=0;
			uiOutBits=0;
			//Information bits
			for (i=0;i<size;i++)
			{
				value	= (outrsc1[i])|((outrsc2[i]>>1)<<3);

				if(uiOutBits>((i*5)%16))
				{
					*output++=uiOut;
					uiOut>>=16;
				}
				uiOutBits=(i*5)%16;
				uiOut|= (value<<uiOutBits);
				
			}
			//Tail bits should be added here
			break;
		default:
			return 1;
			
	
	}
	
	return 0;
	
}
#ifdef __cplusplus
}
#endif



Int32 TurboEncode3GPP(const Uint32 *in, Uint32 *out, Uint16 *table,const Uint32 size, const Uint32 rate)
{
	const Uint16 feedforward[8]={0xD,0,0,0,0,0,0,0};
	const Uint16 feedback=0x3;
	const Uint32 K=4;
	const Uint32 nTails=12;
	Uint8 *outrsc1,*outrsc2;
	Uint32 *inInterleaved;
//	Uint32 i;
//	S8 filename[256];
//	FILE *file;

	if ((outrsc1=(Uint8*)calloc(sizeof(Uint8),size+K-1))==NULL)
		return 1;
	if ((outrsc2=(Uint8*)calloc(sizeof(Uint8),size+K-1))==NULL)
		return 1;
	if ((inInterleaved=(Uint32*)calloc(sizeof(Uint32),TCP2_normalCeil(size,32)))==NULL)
		return 1;

#if VERBOSE_RSC
		printf("\nRSC1\n");
#endif
	if (lrsc(in,outrsc1,size,feedforward,feedback))
		return 1;


	if (TurboInterleaverTableGenerator3GPP(table,size))
		return 1;
	if (TurboInterleaveBits(in,inInterleaved,size,table))
		return 1;

#if VERBOSE_ININTER		
		printf("\nin[0]=%08X",in[0]);
		printf("\nin[1]=%08X",in[1]);
		printf("\ninInterleaved[0]=%08X",inInterleaved[0]);
		printf("\ninInterleaved[1]=%08X",inInterleaved[1]);
		printf("\n");
#endif
#if VERBOSE_RSC
		printf("\nRSC2\n");
#endif
	if (lrsc(inInterleaved,outrsc2,size,feedforward,feedback))
		return 1;
	
#if VERBOSE_MUX
		printf("\nMUX\n");
#endif
	//Puncture
	if (mux3GPP(outrsc1,outrsc2,size,K,rate,nTails,out))
		return 1;
	
	
	free(outrsc1);
	free(outrsc2);
	free(inInterleaved);
	
	return 0;
}

Int32 TurboEncode3GPP2(const Uint32 *in, Uint32 *out, Uint16 *table,const Uint32 size, const Uint32 rate)
{
	const Uint16 feedforward[8]={0xD,0xF,0,0,0,0,0,0};
	const Uint16 feedback=0x3;
	const Uint32 K=4;
	Uint32 nTails;
	Uint8 *outrsc1,*outrsc2;
	Uint32 *inInterleaved;
//	Uint32 i;
//	S8 filename[256];
//	FILE *file;
//	Uint32 verbose=1;
	
	switch (rate)
	{
		case 2:
			nTails=12;
			break;
		case 3:
			nTails=18;
			break;
		case 4:
			nTails=24;
			break;
		case 5:
			nTails=30;
			break;
		default:
			return 1;
	}

	if ((outrsc1=(Uint8*)calloc(sizeof(Uint8),size+K-1))==NULL)
		return 1;
	if ((outrsc2=(Uint8*)calloc(sizeof(Uint8),size+K-1))==NULL)
		return 1;
	if ((inInterleaved=(Uint32*)calloc(sizeof(Uint32),TCP2_normalCeil(size,32)))==NULL)
		return 1;

#if VERBOSE_RSC
		printf("\nRSC1\n");
#endif
	if (lrsc(in,outrsc1,size,feedforward,feedback))
		return 1;

	if ((size>=40)&&(size<=256))
	{
		if (TurboInterleaverTableGenerator3GPP(table,size))
			return 1;
	}
	else
	{	
		if (TurboInterleaverTableGenerator3GPP2(table,size))
			return 1;
	}
	if (TurboInterleaveBits(in,inInterleaved,size,table))
		return 1;

#if VERBOSE_RSC
		printf("\nRSC2\n");
#endif
	if (lrsc(inInterleaved,outrsc2,size,feedforward,feedback))
		return 1;
	
	
#if VERBOSE_MUX
		printf("\nMUX\n");
#endif
	if (mux3GPP2(outrsc1,outrsc2,size,K,rate,nTails,out))
		return 1;

	free(outrsc1);
	free(outrsc2);
	free(inInterleaved);
	
	return 0;
}



Uint32 TurboInterleaveBits(const Uint32 *input, Uint32 *output, const Uint32 dataLen, const Uint16 *iltable)
{
	Uint32 i;  
	Uint32 sourceWord;
	Uint16 interleaverIndex;
	Uint32 sourceWordIndex;
	Uint32 sourceWordBitIndex;
	Uint32 destinationShift=0;
	Uint32 bit;
	Uint32 *out=output;
	const Uint16 *table=iltable;
	
	for (i=0;i<dataLen;i++)
	{
		if (((i & 0x1F)==0) && (i!=0))
		{
			out++;
			destinationShift=0;
		}

		/* get interleaver index */
		interleaverIndex=*(table++);
		/* determine which word and which bit has to be read from the input data */
		sourceWordIndex=interleaverIndex>>5;
		sourceWordBitIndex=interleaverIndex-sourceWordIndex*32;
		/* get bit */
		sourceWord=input[sourceWordIndex];
		bit=(sourceWord>>sourceWordBitIndex)&0x1;
		/* write bit in destination */
		*out|=bit<<destinationShift;
		destinationShift++;
	}  
	
	return 0;
}


typedef struct 
{    
  Uint32 prime, root;
} prime_unit;

Uint32 TurboInterleaverTableGenerator3GPP(Uint16 *T, Uint32 K)
{

  static Uint32 pattern_1_20[20]={19,9,14,4,0,2,5,7,12,18,10,8,13,17,3,1,16, 6,15,11};
  static Uint32 pattern_2_20[20]={19,9,14,4,0,2,5,7,12,18,16,13,17,15,3,1,6,11, 8,10};
  static Uint32 pattern_3_10[10]={ 9,8, 7,6,5,4,3,2, 1, 0};
  static Uint32 pattern_4_5 [ 5]={ 4,3, 2,1,0};

  static prime_unit prime_et_root[56]={
    {1,   0},{2,   1},{3,   2},{5,   2},{7,   3},{11,  2},{13,  2},
    {17,  3},{19,  2},{23,  5},{29,  2},{31,  3},{37,  2},{41,  6},{43,3},{47,  5},{53,  2},
    {59,  2},{61,  2},{67,  2},{71,  7},{73,  5},{79,  3},{83,  2},{89,3},{97,  5},{101, 2},
    {103, 5},{107, 2},{109, 6},{113, 3},{127, 3},{131, 2},{137, 3},{139,2},{149, 2},{151, 6},
    {157, 5},{163, 2},{167, 5},{173, 2},{179, 2},{181, 2},{191,19},{193,5},{197, 2},{199, 3},
    {211, 2},{223, 3},{227, 2},{229, 6},{233, 3},{239, 7},{241, 7},{251,6},{257, 3}};

 Uint32 A,B,lambda,lambda_1,root_mu;
// Uint32 i,j,x,y,z,k,i_o;
 Uint32 i,j,x,z,k,i_o;
 Uint32 BR[256], Q[20], MIS[20], *ROP;


 if ((K<39)||(K>5114))
 {
   return 1;
 }
/*
1. Assign values for the number of rows A, the number of columns B,
   the prime number lambda, and the primitive root mu depending on K:
*/
  if ((480<K)&&(K<531))
  {
    A=10; 
    lambda=53; 
    B=lambda; 
    root_mu=2; 
    lambda_1=lambda-1;
  } 
  else
  {
   if ((39<K)&&(K<160)) 
     A=5;
   else if ((159<K)&&(K<201)) 
     A=10; 
   else 
     A=20;
   x= K/A + ((K%A)>0) - 1;
//   for (i=0; ((i<56)&&(prime_et_root[i].prime<x)); i++);
   for (i=0; ((i<55)&&(prime_et_root[i].prime<x)); i++);
   lambda=prime_et_root[i].prime;
   
   lambda_1=lambda-1;
   B = lambda_1 + ((A*lambda_1)<K) + ((A*lambda)<K);

   root_mu=prime_et_root[i].root;
 }

/*
2. Select the row order pattern ROP out of Pattern1, Pattern2,
   Pattern3, and Pattern4 depending on K:
           --
          |  Pattern4  if 39<K<160,
   ROP = -|  Pattern3  if 159<K<201 or 480<K<531,
          |  Pattern2  if 2280<K<2481 or 3160<K<3211,
          |  Pattern1  otherwise.
           --
*/

 if ((40<= K)&& (K<160))                                
   ROP=&pattern_4_5[0];
 else if (((159<K)&&(K<201)) || ((480<K)&&(K<531)))     
   ROP=&pattern_3_10[0];
 else if (((2280<K)&&(K<2481)) || ((3160<K)&&(K<3211))) 
   ROP=&pattern_2_20[0];
 else if (K<5115)                                       
   ROP=&pattern_1_20[0];
 else 
 	return 1;

/*
3. Construct the base sequence BR(j), j=0,1,2,...,lambda-2, by 
BR(0)=1 and
   BR(j) = (g*BR(j-1)) modulo lambda for j=1,2,...,lambda-2.
*/

 if (A==4) 
   x=BR[0]=lambda_1; 
 else 
   x=BR[0]=1;   
 for(i=1; i<lambda_1; i++)
  { x=(root_mu*x) % lambda;
    BR[i]=x;
  }

/*
4. Selection of the minimun prime integer sequence Q(i) for 
i=0,1,...,A-1:
   assign Q(0)=1 and choose a least prime number Q(i) such that
   gcd(Q(i),lambda-1)=1, Q(i)>6, and Q(i)>Q(i-1) for 
i=1,2,...,A-1.
*/

 Q[0]=1;
 x=prime_et_root[4].prime;
 for(i=1,j=4; (i<A)&&(x<lambda_1);)
   {
     if (lambda_1%x)
       { 
	 Q[i]=x;
	 i++;
       }
     j++;
     x=prime_et_root[j].prime;
   }

 for(; i<A;)
     {
       Q[i]=prime_et_root[j].prime;
       i++; j++;
     }
/* End of 4 */

/*
5. Calculate the minimum row index sequence MIS(i), i=0,1,...,A-1, by
   either
      MIS(i) = ROP(i)*B   if B=lambda-1
   or
      MIS(i) = ROP(i)*B+1 if B=lambda or B=lambda+1.
*/

 x=(lambda<=B);
 for(i=0; i<A; i++) 
   MIS[i]=ROP[i]*B+x;

/* memory for T */


/*
6. All elements of T(k) are same as ones obtained from the steps 
6.1-6.3:
*/

/* 6.1 */

 i_o=0; k=0; /* In C it is convenient to set k=0 instead of k=1 !! */
 if ((K==A*B)&&(B==(lambda+1)))
   {
     T[k]=MIS[i_o]+lambda; i_o=i_o+1; k=k+1;
   }
 for (j=0; j<lambda_1; j++)
   { 
     for (i=i_o; i<A; i++)
       {
	 z=MIS[i]+BR[ (j*Q[i])%lambda_1 ];
	 if ((z<K)||(z==K)) 
	   { 
	     T[k]=z; 
	     k=k+1; 
	   }
       }
     i_o=0;
   }

/* 6.2 */

 if (lambda_1<B)
   {
     for (i=0;i<A; i++)
       {
	 z=MIS[i];
	 if ((z<K)||(z==K)) 
	   { 
	     T[k]=z; 
	     k=k+1; 
	   }
       }
   }


/* 6.3 */

 if (lambda<B)
 {
   i_o=0;
   if (K==(A*B)) 
     { 
       T[k]=MIS[i_o]+BR[0]; 
       k=k+1; 
       i_o=i_o+1; 
     }
   for (i=i_o;i<A; i++)
     {
       z=MIS[i]+lambda;
       if ((z<K)||(z==K)) 
	 { 
	   T[k]=z; 
	   k=k+1; 
	 }
     }
 }
 
 for (i=0;i<K; i++)
 	T[i]-=1;
 
 return 0;

}


Uint32 GenerateTurboInterleaverParameter(Uint32 blockSize)
{
	Uint32 i=0;

	while ((double)blockSize/pow(2,i+5)>1) i++;
	
	return i;

} 
Uint16 InterleaverLookupTable(Uint16 addr, Int32 n)
{
  const Uint16 Table [ 32 ][ 7 ] = {
      {   5,  27,   3,  15,   3,  13,   1 },
      {  15,   3,  27, 127,   1, 335, 349 },
      {   5,   1,   15,  89,  5,  87, 303 },
      {  15,  15,  13,   1,  83,  15, 721 },
      {   1,  13,  29,  31,  19,  15, 973 },
      {   9,  17,   5,  15, 179,   1, 703 },
      {   9,  23,   1,  61,  19, 333, 761 },
      {  15,  13,  31,  47,  99,  11, 327 },
      {  13,   9,   3, 127,  23,  13, 453 },
      {  15,   3,   9,  17,   1,   1,  95 },
      {   7,  15,  15, 119,   3, 121, 241 },
      {  11,   3,  31,  15,  13, 155, 187 },
      {  15,  13,  17,  57,  13,   1, 497 },
      {   3,   1,   5, 123,   3, 175, 909 },
      {  15,  13,  39,  95,  17, 421, 769 },
      {   5,  29,   1,   5,   1,   5, 349 },
      {  13,  21,  19,  85,  63, 509,  71 },
      {  15,  19,  27,  17, 131, 215, 557 },
      {   9,   1,  15,  55,  17,  47, 197 },
      {   3,   3,  13,  57, 131, 425, 499 },
      {   1,  29,  45,  15, 211, 295, 409 },
      {   3,  17,   5,  41, 173, 229, 259 },
      {  15,  25,  33,  93, 231, 427, 335 },
      {   1,  29,  15,  87, 171,  83, 253 },
      {  13,   9,  13,  63,  23, 409, 677 },
      {   1,  13,   9,  15, 147, 387, 717 },
      {   9,  23,  15,  13, 243, 193, 313 },
      {  15,  13,  31,  15, 213,  57, 757 },
      {  11,  13,  17,  81, 189, 501, 189 },
      {   3,   1,   5,  57,  51, 313,  15 },
      {  15,  13,  15,  31,  15, 489,  75 },
      {   5,  13,  33,  69,  67, 391, 163 } };

  return(Table [ addr ][ n-4 ]);
        
} 
Uint16 BitReverse(Uint16 value, Int32 n)
{
  Uint16 temp;
  Int32 cont;
 
  temp = 0;
  for (cont = 0; cont < n; cont++)
  {
    if ((value & 0x1) == 0x1)
      {
         temp |= (1 << (n-1-cont));
      }
    value >>= 1; 
  }
  return(temp); 
}
Uint16 n_msb(Uint16 value,Uint32 n )
{
  return ((Uint16)(value>>((sizeof(Uint16)*8)-n)));   
} /* n_msb */
Uint16 n_lsb(Uint16 value,Uint32 n )
{
  return (n_msb(value<<(sizeof(Uint16)*8-n),n));   
} /* n_lsb */



Uint32 TurboInterleaverTableGenerator3GPP2(Uint16 *T, Uint32 K)
{
  Uint16 *addressGenerated;
  Uint16 counter,n_counter;
//  Uint16 mask;
  Uint16 n1, n2, n3, addr;
  Uint32  allAddressesWritten;
	Uint32 n;
	Uint32 i;

    
	/* 
	1. Determine the turbo Interleaver parameter, n, where n is the smallest integer such that Nturbo <= 2^(n+5) 
	*/  

	n=GenerateTurboInterleaverParameter(K);
	if (n<4)
		return 1;
	
  if ((addressGenerated=(Uint16 *)malloc(K*sizeof(Uint16)))== NULL)  
		return 1;

  /* Initialiaze Nturbo_array with 0xFFFF to verify the all addresses were written */
    for (i = 0; i < K; i++)
    {
      T[ i ] = 0xFFFF;
      addressGenerated[ i ] = 0xFFFF;
    }
	

	/* 
	2. Initialize an (n+5) - bit counter to 0;                     
	*/
  counter = 0; 
	n_counter=0;

  allAddressesWritten = 0;
  while(!allAddressesWritten)
  {
	  /* 
		3. Extract the most significant bits (MSBs) from the counter and add 
 	       one to form a new value. Then discard all except the n least 
 	       significant bits (LSBs) of this value.
		*/
		n1 = n_lsb( n_msb( n_counter, n) + 1, n ); 
 	  
		/* 
		4. Obtain the n-bit output of the table lookup define in 
 	       Table 2.1.3.1.4.2.3-2 with a read address equal to the 
 	       five LSBs of the counter. Note that this table depends on
 	       the value of n                                                 */ 
		n2 = InterleaverLookupTable(n_lsb(n_counter,5),n);
 	  
	  /* 
		5. Multiply the values obtained in Steps 3 and 4, and discard 
 	       all except the n LSBs.                                         
		*/ 
 		n2 = n_lsb((Uint16)(n1 * n2),n);
 	  
 	  
 	 /* 
		6. Bit Reverse the five LSBs of the counter.                      
		*/
 	 	n3 = BitReverse(n_counter, 5);
 	 
 	 /* 
		7. Form a tentative output address that has its MSBs equal 
 	       to the value obtained in Step 6 and its LSBs equal to the 
 	       value obtained in Step 5.                                      
		*/
 	 addr = (n3 << n) | n2;
 	   
    /* 
		8. Accept the tentative output address as an output address
          if it is less than Nturbo otherwise discard it                 
		*/
    if (addr < K)
    {
    	/* 
			9. Increment the counter and repeat Steps 3 through 8 until all   */
    	/*      Nturbo interleaver output addresses are obtained.            */
  
      /* Check to see if that address was already generated */
      if(addressGenerated[ addr ] == 0xFFFF)
      {
        /* Check if the address has not been written */
        if(T[ counter ] == 0xFFFF)
          T[ counter ] = addr;
        /* Interleaved Addres generated for a particular index */ 
        addressGenerated[ addr ] = 0x0000; 
        counter++;
      }
    }

    /* Exit if addresses were generated */  
		
    if (counter >= K) 
      allAddressesWritten = 1;
	

    /* increment temporary counter */
    n_counter++;
		
    
  }   
         
  free(addressGenerated);

	return 0;
}

unsigned char lRscGetFeedforwardBit(const unsigned char state, const unsigned short poly, const unsigned char bitVal, const unsigned int K)
{
	unsigned int i;
	unsigned int val=1<<(K-1);
	unsigned int temp1,temp2;
	unsigned int result=0;
	
	temp1=(poly&val);
	temp1>>= (K-1);
	temp1&=0x1;
	temp2=(bitVal<<(K-1))&val;
	temp2 >>= (K-1);
	temp2 &= 0x1;
	result=temp1&temp2;
	val>>=1;

	for (i=1;i<K;i++)
	{
		temp1 = (poly & val);
		temp1 >>= (K-1-i);
		temp1 &= 0x1;
		temp2 = (state & val);
		temp2 >>= (K-1-i);
		temp2 &= 0x1;
		result ^= temp1 & temp2;
		val>>=1;
	}
	return result;
}


//recursive, systematic, convolutional (RSC) encoder
int lrsc(const unsigned int *in, unsigned char *out, const unsigned int size, const unsigned short *feedforward, const unsigned short feedback)
{
	unsigned char state=0;
	unsigned int i,j,m,n;
	unsigned int K=0,temp;
	unsigned int rate;
	unsigned int nextBit;
	
	unsigned int nbFullWords;
	unsigned int nbRemainingBits;

	/* get inverse rate */
	i=0;
	while ((i<8) && feedforward[i])
	{
	 	i++;
	};
	rate=i+1;
	
	/* get K */
	for (i=0; i<rate-1; i++)
	{
		j=0;
		while (!(feedforward[i]&(temp=(0x8000>>j)))) 
		{
			j++;
		};
		
		temp=16-j;
		if (temp>K)
			K=temp;
	}
	
	nbFullWords=size/32;
	nbRemainingBits=size & 0x1F;

	n=0;
	for (i=0; i<nbFullWords; i++)
	{
		for (j=0; j<32; j++)
		{
			nextBit = ((in[i]>>j) & 0x1);

			out[n]=nextBit;
			temp=lRscGetFeedforwardBit(state,feedback,nextBit,K);
			nextBit ^= temp; /* feedback term */
		
			/* feedforward */
			for (m=0;m<rate-1;m++)
			{
				temp=lRscGetFeedforwardBit(state,feedforward[m],nextBit,K);
				out[n]|= (temp&0x1) << (m+1);
			}
			
#if VERBOSE_RSC
				printf("\nin=%d out=%d%d%d oldState=%01X",((in[i]>>j) & 0x1),\
														out[n]&0x1,\
														(out[n]>>1)&0x1,\
														(out[n]>>2)&0x1,\
														state);
#endif
			state>>=1;
			state|=nextBit<<(K-2);
#if VERBOSE_RSC
			printf(" newState=%01X",state);
#endif
			n++;
		}
	}
	
	for (j=0; j<nbRemainingBits; j++)
	{
		nextBit = ((in[i]>>j) & 0x1);
		out[n]=nextBit;
		temp=lRscGetFeedforwardBit(state,feedback,nextBit,K);
		nextBit ^= temp; /* feedback term */

		/* feedforward */
		for (m=0;m<rate-1;m++)
		{
			temp=lRscGetFeedforwardBit(state,feedforward[m],nextBit,K);
			out[n]|= (temp&0x1) << (m+1);
		}
	
#if VERBOSE_RSC
		printf("\nin=%d out=%d%d%d oldState=%01X",((in[i]>>j) & 0x1),\
													out[n]&0x1,\
													(out[n]>>1)&0x1,\
													(out[n]>>2)&0x1,\
													state);
#endif

		state>>=1;
		state|=nextBit<<(K-2);
#if VERBOSE_RSC
			printf(" newState=%01X",state);
#endif
		n++;
	}
	
	/* add tail bits */
	for (i=0; i<K-1; i++)
	{
		temp=lRscGetFeedforwardBit(state,feedback,nextBit,K);
		out[n]=temp;
		nextBit=0; /* feedback term */

		/* feedforward */
		for (m=0;m<rate-1;m++)
		{
			temp=lRscGetFeedforwardBit(state,feedforward[m],nextBit,K);
			out[n]|= (temp&0x1) << (m+1);
		}
	
#if VERBOSE_RSC
		printf("\nin=%d out=%d%d%d oldState=%01X",out[n]&0x1,\
													out[n]&0x1,\
													(out[n]>>1)&0x1,\
													(out[n]>>2)&0x1,\
													state);
#endif

		state>>=1;
		state|=nextBit<<(K-2);
#if VERBOSE_RSC
		printf(" newState=%01X",state);
#endif
		n++;
		
	}

	
	return 0;
}




